@@ -17,7 +17,7 @@ cd lbuild
     CFLAGS='-static -g1 -gz=zlib -no-pie -O2' \
     CXXFLAGS='-static -g1 -gz=zlib -U__unused -no-pie -O2' \
     LDFLAGS="-L/fake.root/lib" \
-    CPPFLAGS="-I/fake.root/include" \
+    CPPFLAGS="-I/fake.root/include -I/fake.root/include/ncursesw" \
     LIBS="-L/fake.root/lib -lssh2 -llzma -lssl -lcrypto -lz -llz4" \
     --enable-static \
     PATH="/fake.root/bin:${PATH}"
