@@ -38,8 +38,7 @@
 #include "base/opt_util.hh"
 #include "config.h"
 #include "pcrepp/pcre2pp.hh"
-#include "scn/scn.h"
-#include "view_curses.hh"
+#include "scn/scan.h"
 
 static const lnav::pcre2pp::code&
 ansi_regex()
@@ -53,7 +52,7 @@ ansi_regex()
 size_t
 erase_ansi_escapes(string_fragment input)
 {
-    static thread_local auto md = lnav::pcre2pp::match_data::unitialized();
+    thread_local auto md = lnav::pcre2pp::match_data::unitialized();
 
     const auto& regex = ansi_regex();
     std::optional<int> move_start;
@@ -121,8 +120,8 @@ erase_ansi_escapes(string_fragment input)
 void
 scrub_ansi_string(std::string& str, string_attrs_t* sa)
 {
-    static thread_local auto md = lnav::pcre2pp::match_data::unitialized();
-    static const auto semi_pred = string_fragment::tag1{';'};
+    thread_local auto md = lnav::pcre2pp::match_data::unitialized();
+    static constexpr auto semi_pred = string_fragment::tag1{';'};
 
     const auto& regex = ansi_regex();
     std::optional<std::string> href;
@@ -186,8 +185,9 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
                     if (sa != nullptr && bold_range.is_valid()) {
                         shift_string_attrs(
                             *sa, bold_range.lr_start, -bold_range.length() * 2);
-                        tmp_sa.emplace_back(bold_range,
-                                            VC_STYLE.value(text_attrs{A_BOLD}));
+                        tmp_sa.emplace_back(
+                            bold_range,
+                            VC_STYLE.value(text_attrs::with_bold()));
                         bold_range.clear();
                     }
                     if (ul_range.is_valid()) {
@@ -208,7 +208,8 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
                         shift_string_attrs(
                             *sa, ul_range.lr_start, -ul_range.length() * 2);
                         tmp_sa.emplace_back(
-                            ul_range, VC_STYLE.value(text_attrs{A_UNDERLINE}));
+                            ul_range,
+                            VC_STYLE.value(text_attrs::with_underline()));
                         ul_range.clear();
                     }
                     if (bold_range.is_valid()) {
@@ -236,15 +237,15 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
             if (sa != nullptr && ul_range.is_valid()) {
                 shift_string_attrs(
                     *sa, ul_range.lr_start, -ul_range.length() * 2);
-                tmp_sa.emplace_back(ul_range,
-                                    VC_STYLE.value(text_attrs{A_UNDERLINE}));
+                tmp_sa.emplace_back(
+                    ul_range, VC_STYLE.value(text_attrs::with_underline()));
                 ul_range.clear();
             }
             if (sa != nullptr && bold_range.is_valid()) {
                 shift_string_attrs(
                     *sa, bold_range.lr_start, -bold_range.length() * 2);
                 tmp_sa.emplace_back(bold_range,
-                                    VC_STYLE.value(text_attrs{A_BOLD}));
+                                    VC_STYLE.value(text_attrs::with_bold()));
                 bold_range.clear();
             }
             if (sa != nullptr && output_size > 0 && cp_dst > 0) {
@@ -271,7 +272,7 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
             auto osc_id = scn::scan_value<int32_t>(md[3]->to_string_view());
 
             if (osc_id) {
-                switch (osc_id.value()) {
+                switch (osc_id->value()) {
                     case 8:
                         auto split_res = md[4]->split_pair(semi_pred);
                         if (split_res) {
@@ -305,21 +306,23 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
                 case 'm':
                     while (!seq.empty()) {
                         auto ansi_code_res
-                            = scn::scan_value<int>(seq.to_string_view());
+                            = scn::scan_value<uint8_t>(seq.to_string_view());
 
                         if (!ansi_code_res) {
                             break;
                         }
-                        auto ansi_code = ansi_code_res.value();
+                        auto ansi_code = ansi_code_res->value();
                         if (90 <= ansi_code && ansi_code <= 97) {
                             ansi_code -= 60;
-                            attrs.ta_attrs |= A_STANDOUT;
+                            // XXX attrs.ta_attrs |= A_STANDOUT;
                         }
                         if (30 <= ansi_code && ansi_code <= 37) {
-                            attrs.ta_fg_color = ansi_code - 30;
+                            attrs.ta_fg_color = palette_color{
+                                static_cast<uint8_t>(ansi_code - 30)};
                         }
                         if (40 <= ansi_code && ansi_code <= 47) {
-                            attrs.ta_bg_color = ansi_code - 40;
+                            attrs.ta_bg_color = palette_color{
+                                static_cast<uint8_t>(ansi_code - 40)};
                         }
                         if (ansi_code == 38 || ansi_code == 48) {
                             auto color_code_pair
@@ -333,42 +336,57 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
                             if (!color_type.has_value()) {
                                 break;
                             }
-                            if (color_type.value() == 2) {
-                            } else if (color_type.value() == 5) {
+                            if (color_type->value() == 2) {
+                                auto scan_res
+                                    = scn::scan<uint8_t, uint8_t, uint8_t>(
+                                        color_code_pair->second
+                                            .to_string_view(),
+                                        "{};{};{}");
+                                if (scan_res) {
+                                    auto [r, g, b] = scan_res->values();
+                                    attrs.ta_fg_color = rgb_color{r, g, b};
+                                }
+                            } else if (color_type->value() == 5) {
                                 auto color_index_pair
                                     = color_code_pair->second.split_when(
                                         semi_pred);
                                 auto color_index = scn::scan_value<short>(
                                     color_index_pair.first.to_string_view());
                                 if (!color_index.has_value()
-                                    || color_index.value() < 0
-                                    || color_index.value() > 255)
+                                    || color_index->value() < 0
+                                    || color_index->value() > 255)
                                 {
                                     break;
                                 }
                                 if (ansi_code == 38) {
-                                    attrs.ta_fg_color = color_index.value();
+                                    attrs.ta_fg_color = palette_color{
+                                        (uint8_t) color_index->value()};
                                 } else {
-                                    attrs.ta_bg_color = color_index.value();
+                                    attrs.ta_bg_color = palette_color{
+                                        (uint8_t) color_index->value()};
                                 }
                                 seq = color_index_pair.second;
                             }
                         }
                         switch (ansi_code) {
                             case 1:
-                                attrs.ta_attrs |= A_BOLD;
+                                attrs |= text_attrs::style::bold;
                                 break;
 
                             case 2:
-                                attrs.ta_attrs |= A_DIM;
+                                // XXX attrs.ta_attrs |= A_DIM;
+                                break;
+
+                            case 3:
+                                attrs |= text_attrs::style::italic;
                                 break;
 
                             case 4:
-                                attrs.ta_attrs |= A_UNDERLINE;
+                                attrs |= text_attrs::style::underline;
                                 break;
 
                             case 7:
-                                attrs.ta_attrs |= A_REVERSE;
+                                attrs |= text_attrs::style::reverse;
                                 break;
                         }
                         auto split_pair = seq.split_when(semi_pred);
@@ -411,7 +429,7 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
                     auto role_res = scn::scan_value<int>(seq.to_string_view());
 
                     if (role_res) {
-                        role_t role_tmp = (role_t) role_res.value();
+                        role_t role_tmp = (role_t) role_res->value();
                         if (role_tmp > role_t::VCR_NONE
                             && role_tmp < role_t::VCR__MAX)
                         {
@@ -478,16 +496,17 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
 void
 add_ansi_vars(std::map<std::string, scoped_value_t>& vars)
 {
-    vars["ansi_csi"] = ANSI_CSI;
-    vars["ansi_norm"] = ANSI_NORM;
-    vars["ansi_bold"] = ANSI_BOLD_START;
-    vars["ansi_underline"] = ANSI_UNDERLINE_START;
-    vars["ansi_black"] = ANSI_COLOR(COLOR_BLACK);
-    vars["ansi_red"] = ANSI_COLOR(COLOR_RED);
-    vars["ansi_green"] = ANSI_COLOR(COLOR_GREEN);
-    vars["ansi_yellow"] = ANSI_COLOR(COLOR_YELLOW);
-    vars["ansi_blue"] = ANSI_COLOR(COLOR_BLUE);
-    vars["ansi_magenta"] = ANSI_COLOR(COLOR_MAGENTA);
-    vars["ansi_cyan"] = ANSI_COLOR(COLOR_CYAN);
-    vars["ansi_white"] = ANSI_COLOR(COLOR_WHITE);
+    vars["ansi_csi"] = string_fragment::from_const(ANSI_CSI);
+    vars["ansi_norm"] = string_fragment::from_const(ANSI_NORM);
+    vars["ansi_bold"] = string_fragment::from_const(ANSI_BOLD_START);
+    vars["ansi_underline"] = string_fragment::from_const(ANSI_UNDERLINE_START);
+    vars["ansi_black"] = string_fragment::from_const(ANSI_COLOR(COLOR_BLACK));
+    vars["ansi_red"] = string_fragment::from_const(ANSI_COLOR(COLOR_RED));
+    vars["ansi_green"] = string_fragment::from_const(ANSI_COLOR(COLOR_GREEN));
+    vars["ansi_yellow"] = string_fragment::from_const(ANSI_COLOR(COLOR_YELLOW));
+    vars["ansi_blue"] = string_fragment::from_const(ANSI_COLOR(COLOR_BLUE));
+    vars["ansi_magenta"]
+        = string_fragment::from_const(ANSI_COLOR(COLOR_MAGENTA));
+    vars["ansi_cyan"] = string_fragment::from_const(ANSI_COLOR(COLOR_CYAN));
+    vars["ansi_white"] = string_fragment::from_const(ANSI_COLOR(COLOR_WHITE));
 }
