@@ -28,6 +28,7 @@
  */
 
 #include <algorithm>
+#include <string_view>
 
 #include "attr_line.hh"
 
@@ -36,9 +37,138 @@
 #include "ansi_scrubber.hh"
 #include "auto_mem.hh"
 #include "config.h"
+#include "intervaltree/IntervalTree.h"
 #include "lnav_log.hh"
 #include "pcrepp/pcre2pp.hh"
 
+using namespace std::string_view_literals;
+
+attr_line_t
+attr_line_t::from_table_cell_content(const string_fragment& content,
+                                     size_t max_char_width)
+{
+    static constexpr auto TAB_SYMBOL = "\u21e5"sv;
+    static constexpr auto LF_SYMBOL = "\u240a"sv;
+    static constexpr auto CR_SYMBOL = "\u240d"sv;
+    static constexpr auto REP_SYMBOL = "\ufffd"sv;
+    static const std::string ELLIPSIS = "\u22ef";
+
+    auto has_ansi = false;
+    size_t char_width = 0;
+    attr_line_t retval;
+    std::string_view replacement;
+    size_t copy_start = 0;
+
+    retval.al_string.reserve(max_char_width);
+    for (size_t index = 0; index < content.length(); ++index) {
+        const auto ch = content.udata()[index];
+
+        switch (ch) {
+            case '\t':
+                replacement = TAB_SYMBOL;
+                char_width += 1;
+                break;
+            case '\n':
+                replacement = LF_SYMBOL;
+                char_width += 1;
+                break;
+            case '\r':
+                replacement = CR_SYMBOL;
+                char_width += 1;
+                break;
+            case '\b':
+            case '\x1b':
+                has_ansi = true;
+                break;
+            default:
+                if (ch < 0x80) {
+                    char_width += 1;
+                } else if (ch < 0xc0) {
+                    replacement = REP_SYMBOL;
+                    char_width += 1;
+                } else if (ch < 0xe0) {
+                    auto next_ch = content[index + 1];
+                    if (next_ch != 0) {
+                        index += 1;
+                    } else {
+                        replacement = REP_SYMBOL;
+                    }
+                    char_width += 1;
+                } else if (ch < 0xf0) {
+                    if (content[index + 1] != 0 && content[index + 2] != 0) {
+                        index += 2;
+                    } else {
+                        replacement = REP_SYMBOL;
+                    }
+                    char_width += 1;
+                } else if (ch < 0xf8) {
+                    if (content[index + 1] != 0 && content[index + 2] != 0
+                        && content[index + 3] != 0)
+                    {
+                        index += 3;
+                    } else {
+                        replacement = REP_SYMBOL;
+                    }
+                    char_width += 1;
+                } else if (ch < 0xfc) {
+                    if (content[index + 1] != 0 && content[index + 2] != 0
+                        && content[index + 3] != 0 && content[index + 4] != 0)
+                    {
+                        index += 4;
+                    } else {
+                        replacement = REP_SYMBOL;
+                    }
+                    char_width += 1;
+                } else if (ch < 0xfe) {
+                    if (content[index + 1] != 0 && content[index + 2] != 0
+                        && content[index + 3] != 0 && content[index + 4] != 0
+                        && content[index + 5] != 0)
+                    {
+                        index += 5;
+                    } else {
+                        replacement = REP_SYMBOL;
+                    }
+                    char_width += 1;
+                } else {
+                    replacement = REP_SYMBOL;
+                    char_width += 1;
+                }
+                break;
+        }
+
+        if (!replacement.empty()) {
+            auto copy_len = index - copy_start;
+            retval.al_string.append(&content[copy_start], copy_len);
+            copy_start = index + 1;
+            retval.al_string.append(replacement);
+            replacement = ""sv;
+        }
+    }
+    if (copy_start < content.length()) {
+        auto copy_len = content.length() - copy_start;
+        retval.al_string.append(&content[copy_start], copy_len);
+    }
+
+    if (has_ansi) {
+        scrub_ansi_string(retval.al_string, &retval.al_attrs);
+    }
+
+    if (char_width > max_char_width) {
+        auto chars_to_remove = (char_width - max_char_width) + 1;
+        auto midpoint = char_width / 2;
+        auto chars_to_keep_at_front = midpoint - (chars_to_remove / 2);
+        auto bytes_to_keep_at_front
+            = utf8_char_to_byte_index(retval.al_string, chars_to_keep_at_front);
+        auto remove_up_to_bytes = utf8_char_to_byte_index(
+            retval.al_string, chars_to_keep_at_front + chars_to_remove);
+        auto bytes_to_remove = remove_up_to_bytes - bytes_to_keep_at_front;
+        retval.erase(bytes_to_keep_at_front, bytes_to_remove);
+        retval.insert(bytes_to_keep_at_front, ELLIPSIS);
+    }
+
+    return retval;
+}
+
 attr_line_t&
 attr_line_t::with_ansi_string(const char* str, ...)
 {
@@ -66,6 +196,15 @@ attr_line_t::with_ansi_string(const std::string& str)
     return *this;
 }
 
+attr_line_t&
+attr_line_t::with_ansi_string(const string_fragment& str)
+{
+    this->al_string = str.to_string();
+    scrub_ansi_string(this->al_string, &this->al_attrs);
+
+    return *this;
+}
+
 namespace text_stream {
 struct word {
     string_fragment w_word;
@@ -399,13 +538,43 @@ attr_line_t::subline(size_t start, size_t len) const
 void
 attr_line_t::split_lines(std::vector<attr_line_t>& lines) const
 {
+    using line_type_t = interval_tree::Interval<size_t, attr_line_t>;
+    using lines_tree_t = interval_tree::IntervalTree<size_t, attr_line_t>;
+
+    auto eols = std::vector<line_type_t>{};
     size_t pos = 0, next_line;
 
     while ((next_line = this->al_string.find('\n', pos)) != std::string::npos) {
-        lines.emplace_back(this->subline(pos, next_line - pos));
+        eols.emplace_back(
+            pos,
+            next_line > pos ? next_line - 1 : next_line,
+            attr_line_t{this->al_string.substr(pos, next_line - pos)});
         pos = next_line + 1;
     }
-    lines.emplace_back(this->subline(pos));
+    if (pos < this->al_string.length()) {
+        eols.emplace_back(pos,
+                          this->al_string.length() - 1,
+                          attr_line_t{this->al_string.substr(pos)});
+    }
+    auto lines_tree = lines_tree_t{std::move(eols)};
+    for (const auto& sa : this->al_attrs) {
+        if (sa.sa_range.empty()) {
+            continue;
+        }
+        auto range_end = sa.sa_range.end_for_string(this->al_string) - 1;
+        lines_tree.visit_overlapping(
+            sa.sa_range.lr_start, range_end, [&sa](const auto& cinterval) {
+                auto& interval = const_cast<line_type_t&>(cinterval);
+                auto lr = line_range(interval.start, interval.stop + 1);
+                auto ilr = lr.intersection(sa.sa_range).shift(0, -lr.lr_start);
+                interval.value.al_attrs.emplace_back(
+                    ilr, std::make_pair(sa.sa_type, sa.sa_value));
+            });
+    }
+    lines_tree.visit_all([&lines](auto& cinterval) {
+        auto& interval = const_cast<line_type_t&>(cinterval);
+        lines.emplace_back(std::move(interval.value));
+    });
 }
 
 attr_line_t&
@@ -451,7 +620,7 @@ attr_line_t::apply_hide()
     auto& sa = this->al_attrs;
 
     for (auto& sattr : sa) {
-        if (sattr.sa_type == &SA_HIDDEN && sattr.sa_range.length() > 3) {
+        if (sattr.sa_type == &SA_HIDDEN && sattr.sa_range.length() > 1) {
             auto& lr = sattr.sa_range;
 
             std::for_each(sa.begin(), sa.end(), [&](string_attr& attr) {
@@ -538,6 +707,13 @@ attr_line_t::pad_to(ssize_t size)
     return *this;
 }
 
+size_t
+attr_line_t::column_to_byte_index(size_t column) const
+{
+    return string_fragment::from_str(this->al_string)
+        .column_to_byte_index(column);
+}
+
 line_range
 line_range::intersection(const line_range& other) const
 {
@@ -651,7 +827,8 @@ shift_string_attrs(string_attrs_t& sa, const line_range& cover, int32_t amount)
 }
 
 struct line_range
-find_string_attr_range(const string_attrs_t& sa, string_attr_type_base* type)
+find_string_attr_range(const string_attrs_t& sa,
+                       const string_attr_type_base* type)
 {
     auto iter = find_string_attr(sa, type);
 
@@ -673,7 +850,7 @@ remove_string_attr(string_attrs_t& sa, const line_range& lr)
 }
 
 void
-remove_string_attr(string_attrs_t& sa, string_attr_type_base* type)
+remove_string_attr(string_attrs_t& sa, const string_attr_type_base* type)
 {
     for (auto iter = sa.begin(); iter != sa.end();) {
         if (iter->sa_type == type) {
