@@ -65,30 +65,10 @@
 #include <sys/utsname.h>
 #include <sys/wait.h>
 
-#if defined HAVE_NCURSESW_CURSES_H
-#    include <ncursesw/curses.h>
-#    include <ncursesw/termcap.h>
-#elif defined HAVE_NCURSESW_H
-#    include <ncursesw.h>
-#    include <termcap.h>
-#elif defined HAVE_NCURSES_CURSES_H
-#    include <ncurses/curses.h>
-#    include <ncurses/termcap.h>
-#elif defined HAVE_NCURSES_H
-#    include <ncurses.h>
-#    include <termcap.h>
-#elif defined HAVE_CURSESW_H
-#    include <cursesw.h>
-#elif defined HAVE_CURSES_H
-#    include <curses.h>
-#    include <termcap.h>
-#else
-#    error "SysV or X/Open-compatible Curses header file required"
-#endif
-
 #include "ansi_scrubber.hh"
 #include "auto_mem.hh"
 #include "enum_util.hh"
+#include "fs_util.hh"
 #include "lnav_log.hh"
 #include "opt_util.hh"
 
@@ -145,7 +125,7 @@ static struct {
     char lr_data[BUFFER_SIZE];
 } log_ring = {0, BUFFER_SIZE, 0, {}};
 
-static const char* LEVEL_NAMES[] = {
+static constexpr const char* const LEVEL_NAMES[] = {
     "T",
     "D",
     "I",
@@ -248,6 +228,7 @@ log_host_info()
     }
     log_info("Executable:");
     log_info("  version=%s", VCS_PACKAGE_STRING);
+    log_info("  mtime=%lld", lnav::filesystem::self_mtime());
 
     getrusage(RUSAGE_SELF, &ru);
     log_rusage(lnav_log_level_t::INFO, ru);
