@@ -31,6 +31,7 @@
 
 #include "base/itertools.enumerate.hh"
 #include "base/itertools.hh"
+#include "base/keycodes.hh"
 #include "itertools.similar.hh"
 
 using namespace lnav::roles::literals;
@@ -40,8 +41,7 @@ breadcrumb_curses::no_op_action(breadcrumb_curses&)
 {
 }
 
-breadcrumb_curses::
-breadcrumb_curses()
+breadcrumb_curses::breadcrumb_curses()
 {
     this->bc_match_search_overlay.sos_parent = this;
     this->bc_match_source.set_reverse_selection(true);
@@ -63,7 +63,7 @@ breadcrumb_curses::do_update()
     }
 
     size_t sel_crumb_offset = 0;
-    auto width = static_cast<size_t>(getmaxx(this->bc_window));
+    auto width = ncplane_dim_x(this->bc_window);
     auto crumbs = this->bc_focused_crumbs.empty() ? this->bc_line_source()
                                                   : this->bc_focused_crumbs;
     if (this->bc_last_selected_crumb
@@ -93,7 +93,7 @@ breadcrumb_curses::do_update()
         if (is_selected) {
             sel_crumb_offset = accum_width;
             crumbs_line.get_attrs().emplace_back(
-                crumb_range, VC_STYLE.value(text_attrs{A_REVERSE}));
+                crumb_range, VC_STYLE.value(text_attrs::with_reverse()));
         }
 
         this->bc_displayed_crumbs.emplace_back(
@@ -222,11 +222,27 @@ breadcrumb_curses::blur()
 }
 
 bool
-breadcrumb_curses::handle_key(int ch)
+breadcrumb_curses::handle_key(const ncinput& ch)
 {
     bool retval = false;
+    auto mapped_id = ch.id;
 
-    switch (ch) {
+    if (mapped_id == NCKEY_TAB && ncinput_shift_p(&ch)) {
+        mapped_id = NCKEY_LEFT;
+    } else if (ncinput_ctrl_p(&ch)) {
+        switch (mapped_id) {
+            case 'a':
+            case 'A':
+                mapped_id = KEY_CTRL('a');
+                break;
+                break;
+            case 'e':
+            case 'E':
+                mapped_id = KEY_CTRL('e');
+                break;
+        }
+    }
+    switch (mapped_id) {
         case KEY_CTRL('a'):
             if (this->bc_selected_crumb) {
                 this->bc_selected_crumb = 0;
@@ -243,8 +259,7 @@ breadcrumb_curses::handle_key(int ch)
             }
             retval = true;
             break;
-        case KEY_BTAB:
-        case KEY_LEFT:
+        case NCKEY_LEFT:
             if (this->bc_selected_crumb) {
                 if (this->bc_selected_crumb.value() > 0) {
                     this->bc_selected_crumb
@@ -258,8 +273,8 @@ breadcrumb_curses::handle_key(int ch)
             }
             retval = true;
             break;
-        case '\t':
-        case KEY_RIGHT:
+        case NCKEY_TAB:
+        case NCKEY_RIGHT:
             if (this->bc_selected_crumb) {
                 this->perform_selection(perform_behavior_t::if_different);
                 this->blur();
@@ -278,52 +293,52 @@ breadcrumb_curses::handle_key(int ch)
                 retval = true;
             }
             break;
-        case KEY_HOME:
+        case NCKEY_HOME:
             this->bc_match_view.set_selection(0_vl);
             retval = true;
             break;
-        case KEY_END:
+        case NCKEY_END:
             this->bc_match_view.set_selection(
                 this->bc_match_view.get_inner_height() - 1_vl);
             retval = true;
             break;
-        case KEY_NPAGE:
+        case NCKEY_PGDOWN:
             this->bc_match_view.shift_selection(
                 listview_curses::shift_amount_t::down_page);
             retval = true;
             break;
-        case KEY_PPAGE:
+        case NCKEY_PGUP:
             this->bc_match_view.shift_selection(
                 listview_curses::shift_amount_t::up_page);
             retval = true;
             break;
-        case KEY_UP:
+        case NCKEY_UP:
             this->bc_match_view.shift_selection(
                 listview_curses::shift_amount_t::up_line);
             retval = true;
             break;
-        case KEY_DOWN:
+        case NCKEY_DOWN:
             this->bc_match_view.shift_selection(
                 listview_curses::shift_amount_t::down_line);
             retval = true;
             break;
         case KEY_DELETE:
-        case KEY_BACKSPACE:
+        case NCKEY_BACKSPACE:
             if (!this->bc_current_search.empty()) {
                 this->bc_current_search.pop_back();
                 this->reload_data();
             }
             retval = true;
             break;
-        case KEY_ENTER:
+        case NCKEY_ENTER:
         case '\r':
             this->perform_selection(perform_behavior_t::always);
             break;
         case KEY_ESCAPE:
             break;
         default:
-            if (isprint(ch)) {
-                this->bc_current_search.push_back(ch);
+            if (isprint(ch.id)) {
+                this->bc_current_search.push_back(ch.id);
                 this->reload_data();
                 retval = true;
             }
@@ -401,7 +416,7 @@ breadcrumb_curses::search_overlay_source::list_static_overlay(
         | lnav::itertools::unwrap_or(
                          breadcrumb::crumb::expected_input_t::exact);
 
-    value_out.with_attr_for_all(VC_STYLE.value(text_attrs{A_UNDERLINE}));
+    value_out.with_attr_for_all(VC_STYLE.value(text_attrs::with_underline()));
 
     if (!parent->bc_current_search.empty()) {
         value_out = parent->bc_current_search;
@@ -480,6 +495,7 @@ breadcrumb_curses::handle_mouse(mouse_event& me)
         this->focus();
         this->on_focus(*this);
         this->do_update();
+        this->bc_initial_mouse_event = true;
     }
 
     auto find_res = this->bc_displayed_crumbs
@@ -503,7 +519,6 @@ breadcrumb_curses::handle_mouse(mouse_event& me)
                     this->blur();
                     this->reload_data();
                     this->on_blur(*this);
-                    this->bc_initial_mouse_event = true;
                 }
                 return true;
             }
@@ -516,7 +531,6 @@ breadcrumb_curses::handle_mouse(mouse_event& me)
             this->blur();
             this->reload_data();
             this->on_blur(*this);
-            this->bc_initial_mouse_event = true;
             return true;
         }
     }
