@@ -27,6 +27,8 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <stack>
+
 #include "data_parser.hh"
 
 #include "config.h"
@@ -37,8 +39,7 @@ data_format data_parser::FORMAT_COMMA("comma", DT_INVALID, DT_COMMA);
 data_format data_parser::FORMAT_EMDASH("emdash", DT_INVALID, DT_EMDASH);
 data_format data_parser::FORMAT_PLAIN("plain", DT_INVALID, DT_INVALID);
 
-data_parser::
-data_parser(data_scanner* ds)
+data_parser::data_parser(data_scanner* ds)
     : dp_errors("dp_errors", __FILE__, __LINE__),
       dp_pairs("dp_pairs", __FILE__, __LINE__), dp_msg_format(nullptr),
       dp_msg_format_begin(ds->get_init_offset()), dp_scanner(ds)
@@ -245,8 +246,7 @@ data_parser::pairup(data_parser::schema_id_t* schema,
                 } while (key_iter != key_comps.begin() && !found);
             }
             if (!mixed_queue.empty()) {
-                if (!el_stack.empty() &&
-                    el_stack.back().e_token == DNT_KEY
+                if (!el_stack.empty() && el_stack.back().e_token == DNT_KEY
                     && mixed_queue.front().e_token == DNT_KEY)
                 {
                     el_stack.POP_BACK();
@@ -819,8 +819,7 @@ data_parser::end_of_value(data_parser::element_list_t& el_stack,
         } while (key_iter != key_comps.begin() && !found);
     }
     if (!mixed_queue.empty()) {
-        if (!el_stack.empty() &&
-            el_stack.back().e_token == DNT_KEY
+        if (!el_stack.empty() && el_stack.back().e_token == DNT_KEY
             && mixed_queue.front().e_token == DNT_KEY)
         {
             el_stack.POP_BACK();
@@ -856,8 +855,7 @@ data_parser::end_of_value(data_parser::element_list_t& el_stack,
             && (value.front().e_token == DT_WORD
                 || value.front().e_token == DT_SYMBOL
                 || value.front().e_token == DT_ID)
-            && !el_stack.empty()
-            && el_stack.back().e_token != DNT_KEY)
+            && !el_stack.empty() && el_stack.back().e_token != DNT_KEY)
         {
             element_list_t ELEMENT_LIST_T(mixed_key);
 
@@ -1169,16 +1167,14 @@ dfs_comma_next(data_format_state_t state, data_token_t next_token)
     return retval;
 }
 
-data_parser::element::
-element()
+data_parser::element::element()
     : e_capture(-1, -1), e_token(DT_INVALID), e_sub_elements(nullptr)
 {
 }
 
-data_parser::element::
-element(data_parser::element_list_t& subs,
-        data_token_t token,
-        bool assign_subs_elements)
+data_parser::element::element(data_parser::element_list_t& subs,
+                              data_token_t token,
+                              bool assign_subs_elements)
     : e_capture(subs.front().e_capture.c_begin, subs.back().e_capture.c_end),
       e_token(token), e_sub_elements(nullptr)
 {
@@ -1187,8 +1183,7 @@ element(data_parser::element_list_t& subs,
     }
 }
 
-data_parser::element::
-element(const data_parser::element& other)
+data_parser::element::element(const data_parser::element& other)
 {
     /* require(other.e_sub_elements == nullptr); */
 
@@ -1200,8 +1195,7 @@ element(const data_parser::element& other)
     }
 }
 
-data_parser::element::~
-element()
+data_parser::element::~element()
 {
     delete this->e_sub_elements;
     this->e_sub_elements = nullptr;
@@ -1355,8 +1349,7 @@ data_parser::element::is_value() const
     }
 }
 
-data_parser::discover_format_state::
-discover_format_state()
+data_parser::discover_format_state::discover_format_state()
     : dfs_prefix_state(DFS_INIT), dfs_semi_state(DFS_INIT),
       dfs_comma_state(DFS_INIT)
 {
