@@ -39,6 +39,7 @@
 #include "md2attr_line.hh"
 #include "ptimec.hh"
 #include "readline_highlighters.hh"
+#include "sql_util.hh"
 #include "vtab_module.hh"
 #include "vtab_module_json.hh"
 
@@ -95,13 +96,14 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             auto emsg = fmt::format(
                 FMT_STRING("   Invalid log message: {}"),
                 sattr.sa_value.get<decltype(SA_INVALID)::value_type>());
-            auto al = attr_line_t(emsg)
-                          .with_attr(string_attr(
-                              line_range{1, 2}, VC_GRAPHIC.value(ACS_LLCORNER)))
-                          .with_attr(string_attr(
-                              line_range{0, 22},
-                              VC_ROLE.value(role_t::VCR_INVALID_MSG)))
-                          .move();
+            auto al
+                = attr_line_t(emsg)
+                      .with_attr(string_attr(line_range{1, 2},
+                                             VC_GRAPHIC.value(NCACS_LLCORNER)))
+                      .with_attr(
+                          string_attr(line_range{0, 22},
+                                      VC_ROLE.value(role_t::VCR_INVALID_MSG)))
+                      .move();
             this->fos_lines.emplace_back(al);
         }
     }
@@ -111,18 +113,15 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
     attr_line_t time_line;
     auto& time_str = time_line.get_string();
     struct line_range time_lr;
-    off_t ts_len = sql_strftime(curr_timestamp,
-                                sizeof(curr_timestamp),
-                                ll->get_time(),
-                                ll->get_millis(),
-                                'T');
+    off_t ts_len = sql_strftime(
+        curr_timestamp, sizeof(curr_timestamp), ll->get_timeval(), 'T');
     {
         exttm tmptm;
 
         tmptm.et_flags |= ETF_ZONE_SET;
         tmptm.et_gmtoff
             = lnav::local_time_to_info(
-                  date::local_seconds{std::chrono::seconds{ll->get_time()}})
+                  date::local_seconds{ll->get_time<std::chrono::seconds>()})
                   .first.offset.count();
         ftime_z(curr_timestamp, ts_len, sizeof(curr_timestamp), tmptm);
         curr_timestamp[ts_len] = '\0';
@@ -132,7 +131,7 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         time_lr.lr_start = 1;
         time_lr.lr_end = 2;
         time_line.with_attr(
-            string_attr(time_lr, VC_GRAPHIC.value(ACS_LLCORNER)));
+            string_attr(time_lr, VC_GRAPHIC.value(NCACS_LLCORNER)));
         time_str.append("   Out-Of-Time-Order Message");
         time_lr.lr_start = 3;
         time_lr.lr_end = time_str.length();
@@ -146,7 +145,7 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
     time_str.append(curr_timestamp);
     time_lr.lr_end = time_str.length();
     time_line.with_attr(
-        string_attr(time_lr, VC_STYLE.value(text_attrs{A_BOLD})));
+        string_attr(time_lr, VC_STYLE.value(text_attrs::with_bold())));
     time_str.append(" \u2014 ");
     time_lr.lr_start = time_str.length();
     time_str.append(humanize::time::point::from_tv(ll->get_timeval())
@@ -154,10 +153,10 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
                         .as_precise_time_ago());
     time_lr.lr_end = time_str.length();
     time_line.with_attr(
-        string_attr(time_lr, VC_STYLE.value(text_attrs{A_BOLD})));
+        string_attr(time_lr, VC_STYLE.value(text_attrs::with_bold())));
 
-    struct line_range time_range = find_string_attr_range(
-        this->fos_log_helper.ldh_line_attrs, &logline::L_TIMESTAMP);
+    auto time_range = find_string_attr_range(
+        this->fos_log_helper.ldh_line_attrs, &L_TIMESTAMP);
 
     curr_tv = this->fos_log_helper.ldh_line->get_timeval();
     if (ll->is_time_skewed() && time_range.lr_end != -1) {
@@ -194,7 +193,7 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
                 humanize::time::duration::from_tv(diff_tv).to_string());
             time_lr.lr_end = time_str.length();
             time_line.with_attr(
-                string_attr(time_lr, VC_STYLE.value(text_attrs{A_BOLD})));
+                string_attr(time_lr, VC_STYLE.value(text_attrs::with_bold())));
         }
     }
 
@@ -326,7 +325,7 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         auto* curr_elf = dynamic_cast<external_log_format*>(curr_format);
         const auto format_name = curr_format->get_name().to_string();
         attr_line_t al;
-        std::string value_str = lv.to_string();
+        auto value_str = lv.to_string();
 
         if (curr_format != last_format) {
             this->fos_lines.emplace_back(" Known message fields for table "
@@ -334,13 +333,13 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             this->fos_lines.back().with_attr(
                 string_attr(line_range(32, 32 + format_name.length()),
                             VC_STYLE.value(vc.attrs_for_ident(format_name)
-                                           | text_attrs{A_BOLD})));
+                                           | text_attrs::style::bold)));
             last_format = curr_format;
         }
 
         std::string field_name, orig_field_name;
         line_range hl_range;
-        al.append(" ").append("|", VC_GRAPHIC.value(ACS_LTEE)).append(" ");
+        al.append(" ").append("|", VC_GRAPHIC.value(NCACS_LTEE)).append(" ");
         if (meta.lvm_struct_name.empty()) {
             if (curr_elf && curr_elf->elf_body_field == meta.lvm_name) {
                 field_name = LOG_BODY;
@@ -377,7 +376,8 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             hl_range.lr_end = al.get_string().length();
             al.pad_to(prefix_len + this->fos_known_key_size);
 
-            this->fos_row_to_field_meta.emplace(this->fos_lines.size(), meta);
+            this->fos_row_to_field_meta.emplace(this->fos_lines.size(),
+                                                row_info{meta, value_str});
         } else {
             auto jget_str = lnav::sql::mprintf("jget(%s, '/%q')",
                                                meta.lvm_struct_name.get(),
@@ -385,6 +385,9 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             hl_range.lr_start = al.get_string().length();
             al.append(jget_str.in());
             hl_range.lr_end = al.get_string().length();
+
+            this->fos_row_to_field_meta.emplace(
+                this->fos_lines.size(), row_info{std::nullopt, value_str});
         }
         readline_sqlite_highlighter_int(al, std::nullopt, hl_range);
 
@@ -425,6 +428,11 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         readline_sqlite_highlighter(key_line, std::nullopt);
         auto key_size = key_line.length();
         key_line.append(" = ").append(scrub_ws(extra_pair.second));
+        this->fos_row_to_field_meta.emplace(this->fos_lines.size(),
+                                            row_info{
+                                                std::nullopt,
+                                                extra_pair.second,
+                                            });
         this->fos_lines.emplace_back(key_line);
         this->add_key_line_attrs(key_size - 3);
     }
@@ -440,6 +448,9 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             readline_sqlite_highlighter(key_line, std::nullopt);
             auto key_size = key_line.length();
             key_line.append(" = ").append(scrub_ws(jpairs[lpc].wt_value));
+            this->fos_row_to_field_meta.emplace(
+                this->fos_lines.size(),
+                row_info{std::nullopt, jpairs[lpc].wt_value});
             this->fos_lines.emplace_back(key_line);
             this->add_key_line_attrs(key_size - 3);
         }
@@ -460,6 +471,8 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         readline_sqlite_highlighter(key_line, std::nullopt);
         auto key_size = key_line.length();
         key_line.append(" = ").append(scrub_ws(xml_pair.second));
+        this->fos_row_to_field_meta.emplace(
+            this->fos_lines.size(), row_info{std::nullopt, xml_pair.second});
         this->fos_lines.emplace_back(key_line);
         this->add_key_line_attrs(key_size - 3);
     }
@@ -477,7 +490,7 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         auto& disc_str = al.get_string();
 
         al.with_attr(string_attr(line_range(disc_str.length(), -1),
-                                 VC_STYLE.value(text_attrs{A_BOLD})));
+                                 VC_STYLE.value(text_attrs::with_bold())));
         disc_str.append(this->fos_log_helper.ldh_msg_format);
     }
 
@@ -494,6 +507,8 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             string_attr(line_range(3, 3 + name.length()),
                         VC_STYLE.value(vc.attrs_for_ident(name.to_string()))));
 
+        this->fos_row_to_field_meta.emplace(this->fos_lines.size(),
+                                            row_info{std::nullopt, val});
         this->fos_lines.emplace_back(al);
         this->add_key_line_attrs(
             this->fos_unknown_key_size,
@@ -546,7 +561,20 @@ field_overlay_source::build_meta_line(const listview_curses& lv,
     const auto& line_meta = *(line_meta_opt.value());
     size_t filename_width = this->fos_lss.get_filename_offset();
 
-    if (!line_meta.bm_opid.empty()) {
+    auto file_and_line = this->fos_lss.find_line_with_file(row);
+    auto* format = file_and_line->first->get_format_ptr();
+    auto field_states = format->get_field_states();
+    auto show_opid = false;
+    auto field_iter = field_states.find(log_format::LOG_OPID_STR);
+    if (field_iter != field_states.end() && !field_iter->second.is_hidden()) {
+        show_opid = true;
+    }
+    if (row == tc->get_selection() && !this->fos_contexts.empty()
+        && this->fos_contexts.top().c_show)
+    {
+        show_opid = true;
+    }
+    if (show_opid && !line_meta.bm_opid.empty()) {
         auto al = attr_line_t()
                       .append(" Op ID: "_table_header)
                       .append(lnav::roles::identifier(line_meta.bm_opid))
@@ -682,12 +710,13 @@ field_overlay_source::add_key_line_attrs(int key_size, bool last_line)
 {
     auto& sa = this->fos_lines.back().get_attrs();
     struct line_range lr(1, 2);
-    int64_t graphic = (int64_t) (last_line ? ACS_LLCORNER : ACS_LTEE);
+
+    auto graphic = (last_line ? NCACS_LLCORNER : NCACS_LTEE);
     sa.emplace_back(lr, VC_GRAPHIC.value(graphic));
 
     lr.lr_start = 3 + key_size + 3;
     lr.lr_end = -1;
-    sa.emplace_back(lr, VC_STYLE.value(text_attrs{A_BOLD}));
+    sa.emplace_back(lr, VC_STYLE.value(text_attrs::with_bold()));
 }
 
 void
@@ -728,6 +757,8 @@ field_overlay_source::list_header_for_overlay(const listview_curses& lv,
         retval.append("  ")
             .append("SPC"_hotkey)
             .append(": hide/show field  ")
+            .append("c"_hotkey)
+            .append(": copy field value  ")
             .append("Esc"_hotkey)
             .append(": exit this panel");
     } else {
