@@ -42,8 +42,10 @@
 #include "config.h"
 #include "lnav.hh"
 #include "mapbox/variant.hpp"
+#include "md4cpp.hh"
 #include "sql_util.hh"
 
+using namespace md4cpp::literals;
 using namespace lnav::roles::literals;
 
 namespace files_model {
@@ -87,16 +89,13 @@ from_selection(vis_line_t sel_vis)
 }
 }  // namespace files_model
 
-files_sub_source::
-files_sub_source()
-{
-}
+files_sub_source::files_sub_source() {}
 
 bool
-files_sub_source::list_input_handle_key(listview_curses& lv, int ch)
+files_sub_source::list_input_handle_key(listview_curses& lv, const ncinput& ch)
 {
-    switch (ch) {
-        case KEY_ENTER:
+    switch (ch.eff_text[0]) {
+        case NCKEY_ENTER:
         case '\r': {
             auto sel = files_model::from_selection(lv.get_selection());
 
@@ -248,7 +247,7 @@ files_sub_source::text_line_width(textview_curses& curses)
     return 512;
 }
 
-void
+line_info
 files_sub_source::text_value_for_line(textview_curses& tc,
                                       int line,
                                       std::string& value_out,
@@ -269,7 +268,7 @@ files_sub_source::text_value_for_line(textview_curses& tc,
     attr_line_builder alb(al);
 
     if (selected) {
-        al.append(" ", VC_GRAPHIC.value(ACS_RARROW));
+        al.append(" ", VC_GRAPHIC.value(NCACS_RARROW));
     } else {
         al.append(" ");
     }
@@ -294,7 +293,7 @@ files_sub_source::text_value_for_line(textview_curses& tc,
             }
 
             value_out = al.get_string();
-            return;
+            return {};
         }
 
         line -= errs->size();
@@ -320,11 +319,11 @@ files_sub_source::text_value_for_line(textview_curses& tc,
             al.with_attr_for_all(VC_ROLE.value(cursor_role));
         }
         if (line == fc.fc_other_files.size() - 1) {
-            al.with_attr_for_all(VC_STYLE.value(text_attrs{A_UNDERLINE}));
+            al.with_attr_for_all(VC_STYLE.value(text_attrs::with_underline()));
         }
 
         value_out = al.get_string();
-        return;
+        return {};
     }
 
     line -= fc.fc_other_files.size();
@@ -360,6 +359,8 @@ files_sub_source::text_value_for_line(textview_curses& tc,
 
     value_out = al.get_string();
     this->fss_last_line_len = value_out.length();
+
+    return {};
 }
 
 void
@@ -438,11 +439,16 @@ files_sub_source::text_handle_mouse(
     const listview_curses::display_line_content_t&,
     mouse_event& me)
 {
+    auto nci = ncinput{};
     if (me.is_click_in(mouse_button_t::BUTTON_LEFT, 1, 3)) {
-        this->list_input_handle_key(tc, ' ');
+        nci.id = ' ';
+        nci.eff_text[0] = ' ';
+        this->list_input_handle_key(tc, nci);
     }
     if (me.is_double_click_in(mouse_button_t::BUTTON_LEFT, line_range{4, -1})) {
-        this->list_input_handle_key(tc, '\r');
+        nci.id = '\r';
+        nci.eff_text[0] = '\r';
+        this->list_input_handle_key(tc, nci);
     }
 
     return false;
@@ -488,10 +494,14 @@ files_sub_source::text_selection_changed(textview_curses& tc)
             auto actual_path = lf->get_actual_path();
             auto format = lf->get_format();
 
+            details.emplace_back(attr_line_t()
+                                     .appendf(FMT_STRING("{}"), path.filename())
+                                     .with_attr_for_all(VC_STYLE.value(
+                                         text_attrs::with_bold())));
             details.emplace_back(
-                attr_line_t()
-                    .appendf(FMT_STRING("{}"), path)
-                    .with_attr_for_all(VC_ROLE.value(role_t::VCR_IDENTIFIER)));
+                attr_line_t("  ")
+                    .append(":open_file_folder:"_emoji)
+                    .appendf(FMT_STRING(" {}"), path.parent_path()));
             const auto notes = lf->get_notes();
             if (!notes.empty()) {
                 details.emplace_back(
@@ -532,7 +542,10 @@ files_sub_source::text_selection_changed(textview_curses& tc)
                     .right_justify(NAME_WIDTH)
                     .append(": ")
                     .append(lnav::to_rfc3339_string(
-                        convert_log_time_to_local(lf->get_modified_time()),
+                        convert_log_time_to_local(
+                            std::chrono::duration_cast<std::chrono::seconds>(
+                                lf->get_modified_time())
+                                .count()),
                         0,
                         'T')));
             details.emplace_back(
@@ -542,6 +555,15 @@ files_sub_source::text_selection_changed(textview_curses& tc)
                     .append(": ")
                     .append(humanize::file_size(lf->get_index_size(),
                                                 humanize::alignment::none)));
+            if (lf->is_compressed()) {
+                details.emplace_back(attr_line_t()
+                                         .append("Compressed Size"_h3)
+                                         .right_justify(NAME_WIDTH)
+                                         .append(": ")
+                                         .append(humanize::file_size(
+                                             lf->get_stat().st_size,
+                                             humanize::alignment::none)));
+            }
 
             details.emplace_back(attr_line_t()
                                      .append("Lines"_h3)
@@ -569,6 +591,27 @@ files_sub_source::text_selection_changed(textview_curses& tc)
                                     - lf->front().get_timeval())
                                     .to_string()));
             }
+
+            auto file_options_opt = lf->get_file_options();
+            if (file_options_opt) {
+                auto& [path, file_options] = file_options_opt.value();
+
+                details.emplace_back(attr_line_t()
+                                         .append("Options Path"_h3)
+                                         .right_justify(NAME_WIDTH)
+                                         .append(": ")
+                                         .append(lnav::roles::file(path)));
+                if (file_options.fo_default_zone.pp_value != nullptr) {
+                    details.emplace_back(attr_line_t()
+                                             .append("Timezone"_h3)
+                                             .right_justify(NAME_WIDTH)
+                                             .append(": ")
+                                             .append(lnav::roles::symbol(
+                                                 file_options.fo_default_zone
+                                                     .pp_value->name())));
+                }
+            }
+
             {
                 auto line = attr_line_t("  ")
                                 .append("Log Format"_h2)
@@ -593,9 +636,25 @@ files_sub_source::text_selection_changed(textview_curses& tc)
                           details.emplace_back(attr_line_t("    ").append(al));
                       });
             } else {
-                const auto um = lnav::console::user_message::info(attr_line_t(
-                    "The file contents did not match any log "
-                    "formats and can be accessed in the TEXT view"));
+                auto cmd
+                    = attr_line_t("lnav -m format ")
+                          .append("format-name",
+                                  VC_STYLE.value(text_attrs::with_underline()))
+                          .append(" test ")
+                          .append(lf->get_filename())
+                          .with_attr_for_all(
+                              VC_ROLE.value(role_t::VCR_QUOTED_CODE));
+                const auto um
+                    = lnav::console::user_message::info(
+
+                          "The file contents did not match any log "
+                          "formats and can be accessed in the TEXT view")
+                          .with_help(attr_line_t("If you expected this file to "
+                                                 "match a particular "
+                                                 "format, you can run the "
+                                                 "following to get more "
+                                                 "details:\n  ")
+                                         .append(cmd));
                 um.to_attr_line().rtrim().split_lines()
                     | lnav::itertools::for_each([&details](const auto& al) {
                           details.emplace_back(attr_line_t("    ").append(al));
@@ -614,6 +673,30 @@ files_sub_source::text_selection_changed(textview_curses& tc)
                 }
             }
 
+            const auto& ili = lf->get_invalid_line_info();
+            if (ili.ili_total > 0) {
+                auto dotdot = ili.ili_total
+                    > logfile::invalid_line_info::MAX_INVALID_LINES;
+                auto um = lnav::console::user_message::error(
+                              attr_line_t()
+                                  .append(lnav::roles::number(
+                                      fmt::to_string(ili.ili_total)))
+                                  .append(" line(s) are not handled by the "
+                                          "format and considered invalid"))
+                              .with_note(attr_line_t("Lines: ")
+                                             .join(ili.ili_lines, ", ")
+                                             .append(dotdot ? ", ..." : ""))
+                              .with_help(
+                                  "The format may be need to be adjusted to "
+                                  "capture these lines");
+
+                details.emplace_back(attr_line_t());
+                um.to_attr_line().rtrim().split_lines()
+                    | lnav::itertools::for_each([&details](const auto& al) {
+                          details.emplace_back(attr_line_t("    ").append(al));
+                      });
+            }
+
             {
                 const auto& meta = lf->get_embedded_metadata();
 
@@ -634,7 +717,7 @@ files_sub_source::text_selection_changed(textview_curses& tc)
                                     meta_pair.second.m_format))));
                         line_range lr{6, -1};
                         details.emplace_back(attr_line_t().with_attr(
-                            string_attr{lr, VC_GRAPHIC.value(ACS_HLINE)}));
+                            string_attr{lr, VC_GRAPHIC.value(NCACS_HLINE)}));
                         const auto val_sf = string_fragment::from_str(
                             meta_pair.second.m_value);
                         for (const auto val_line : val_sf.split_lines()) {
