@@ -42,8 +42,7 @@
 
 using namespace lnav::roles::literals;
 
-filter_sub_source::
-filter_sub_source(std::shared_ptr<readline_curses> editor)
+filter_sub_source::filter_sub_source(std::shared_ptr<readline_curses> editor)
     : fss_editor(editor)
 {
     this->fss_editor->set_x(25);
@@ -81,12 +80,12 @@ filter_sub_source::register_view(textview_curses* tc)
 }
 
 bool
-filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
+filter_sub_source::list_input_handle_key(listview_curses& lv, const ncinput& ch)
 {
     if (this->fss_editing) {
-        switch (ch) {
-            case KEY_ESCAPE:
-            case KEY_CTRL(']'):
+        switch (ch.eff_text[0]) {
+            case NCKEY_ESC:
+            case NCKEY_GS:
                 this->fss_editor->abort();
                 return true;
             default:
@@ -95,7 +94,7 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
         }
     }
 
-    switch (ch) {
+    switch (ch.eff_text[0]) {
         case 'f': {
             auto* top_view = *lnav_data.ld_view_stack.top();
             auto* tss = top_view->get_sub_source();
@@ -233,7 +232,7 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
             return true;
         }
         case '\r':
-        case KEY_ENTER: {
+        case NCKEY_ENTER: {
             textview_curses* top_view = *lnav_data.ld_view_stack.top();
             text_sub_source* tss = top_view->get_sub_source();
             filter_stack& fs = tss->get_filters();
@@ -318,7 +317,7 @@ filter_sub_source::text_line_width(textview_curses& curses)
     return retval;
 }
 
-void
+line_info
 filter_sub_source::text_value_for_line(textview_curses& tc,
                                        int line,
                                        std::string& value_out,
@@ -336,7 +335,7 @@ filter_sub_source::text_value_for_line(textview_curses& tc,
     attr_line_builder alb(al);
 
     if (selected) {
-        al.append(" ", VC_GRAPHIC.value(ACS_RARROW));
+        al.append(" ", VC_GRAPHIC.value(NCACS_RARROW));
     } else {
         al.append(" ");
     }
@@ -373,7 +372,7 @@ filter_sub_source::text_value_for_line(textview_curses& tc,
         }
     }
 
-    al.append(" hits ").append("|", VC_GRAPHIC.value(ACS_VLINE)).append(" ");
+    al.append(" hits ").append("|", VC_GRAPHIC.value(NCACS_VLINE)).append(" ");
 
     attr_line_t content{tf->get_id()};
     switch (tf->get_lang()) {
@@ -393,6 +392,8 @@ filter_sub_source::text_value_for_line(textview_curses& tc,
     }
 
     value_out = al.get_string();
+
+    return {};
 }
 
 void
@@ -457,8 +458,8 @@ filter_sub_source::rl_change(readline_curses* rc)
                         ? role_t::VCR_DIFF_DELETE
                         : role_t::VCR_DIFF_ADD;
                     hl.with_role(role);
-                    hl.with_attrs(text_attrs{A_BLINK | A_REVERSE});
-
+                    hl.with_attrs(text_attrs::with_styles(
+                        text_attrs::style::blink, text_attrs::style::reverse));
                     hm[{highlight_source_t::PREVIEW, "preview"}] = hl;
                     top_view->set_needs_update();
                     lnav_data.ld_filter_help_status_source.fss_error.clear();
@@ -534,8 +535,7 @@ filter_sub_source::rl_perform(readline_curses* rc)
                 if (compile_res.isErr()) {
                     auto ce = compile_res.unwrapErr();
                     auto um = lnav::console::to_user_message(INPUT_SRC, ce);
-                    lnav_data.ld_exec_context.ec_error_callback_stack.back()(
-                        um);
+                    lnav_data.ld_exec_context.ec_msg_callback_stack.back()(um);
                     this->rl_abort(rc);
                 } else {
                     tf->lf_deleted = true;
@@ -579,8 +579,7 @@ filter_sub_source::rl_perform(readline_curses* rc)
                               .with_reason(sqlite3_errmsg(lnav_data.ld_db.in()))
                               .with_snippet(lnav::console::snippet::from(
                                   INPUT_SRC, sqlerr));
-                    lnav_data.ld_exec_context.ec_error_callback_stack.back()(
-                        um);
+                    lnav_data.ld_exec_context.ec_msg_callback_stack.back()(um);
                     this->rl_abort(rc);
                 } else {
                     lnav_data.ld_log_source.set_sql_filter(new_value,
@@ -643,7 +642,7 @@ filter_sub_source::rl_display_matches(readline_curses* rc)
 
         for (const auto& match : matches) {
             if (match == current_match) {
-                al.append(match, VC_STYLE.value(text_attrs{A_REVERSE}));
+                al.append(match, VC_STYLE.value(text_attrs::with_reverse()));
                 selected_line = line;
             } else {
                 al.append(match);
@@ -692,18 +691,26 @@ filter_sub_source::text_handle_mouse(
     const listview_curses::display_line_content_t&,
     mouse_event& me)
 {
+    log_debug("filter mouse %d", me.me_x);
     if (this->fss_editing) {
         return true;
     }
+    auto nci = ncinput{};
     if (me.is_click_in(mouse_button_t::BUTTON_LEFT, 1, 3)) {
-        this->list_input_handle_key(tc, ' ');
+        nci.id = ' ';
+        nci.eff_text[0] = ' ';
+        this->list_input_handle_key(tc, nci);
     }
     if (me.is_click_in(mouse_button_t::BUTTON_LEFT, 4, 7)) {
-        this->list_input_handle_key(tc, 't');
+        nci.id = 't';
+        nci.eff_text[0] = 't';
+        this->list_input_handle_key(tc, nci);
     }
     if (me.is_double_click_in(mouse_button_t::BUTTON_LEFT, line_range{25, -1}))
     {
-        this->list_input_handle_key(tc, '\r');
+        nci.id = '\r';
+        nci.eff_text[0] = '\r';
+        this->list_input_handle_key(tc, nci);
     }
     return true;
 }
