@@ -32,6 +32,7 @@
 #include "base/ansi_scrubber.hh"
 #include "base/injector.hh"
 #include "base/itertools.hh"
+#include "base/keycodes.hh"
 #include "base/math_util.hh"
 #include "base/opt_util.hh"
 #include "bookmarks.hh"
@@ -63,13 +64,13 @@ handle_keyseq(const char* keyseq)
 
     ec.ec_label_source_stack.push_back(&lnav_data.ld_db_row_source);
     ec.ec_global_vars = lnav_data.ld_exec_context.ec_global_vars;
-    ec.ec_error_callback_stack
-        = lnav_data.ld_exec_context.ec_error_callback_stack;
+    ec.ec_msg_callback_stack = lnav_data.ld_exec_context.ec_msg_callback_stack;
+    ec.ec_ui_callbacks = lnav_data.ld_exec_context.ec_ui_callbacks;
     var_stack.push(std::map<std::string, scoped_value_t>());
     // XXX push another so it doesn't look like interactive use
     var_stack.push(std::map<std::string, scoped_value_t>());
     auto& vars = var_stack.top();
-    vars["keyseq"] = keyseq;
+    vars["keyseq"] = string_fragment::from_c_str(keyseq);
     const auto& kc = iter->second;
 
     log_debug(
@@ -83,7 +84,7 @@ handle_keyseq(const char* keyseq)
     } else {
         auto um = result.unwrapErr();
 
-        ec.ec_error_callback_stack.back()(um);
+        ec.ec_msg_callback_stack.back()(um);
     }
 
     if (!kc.kc_alt_msg.empty()) {
@@ -103,17 +104,72 @@ handle_keyseq(const char* keyseq)
     return true;
 }
 
+void
+handle_paste_content(notcurses* nc, const ncinput& ch)
+{
+    auto& ec = lnav_data.ld_exec_context;
+
+    switch (ch.paste_content[0]) {
+        case '/':
+        case ':':
+        case ';':
+        case '|': {
+            static const auto lf_re = lnav::pcre2pp::code::from_const("\r\n?");
+            static const intern_string_t SRC
+                = intern_string::lookup("pasted-content");
+
+            auto paste_sf = string_fragment::from_c_str(ch.paste_content);
+            auto cmdline = lf_re.replace(paste_sf, "\n");
+            auto sg = ec.enter_source(SRC, 0, cmdline);
+
+            auto exec_res = ec.execute(cmdline);
+            if (exec_res.isOk()) {
+                lnav_data.ld_rl_view->set_value(exec_res.unwrap());
+            } else {
+                auto um = exec_res.unwrapErr();
+
+                ec.ec_msg_callback_stack.back()(um);
+            }
+            break;
+        }
+        default: {
+            auto um
+                = lnav::console::user_message::error(
+                      attr_line_t("ignoring pasted content"))
+                      .with_reason(
+                          attr_line_t("content does not start with one of the "
+                                      "expected prefixes: ")
+                              .append(":"_quoted_code)
+                              .append(" for lnav commands; ")
+                              .append(";"_quoted_code)
+                              .append(" for SQL queries; ")
+                              .append("/"_quoted_code)
+                              .append(" for searches; ")
+                              .append("|"_quoted_code)
+                              .append(" scripts"));
+
+            ec.ec_msg_callback_stack.back()(um);
+            break;
+        }
+    }
+}
+
 bool
-handle_paging_key(int ch, const char* keyseq)
+handle_paging_key(notcurses* nc, const ncinput& ch, const char* keyseq)
 {
     if (lnav_data.ld_view_stack.empty()) {
         return false;
     }
 
-    textview_curses* tc = *lnav_data.ld_view_stack.top();
-    exec_context& ec = lnav_data.ld_exec_context;
-    text_sub_source* tc_tss = tc->get_sub_source();
-    bookmarks<vis_line_t>::type& bm = tc->get_bookmarks();
+    auto* tc = *lnav_data.ld_view_stack.top();
+    auto& ec = lnav_data.ld_exec_context;
+    auto* tc_tss = tc->get_sub_source();
+    auto& bm = tc->get_bookmarks();
+
+    if (ch.id == NCKEY_PASTE) {
+        handle_paste_content(nc, ch);
+        return true;
+    }
 
     if (tc->get_overlay_selection()) {
         if (tc->handle_key(ch)) {
@@ -129,13 +185,13 @@ handle_paging_key(int ch, const char* keyseq)
         return true;
     }
 
-    auto lss = dynamic_cast<logfile_sub_source*>(tc->get_sub_source());
-    auto text_accel_p = dynamic_cast<text_accel_source*>(tc->get_sub_source());
+    auto* lss = dynamic_cast<logfile_sub_source*>(tc->get_sub_source());
+    auto* text_accel_p = dynamic_cast<text_accel_source*>(tc->get_sub_source());
 
     /* process the command keystroke */
-    switch (ch) {
+    switch (ch.eff_text[0]) {
         case 0x7f:
-        case KEY_BACKSPACE:
+        case NCKEY_BACKSPACE:
             break;
 
         case 'a':
@@ -172,34 +228,29 @@ handle_paging_key(int ch, const char* keyseq)
             }
             break;
 
-        case KEY_F(2):
-            if (xterm_mouse::is_available()) {
-                auto& mouse_i = injector::get<xterm_mouse&>();
-                mouse_i.set_enabled(!mouse_i.is_enabled());
+        case NCKEY_F02: {
+            auto& mouse_i = injector::get<xterm_mouse&>();
+            mouse_i.set_enabled(nc, !mouse_i.is_enabled());
 
-                auto al = attr_line_t("mouse mode -- ")
-                              .append(mouse_i.is_enabled() ? "enabled"_symbol
-                                                           : "disabled"_symbol)
-                              .move();
-                if (mouse_i.is_enabled()
-                    && lnav_config.lc_mouse_mode == lnav_mouse_mode::disabled)
-                {
-                    al.append(" -- enable permanently with ")
-                        .append(":config /ui/mouse/mode enabled"_quoted_code);
+            auto al = attr_line_t("mouse mode -- ")
+                          .append(mouse_i.is_enabled() ? "enabled"_symbol
+                                                       : "disabled"_symbol)
+                          .move();
+            if (mouse_i.is_enabled()
+                && lnav_config.lc_mouse_mode == lnav_mouse_mode::disabled)
+            {
+                al.append(" -- enable permanently with ")
+                    .append(":config /ui/mouse/mode enabled"_quoted_code);
 
-                    auto clear_note = prepare_stmt(lnav_data.ld_db, R"(
+                auto clear_note = prepare_stmt(lnav_data.ld_db, R"(
 DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
 )");
-                    clear_note.unwrap().execute();
-                }
-                auto um = lnav::console::user_message::ok(al);
-                lnav_data.ld_rl_view->set_attr_value(um.to_attr_line());
-            } else {
-                lnav_data.ld_rl_view->set_value(
-                    "error: mouse support is not available, make sure your "
-                    "TERM is set to xterm or xterm-256color");
+                clear_note.unwrap().execute();
             }
+            auto um = lnav::console::user_message::ok(al);
+            lnav_data.ld_rl_view->set_attr_value(um.to_attr_line());
             break;
+        }
 
         case 'C':
             if (lss) {
@@ -435,7 +486,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                              ->is_message())
                     {
                     } else if (text_accel_p->get_line_accel_direction(next_top)
-                               == log_accel::A_DECEL)
+                               == log_accel::direction_t::A_DECEL)
                     {
                         if (!tc->is_selectable()) {
                             --next_top;
@@ -466,7 +517,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                              ->is_message())
                     {
                     } else if (text_accel_p->get_line_accel_direction(next_top)
-                               == log_accel::A_DECEL)
+                               == log_accel::direction_t::A_DECEL)
                     {
                         if (!tc->is_selectable()) {
                             --next_top;
@@ -531,7 +582,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
             } else if (lss) {
                 lss->time_for_row(tc->get_selection()) |
                     [lss, ch, tc](auto first_ri) {
-                        int step = ch == 'D' ? (24 * 60 * 60) : (60 * 60);
+                        int step = ch.id == 'D' ? (24 * 60 * 60) : (60 * 60);
                         time_t top_time = first_ri.ri_time.tv_sec;
                         lss->find_from_time(top_time - step) | [tc](auto line) {
                             if (line != 0_vl) {
@@ -549,7 +600,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
             if (lss) {
                 lss->time_for_row(tc->get_selection()) |
                     [ch, lss, tc](auto first_ri) {
-                        int step = ch == 'd' ? (24 * 60 * 60) : (60 * 60);
+                        int step = ch.id == 'd' ? (24 * 60 * 60) : (60 * 60);
                         lss->find_from_time(first_ri.ri_time.tv_sec + step) |
                             [tc](auto line) { tc->set_selection(line); };
                     };
@@ -581,7 +632,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                     bool found = false;
 
                     while (true) {
-                        if (ch == 'o') {
+                        if (ch.id == 'o') {
                             ++next_win_iter;
                             if (next_win_iter == next_win.end()) {
                                 break;
@@ -682,32 +733,25 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
         }
 
         case 'V': {
-            textview_curses* db_tc = &lnav_data.ld_views[LNV_DB];
-            db_label_source& dls = lnav_data.ld_db_row_source;
+            auto* db_tc = &lnav_data.ld_views[LNV_DB];
+            auto& dls = lnav_data.ld_db_row_source;
 
             if (toggle_view(db_tc)) {
-                auto log_line_index = dls.column_name_to_index("log_line");
+                auto log_line_col = dls.column_name_to_index("log_line");
 
-                if (!log_line_index) {
-                    log_line_index = dls.column_name_to_index("min(log_line)");
+                if (!log_line_col) {
+                    log_line_col = dls.column_name_to_index("min(log_line)");
                 }
 
-                if (log_line_index) {
-                    fmt::memory_buffer linestr;
-                    int line_number = (int) tc->get_selection();
+                if (log_line_col) {
+                    auto line_number = tc->get_selection();
                     unsigned int row;
 
-                    fmt::format_to(std::back_inserter(linestr),
-                                   FMT_STRING("{}"),
-                                   line_number);
-                    linestr.push_back('\0');
-                    for (row = 0; row < dls.dls_rows.size(); row++) {
-                        if (strcmp(dls.dls_rows[row][log_line_index.value()],
-                                   linestr.data())
-                            == 0)
+                    for (row = 0; row < dls.dls_row_cursors.size(); row++) {
+                        auto db_line = vis_line_t(row);
+                        if (dls.get_cell_as_int64(db_line, log_line_col.value())
+                            == line_number)
                         {
-                            vis_line_t db_line(row);
-
                             db_tc->set_selection(db_line);
                             db_tc->set_needs_update();
                             break;
@@ -715,34 +759,35 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                     }
                 }
             } else if (db_tc->get_inner_height() > 0) {
-                int db_row = db_tc->get_selection();
+                auto db_row = db_tc->get_selection();
                 tc = &lnav_data.ld_views[LNV_LOG];
-                auto log_line_index = dls.column_name_to_index("log_line");
+                auto log_line_col = dls.column_name_to_index("log_line");
 
-                if (!log_line_index) {
-                    log_line_index = dls.column_name_to_index("min(log_line)");
+                if (!log_line_col) {
+                    log_line_col = dls.column_name_to_index("min(log_line)");
                 }
 
-                if (log_line_index) {
-                    unsigned int line_number;
+                if (log_line_col) {
+                    auto line_number
+                        = dls.get_cell_as_int64(db_row, log_line_col.value());
 
-                    if (sscanf(dls.dls_rows[db_row][log_line_index.value()],
-                               "%d",
-                               &line_number)
-                        && line_number < tc->listview_rows(*tc))
-                    {
-                        tc->set_selection(vis_line_t(line_number));
+                    if (line_number < tc->listview_rows(*tc)) {
+                        tc->set_selection(vis_line_t(line_number.value()));
                         tc->set_needs_update();
                     }
                 } else {
                     for (size_t lpc = 0; lpc < dls.dls_headers.size(); lpc++) {
                         date_time_scanner dts;
-                        struct timeval tv;
-                        struct exttm tm;
-                        const char* col_value = dls.dls_rows[db_row][lpc];
-                        size_t col_len = strlen(col_value);
+                        timeval tv;
+                        exttm tm;
+                        const auto col_value
+                            = dls.get_cell_as_string(db_row, lpc);
 
-                        if (dts.scan(col_value, col_len, nullptr, &tm, tv)
+                        if (dts.scan(col_value.c_str(),
+                                     col_value.length(),
+                                     nullptr,
+                                     &tm,
+                                     tv)
                             != nullptr)
                         {
                             lnav_data.ld_log_source.find_from_time(tv) |
@@ -757,8 +802,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
             }
         } break;
 
-        case '\t':
-        case KEY_BTAB:
+        case NCKEY_TAB:
             if (tc == &lnav_data.ld_views[LNV_DB]) {
             } else if (tc == &lnav_data.ld_views[LNV_SPECTRO]) {
                 set_view_mode(ln_mode_t::SPECTRO_DETAILS);
@@ -771,8 +815,8 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
 
                     tc->get_data_source()->listview_value_for_rows(
                         *tc, tc->get_top(), rows);
-                    string_attrs_t& sa = rows[0].get_attrs();
-                    auto line_attr_opt = get_string_attr(sa, logline::L_FILE);
+                    auto& sa = rows[0].get_attrs();
+                    auto line_attr_opt = get_string_attr(sa, L_FILE);
                     if (line_attr_opt) {
                         const auto& fc = lnav_data.ld_active_files;
                         auto lf = line_attr_opt.value().get();
@@ -809,11 +853,11 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                     struct exttm tm;
                     bool done = false;
 
-                    if (ch == 'r') {
+                    if (ch.id == 'r') {
                         if (rt.is_negative()) {
                             rt.negate();
                         }
-                    } else if (ch == 'R') {
+                    } else if (ch.id == 'R') {
                         if (!rt.is_negative()) {
                             rt.negate();
                         }
