@@ -38,6 +38,7 @@
 #include <time.h>
 
 #include "base/func_util.hh"
+#include "base/keycodes.hh"
 #include "base/lnav_log.hh"
 #include "config.h"
 
@@ -45,11 +46,7 @@ using namespace std::chrono_literals;
 
 list_gutter_source listview_curses::DEFAULT_GUTTER_SOURCE;
 
-listview_curses::
-listview_curses()
-    : lv_scroll(noop_func{})
-{
-}
+listview_curses::listview_curses() : lv_scroll(noop_func{}) {}
 
 bool
 listview_curses::contains(int x, int y) const
@@ -79,10 +76,7 @@ listview_curses::update_top_from_selection()
         return;
     }
 
-    vis_line_t height;
-    unsigned long width;
-
-    this->get_dimensions(height, width);
+    auto [height, width] = this->get_dimensions();
     const auto inner_height = this->get_inner_height();
 
     if (this->lv_selection >= inner_height) {
@@ -98,31 +92,60 @@ listview_curses::update_top_from_selection()
         return;
     }
 
-    if (this->lv_sync_selection_and_top) {
+    if (this->lv_sync_selection_and_top || height <= this->lv_tail_space
+        || this->lv_top > this->lv_selection)
+    {
         this->set_top(this->lv_selection);
-    } else if (height <= this->lv_tail_space) {
-        this->set_top(this->lv_selection);
-    } else if (this->lv_selection > (this->lv_top + height - 1_vl)) {
-        auto diff = this->lv_selection - (this->lv_top + height - 1_vl);
+        return;
+    }
 
-        if (height < 10 || diff < (height / 8_vl)) {
-            // for small differences between the bottom and the
-            // selection, just move a little bit.
-            this->set_top(this->lv_selection - height + 1_vl, true);
-        } else {
-            // for large differences, put the focus in the middle
-            this->set_top(this->lv_selection - height / 2_vl, true);
+    auto layout = this->layout_for_row(this->lv_selection);
+    auto min_top_for_sel
+        = vis_line_t(this->lv_selection - layout.lr_above_line_heights.size());
+    if (this->lv_top < min_top_for_sel) {
+        this->set_top(min_top_for_sel);
+    } else if (this->lv_top == this->lv_selection) {
+        if (!layout.lr_above_line_heights.empty()) {
+            auto avail_height = height - layout.lr_desired_row_height;
+            if (layout.lr_above_line_heights.front() < avail_height) {
+                this->lv_top -= 1_vl;
+            }
         }
-    } else if (this->lv_selection < this->lv_top) {
-        auto diff = this->lv_top - this->lv_selection;
+    }
+}
 
-        if (this->lv_selection > 0 && (height < 10 || diff < (height / 8_vl))) {
-            this->set_top(this->lv_selection - 1_vl);
-        } else if (this->lv_selection < height) {
-            this->set_top(0_vl);
+void
+listview_curses::get_dimensions(vis_line_t& height_out,
+                                unsigned long& width_out) const
+{
+    unsigned int height;
+
+    if (this->lv_window == nullptr) {
+        height_out = std::max(this->lv_height, 1_vl);
+        if (this->lv_source) {
+            width_out = this->lv_source->listview_width(*this);
         } else {
-            this->set_top(this->lv_selection - height / 2_vl, true);
+            width_out = 80;
         }
+    } else {
+        unsigned int width_tmp;
+
+        ncplane_dim_yx(this->lv_window, &height, &width_tmp);
+        width_out = width_tmp;
+        if (this->lv_height < 0) {
+            height_out
+                = vis_line_t(height) + this->lv_height - vis_line_t(this->vc_y);
+            if (height_out < 0_vl) {
+                height_out = 0_vl;
+            }
+        } else {
+            height_out = this->lv_height;
+        }
+    }
+    if (this->vc_x < width_out) {
+        width_out -= this->vc_x;
+    } else {
+        width_out = 0;
     }
 }
 
@@ -166,7 +189,7 @@ listview_curses::reload_data()
 }
 
 bool
-listview_curses::handle_key(int ch)
+listview_curses::handle_key(const ncinput& ch)
 {
     for (auto& lv_input_delegate : this->lv_input_delegates) {
         if (lv_input_delegate->list_input_handle_key(*this, ch)) {
@@ -180,33 +203,40 @@ listview_curses::handle_key(int ch)
     bool retval = true;
 
     this->get_dimensions(height, width);
-    switch (ch) {
+    switch (ch.eff_text[0]) {
         case 'l':
-        case KEY_RIGHT:
-            this->shift_left(width / 2);
+        case NCKEY_RIGHT:
+            if (ncinput_shift_p(&ch)) {
+                this->shift_left(10);
+            } else {
+                this->shift_left(width / 2);
+            }
             break;
 
         case 'h':
-        case KEY_LEFT:
-            this->shift_left(-(width / 2));
+        case NCKEY_LEFT:
+            if (ncinput_shift_p(&ch)) {
+                this->shift_left(-10);
+            } else {
+                this->shift_left(-(width / 2));
+            }
             break;
         case 'L':
-        case KEY_SRIGHT:
             this->shift_left(10);
             break;
         case 'H':
-        case KEY_SLEFT:
             this->shift_left(-10);
             break;
 
         case '\r':
         case 'j':
-        case KEY_DOWN:
+        case NCKEY_DOWN:
+        case NCKEY_ENTER:
             this->shift_selection(shift_amount_t::down_line);
             break;
 
         case 'k':
-        case KEY_UP:
+        case NCKEY_UP:
             this->shift_selection(shift_amount_t::up_line);
             break;
 
@@ -228,7 +258,14 @@ listview_curses::handle_key(int ch)
                 this->lv_overlay_source->list_value_for_overlay(
                     *this, this->get_selection(), overlay_content);
                 if (!overlay_content.empty()) {
+                    auto bot = this->get_bottom();
                     this->lv_overlay_focused = !this->lv_overlay_focused;
+                    auto overlay_height = vis_line_t(this->get_overlay_height(
+                        overlay_content.size(), height));
+
+                    if (this->lv_selection + overlay_height >= bot) {
+                        this->shift_top(overlay_height, true);
+                    }
                     this->lv_source->listview_selection_changed(*this);
                     this->set_needs_update();
                 }
@@ -238,8 +275,8 @@ listview_curses::handle_key(int ch)
             break;
 
         case 'b':
-        case KEY_BACKSPACE:
-        case KEY_PPAGE:
+        case NCKEY_BACKSPACE:
+        case NCKEY_PGUP:
             if (this->lv_overlay_focused) {
                 this->shift_selection(shift_amount_t::up_page);
                 break;
@@ -257,7 +294,7 @@ listview_curses::handle_key(int ch)
             break;
 
         case ' ':
-        case KEY_NPAGE: {
+        case NCKEY_PGDOWN: {
             if (this->lv_overlay_source != nullptr) {
                 std::vector<attr_line_t> overlay_content;
                 this->lv_overlay_source->list_value_for_overlay(
@@ -299,7 +336,7 @@ listview_curses::handle_key(int ch)
         }
 
         case 'g':
-        case KEY_HOME:
+        case NCKEY_HOME:
             if (this->lv_overlay_focused) {
                 this->lv_focused_overlay_top = 0_vl;
                 this->lv_focused_overlay_selection = 0_vl;
@@ -316,7 +353,7 @@ listview_curses::handle_key(int ch)
             break;
 
         case 'G':
-        case KEY_END: {
+        case NCKEY_END: {
             if (this->lv_overlay_focused) {
                 std::vector<attr_line_t> overlay_content;
                 this->lv_overlay_source->list_value_for_overlay(
@@ -334,8 +371,8 @@ listview_curses::handle_key(int ch)
                 break;
             }
 
-            vis_line_t last_line(this->get_inner_height() - 1);
-            vis_line_t tail_bottom(this->get_top_for_last_row());
+            auto last_line = this->get_inner_height() - 1_vl;
+            auto tail_bottom = this->get_top_for_last_row();
 
             if (this->is_selectable()) {
                 this->set_selection(last_line);
@@ -409,6 +446,7 @@ listview_curses::get_overlay_top(vis_line_t row, size_t count, size_t total)
 bool
 listview_curses::do_update()
 {
+    static auto& vc = view_colors::singleton();
     bool retval = false;
 
     if (this->lv_window == nullptr || this->lv_height == 0 || !this->vc_visible)
@@ -427,7 +465,6 @@ listview_curses::do_update()
 
     this->update_top_from_selection();
     while (this->vc_needs_update) {
-        auto& vc = view_colors::singleton();
         vis_line_t row;
         attr_line_t overlay_line;
         struct line_range lr;
@@ -452,6 +489,7 @@ listview_curses::do_update()
             std::min((size_t) height, row_count - (int) this->lv_top));
         this->lv_source->listview_value_for_rows(*this, row, rows);
         this->lv_display_lines.clear();
+        this->lv_display_lines_row = row;
         while (y < bottom) {
             lr.lr_start = this->lv_left;
             lr.lr_end = this->lv_left + wrap_width;
@@ -470,11 +508,12 @@ listview_curses::do_update()
                     require_ge(attr.sa_range.lr_start, 0);
                 }
 
-                this->lv_display_lines.push_back(main_content{row});
-                view_curses::mvwattrline_result write_res;
+                mvwattrline_result write_res;
                 do {
+                    this->lv_display_lines.push_back(main_content{row});
                     if (this->lv_word_wrap) {
-                        mvwhline(this->lv_window, y, this->vc_x, ' ', width);
+                        // XXX mvwhline(this->lv_window, y, this->vc_x, ' ',
+                        // width);
                     }
                     write_res = mvwattrline(this->lv_window,
                                             y,
@@ -528,11 +567,11 @@ listview_curses::do_update()
                             = this->lv_overlay_source->list_header_for_overlay(
                                 *this, row);
                         if (hdr) {
-                            auto ov_hdr_attrs = text_attrs{};
-                            ov_hdr_attrs.ta_attrs |= A_UNDERLINE;
-                            auto ov_hdr = hdr.value().with_attr_for_all(
+                            auto ov_hdr_attrs = text_attrs::with_underline();
+                            auto ov_hdr = hdr.value();
+                            ov_hdr.pad_to(width).with_attr_for_all(
                                 VC_STYLE.value(ov_hdr_attrs));
-                            this->lv_display_lines.push_back(
+                            this->lv_display_lines.emplace_back(
                                 static_overlay_content{});
                             mvwattrline(this->lv_window,
                                         y,
@@ -554,8 +593,11 @@ listview_curses::do_update()
                                 VC_ROLE.value(role_t::VCR_CURSOR_LINE));
                         }
 
-                        this->lv_display_lines.push_back(
-                            overlay_content{row, overlay_row});
+                        this->lv_display_lines.emplace_back(
+                            overlay_content{row,
+                                            overlay_row,
+                                            overlay_height,
+                                            row_overlay_content.size()});
                         mvwattrline(this->lv_window,
                                     y,
                                     this->vc_x,
@@ -594,11 +636,11 @@ listview_curses::do_update()
                             auto role = this->vc_default_role;
                             auto bar_role = role_t::VCR_SCROLLBAR;
                             text_attrs attrs;
-                            chtype ch = gutter_y == overlay_y
-                                ? ACS_URCORNER
+                            auto ch = gutter_y == overlay_y
+                                ? NCACS_URCORNER
                                 : (gutter_y == (overlay_y + overlay_height - 1)
-                                       ? ACS_LRCORNER
-                                       : ACS_VLINE);
+                                       ? NCACS_LRCORNER
+                                       : NCACS_VLINE);
 
                             if (gutter_y >= (unsigned int) scroll_y
                                 && gutter_y <= (unsigned int) lines)
@@ -606,28 +648,34 @@ listview_curses::do_update()
                                 role = bar_role;
                             }
                             attrs = vc.attrs_for_role(role);
-                            wattr_set(this->lv_window,
-                                      attrs.ta_attrs,
-                                      vc.ensure_color_pair(attrs.ta_fg_color,
-                                                           attrs.ta_bg_color),
-                                      nullptr);
-                            mvwaddch(this->lv_window,
-                                     gutter_y,
-                                     this->vc_x + width - 2,
-                                     ch);
+                            ncplane_putstr_yx(this->lv_window,
+                                              gutter_y,
+                                              this->vc_x + width - 2,
+                                              ch);
+                            ncplane_set_cell_yx(
+                                this->lv_window,
+                                gutter_y,
+                                this->vc_x + width - 2,
+                                attrs.ta_attrs | NCSTYLE_ALTCHARSET,
+                                view_colors::to_channels(attrs));
                         }
                     }
                 }
 
                 ++row;
             } else {
-                wattr_set(this->lv_window,
-                          role_attrs.ta_attrs,
-                          vc.ensure_color_pair(role_attrs.ta_fg_color,
-                                               role_attrs.ta_bg_color),
-                          nullptr);
+                nccell clear_cell;
+                nccell_init(&clear_cell);
+                nccell_prime(this->lv_window,
+                             &clear_cell,
+                             " ",
+                             0,
+                             view_colors::to_channels(role_attrs));
+                ncplane_cursor_move_yx(this->lv_window, y, this->vc_x);
+                ncplane_hline(this->lv_window, &clear_cell, width);
+                nccell_release(this->lv_window, &clear_cell);
+
                 this->lv_display_lines.push_back(empty_space{});
-                mvwhline(this->lv_window, y, this->vc_x, ' ', width);
                 ++y;
             }
         }
@@ -658,10 +706,10 @@ listview_curses::do_update()
                  gutter_y++)
             {
                 int range_start = 0, range_end;
-                role_t role = this->vc_default_role;
-                role_t bar_role = role_t::VCR_SCROLLBAR;
+                auto role = this->vc_default_role;
+                auto bar_role = role_t::VCR_SCROLLBAR;
                 text_attrs attrs;
-                chtype ch = ACS_VLINE;
+                const char* ch;
 
                 if (row_count > 0) {
                     range_start
@@ -677,27 +725,24 @@ listview_curses::do_update()
                     role = bar_role;
                 }
                 attrs = vc.attrs_for_role(role);
-                wattr_set(
-                    this->lv_window,
-                    attrs.ta_attrs,
-                    vc.ensure_color_pair(attrs.ta_fg_color, attrs.ta_bg_color),
-                    nullptr);
-                mvwaddch(this->lv_window, gutter_y, this->vc_x + width - 1, ch);
+                ncplane_putstr_yx(
+                    this->lv_window, gutter_y, this->vc_x + width - 1, ch);
+                ncplane_set_cell_yx(this->lv_window,
+                                    gutter_y,
+                                    this->vc_x + width - 1,
+                                    attrs.ta_attrs | NCSTYLE_ALTCHARSET,
+                                    view_colors::to_channels(attrs));
             }
-            wmove(this->lv_window, this->vc_y + height - 1, this->vc_x);
         }
 
         if (this->lv_show_bottom_border) {
-            cchar_t row_ch[width];
             int bottom_y = this->vc_y + height - 1;
-
-            mvwin_wchnstr(
-                this->lv_window, bottom_y, this->vc_x, row_ch, width - 1);
-            for (unsigned long lpc = 0; lpc < width - 1; lpc++) {
-                row_ch[lpc].attr |= A_UNDERLINE;
+            for (int lpc = 0; lpc < width - 1; lpc++) {
+                ncplane_on_styles_yx(this->lv_window,
+                                     bottom_y,
+                                     this->vc_x + lpc,
+                                     NCSTYLE_UNDERLINE);
             }
-            mvwadd_wchnstr(
-                this->lv_window, bottom_y, this->vc_x, row_ch, width - 1);
         }
 
         this->vc_needs_update = false;
@@ -706,6 +751,21 @@ listview_curses::do_update()
     return view_curses::do_update() || retval;
 }
 
+void
+listview_curses::set_show_details_in_overlay(bool val)
+{
+    if (this->lv_overlay_source == nullptr) {
+        return;
+    }
+
+    this->lv_overlay_source->set_show_details_in_overlay(val);
+    if (!val) {
+        return;
+    }
+
+    this->update_top_from_selection();
+}
+
 void
 listview_curses::shift_selection(shift_amount_t sa)
 {
@@ -714,7 +774,7 @@ listview_curses::shift_selection(shift_amount_t sa)
 
     this->get_dimensions(height, width);
     if (this->lv_overlay_focused) {
-        vis_line_t focused = this->get_selection();
+        const auto focused = this->get_selection();
         std::vector<attr_line_t> overlay_content;
 
         this->lv_overlay_source->list_value_for_overlay(
@@ -811,19 +871,7 @@ listview_curses::shift_selection(shift_amount_t sa)
         }
 
         this->set_selection_without_context(new_selection);
-        auto rows_avail = this->rows_available(this->lv_top, RD_DOWN);
-        if (height > rows_avail) {
-            rows_avail = height;
-        }
-        if (this->lv_selection > 0 && this->lv_selection <= this->lv_top) {
-            this->set_top(this->lv_selection - 1_vl);
-        } else if (rows_avail > this->lv_tail_space
-                   && (this->lv_selection > (this->lv_top + (rows_avail - 1_vl)
-                                             - this->lv_tail_space)))
-        {
-            this->set_top(this->lv_selection + this->lv_tail_space
-                          - (rows_avail - 1_vl));
-        }
+        this->update_top_from_selection();
     } else {
         this->shift_top(vis_line_t{offset});
     }
@@ -838,7 +886,7 @@ scroll_polarity(mouse_button_t button)
 bool
 listview_curses::handle_mouse(mouse_event& me)
 {
-    auto GUTTER_REPEAT_DELAY
+    static constexpr auto GUTTER_REPEAT_DELAY
         = std::chrono::duration_cast<std::chrono::microseconds>(100ms).count();
 
     if (view_curses::handle_mouse(me)) {
@@ -850,16 +898,50 @@ listview_curses::handle_mouse(mouse_event& me)
     }
 
     vis_line_t inner_height, height;
-    struct timeval diff;
     unsigned long width;
 
-    timersub(&me.me_time, &this->lv_mouse_time, &diff);
+    auto diff = me.me_time - this->lv_mouse_time;
     this->get_dimensions(height, width);
     inner_height = this->get_inner_height();
 
     switch (me.me_button) {
         case mouse_button_t::BUTTON_SCROLL_UP:
         case mouse_button_t::BUTTON_SCROLL_DOWN: {
+            if (me.me_y < this->lv_display_lines.size()) {
+                auto display_content = this->lv_display_lines[me.me_y];
+
+                if (display_content.is<overlay_content>()
+                    && this->lv_overlay_focused)
+                {
+                    auto oc = display_content.get<overlay_content>();
+
+                    if (oc.oc_height < oc.oc_inner_height) {
+                        if (me.me_button == mouse_button_t::BUTTON_SCROLL_UP
+                            && this->lv_focused_overlay_top > 0_vl)
+                        {
+                            this->lv_mouse_time = me.me_time;
+                            this->lv_focused_overlay_top -= 1_vl;
+                            this->set_needs_update();
+                        }
+
+                        if (me.me_button == mouse_button_t::BUTTON_SCROLL_DOWN
+                            && this->lv_focused_overlay_top
+                                < (oc.oc_inner_height - oc.oc_height))
+                        {
+                            this->lv_mouse_time = me.me_time;
+                            this->lv_focused_overlay_top += 1_vl;
+                            if (this->lv_focused_overlay_selection
+                                <= this->lv_focused_overlay_top)
+                            {
+                                this->lv_focused_overlay_selection
+                                    = this->lv_focused_overlay_top + 1_vl;
+                            }
+                            this->set_needs_update();
+                        }
+                        return true;
+                    }
+                }
+            }
             this->shift_top(vis_line_t(scroll_polarity(me.me_button) * 2_vl),
                             true);
             return true;
@@ -960,6 +1042,12 @@ listview_curses::handle_mouse(mouse_event& me)
     return true;
 }
 
+enum class selection_location_t {
+    upper,
+    middle,
+    lower,
+};
+
 void
 listview_curses::set_top(vis_line_t top, bool suppress_flash)
 {
@@ -973,33 +1061,51 @@ listview_curses::set_top(vis_line_t top, bool suppress_flash)
             alerter::singleton().chime("invalid top");
         }
     } else if (this->lv_top != top) {
-        auto old_top = this->lv_top;
         this->lv_top = top;
-        this->lv_focused_overlay_top = 0_vl;
-        this->lv_focused_overlay_selection = 0_vl;
         if (this->lv_selectable) {
             if (this->lv_selection < 0_vl) {
                 this->set_selection_without_context(top);
-            } else if (this->lv_selection < top) {
-                auto sel_diff = this->lv_selection - old_top;
-                this->set_selection_without_context(top + sel_diff);
             } else {
-                auto sel_diff = this->lv_selection - old_top;
-                auto bot = this->get_bottom();
-                unsigned long width;
-                vis_line_t height;
+                auto layout = this->layout_for_row(this->lv_top);
+                auto last_row = this->lv_top
+                    + vis_line_t(layout.lr_below_line_heights.size());
 
-                this->get_dimensions(height, width);
-
-                if (bot == -1_vl) {
-                    this->set_selection_without_context(this->lv_top);
-                } else if (this->lv_selection < this->lv_top
-                           || bot < this->lv_selection)
+                if (this->lv_top <= this->lv_selection
+                    && this->lv_selection <= last_row)
                 {
-                    if (top + sel_diff > bot) {
-                        this->set_selection_without_context(bot);
-                    } else {
-                        this->set_selection_without_context(top + sel_diff);
+                    // selection is already in view, nothing to do
+                } else if (layout.lr_below_line_heights.size() < 2) {
+                    this->set_selection_without_context(this->lv_top);
+                } else {
+                    auto sel_location = selection_location_t::middle;
+
+                    if (this->lv_top - 5_vl <= this->lv_selection
+                        && this->lv_selection < this->lv_top)
+                    {
+                        sel_location = selection_location_t::upper;
+                    } else if (last_row < this->lv_selection
+                               && this->lv_selection <= last_row + 5_vl)
+                    {
+                        sel_location = selection_location_t::lower;
+                    }
+
+                    switch (sel_location) {
+                        case selection_location_t::upper: {
+                            this->set_selection_without_context(this->lv_top
+                                                                + 1_vl);
+                            break;
+                        }
+                        case selection_location_t::middle: {
+                            auto middle_of_below = vis_line_t(
+                                layout.lr_below_line_heights.size() / 2);
+                            this->set_selection_without_context(
+                                this->lv_top + middle_of_below);
+                            break;
+                        }
+                        case selection_location_t::lower: {
+                            this->set_selection_without_context(last_row);
+                            break;
+                        }
                     }
                 }
             }
@@ -1023,8 +1129,80 @@ listview_curses::get_bottom() const
 }
 
 vis_line_t
-listview_curses::rows_available(vis_line_t line,
-                                listview_curses::row_direction_t dir) const
+listview_curses::height_for_row(vis_line_t row,
+                                vis_line_t height,
+                                unsigned long width) const
+{
+    auto retval = 1_vl;
+
+    if (this->lv_word_wrap) {
+        // source size plus some padding for decorations
+        auto len = this->lv_source->listview_size_for_row(*this, row) + 5;
+
+        while (len > width) {
+            len -= width;
+            retval += 1_vl;
+        }
+    }
+    if (this->lv_overlay_source != nullptr) {
+        std::vector<attr_line_t> overlay_content;
+
+        this->lv_overlay_source->list_value_for_overlay(
+            *this, row, overlay_content);
+        retval += vis_line_t(
+            this->get_overlay_height(overlay_content.size(), height));
+    }
+
+    return retval;
+}
+
+listview_curses::layout_result_t
+listview_curses::layout_for_row(vis_line_t row) const
+{
+    auto [height, width] = this->get_dimensions();
+    const auto inner_height = this->get_inner_height();
+    layout_result_t retval;
+
+    retval.lr_desired_row = row;
+    retval.lr_desired_row_height = this->height_for_row(row, height, width);
+    {
+        auto above_height_avail
+            = height - retval.lr_desired_row_height - this->lv_tail_space;
+        auto curr_above_row = row - 1_vl;
+        while (curr_above_row >= 0_vl && above_height_avail > 0_vl) {
+            auto curr_above_height
+                = this->height_for_row(curr_above_row, height, width);
+
+            above_height_avail -= curr_above_height;
+            if (above_height_avail < 0_vl) {
+                break;
+            }
+            curr_above_row -= 1_vl;
+            retval.lr_above_line_heights.emplace_back(curr_above_height);
+        }
+    }
+    {
+        auto below_height_avail
+            = height - retval.lr_desired_row_height - this->lv_tail_space;
+        auto curr_below_row = row + 1_vl;
+        while (curr_below_row < inner_height && below_height_avail > 0_vl) {
+            auto curr_below_height
+                = this->height_for_row(curr_below_row, height, width);
+
+            below_height_avail -= curr_below_height;
+            if (below_height_avail < 0_vl) {
+                break;
+            }
+            curr_below_row += 1_vl;
+            retval.lr_below_line_heights.emplace_back(curr_below_height);
+        }
+    }
+
+    return retval;
+}
+
+vis_line_t
+listview_curses::rows_available(vis_line_t line, row_direction_t dir) const
 {
     unsigned long width;
     vis_line_t height;
@@ -1133,31 +1311,12 @@ listview_curses::set_selection(vis_line_t sel)
         return;
     }
 
-    auto dim = this->get_dimensions();
-    auto diff = std::optional<vis_line_t>{};
-
-    if (this->lv_selection >= 0_vl && this->lv_selection > this->lv_top
-        && this->lv_selection < this->lv_top + dim.first
-        && (sel < this->lv_top || sel > this->lv_top + dim.first))
-    {
-        diff = this->lv_selection - this->lv_top;
+    if (sel < 0_vl) {
+        return;
     }
+
     this->set_selection_without_context(sel);
-
-    if (diff) {
-        auto new_top = std::max(0_vl, this->lv_selection - diff.value());
-        this->set_top(new_top);
-    }
-
-    if (this->lv_selection > 0 && this->lv_selection <= this->lv_top) {
-        this->set_top(this->lv_selection - 1_vl);
-    } else if (dim.first > this->lv_tail_space
-               && (this->lv_selection
-                   > (this->lv_top + (dim.first - 1_vl) - this->lv_tail_space)))
-    {
-        this->set_top(this->lv_selection + this->lv_tail_space
-                      - (dim.first - 1_vl));
-    }
+    this->update_top_from_selection();
 }
 
 vis_line_t
@@ -1168,16 +1327,9 @@ listview_curses::get_top_for_last_row()
 
     if (inner_height > 0) {
         auto last_line = inner_height - 1_vl;
-        unsigned long width;
-        vis_line_t height;
+        const auto layout = this->layout_for_row(last_line);
 
-        this->get_dimensions(height, width);
-        retval = last_line - this->rows_available(last_line, RD_UP) + 1_vl;
-        if (inner_height >= (height - this->lv_tail_space)
-            && (retval + this->lv_tail_space) < inner_height)
-        {
-            retval += this->lv_tail_space;
-        }
+        retval = last_line - vis_line_t(layout.lr_above_line_heights.size());
     }
 
     return retval;
@@ -1225,7 +1377,7 @@ listview_curses::set_left(int left)
 }
 
 size_t
-listview_curses::get_overlay_height(size_t total, vis_line_t view_height)
+listview_curses::get_overlay_height(size_t total, vis_line_t view_height) const
 {
     return std::min(total, static_cast<size_t>(2 * (view_height / 3)));
 }
