@@ -169,7 +169,10 @@ eval_with(logfile& lf, logfile::iterator ll)
                 continue;
             }
             if (strcmp(name, ":log_time_msecs") == 0) {
-                sqlite3_bind_int64(stmt, lpc + 1, ll->get_time_in_millis());
+                sqlite3_bind_int64(
+                    stmt,
+                    lpc + 1,
+                    ll->get_time<std::chrono::milliseconds>().count());
                 continue;
             }
             if (strcmp(name, ":log_format") == 0) {
@@ -358,7 +361,7 @@ eval_with(logfile& lf, logfile::iterator ll)
             switch (lv.lv_meta.lvm_kind) {
                 case value_kind_t::VALUE_NULL:
                     lmd.md_values[lv.lv_meta.lvm_name.to_string()]
-                        = json_null_t{};
+                        = null_value_t{};
                     break;
                 case value_kind_t::VALUE_BOOLEAN:
                     lmd.md_values[lv.lv_meta.lvm_name.to_string()]
