@@ -42,7 +42,7 @@
 #include "formats/logfmt/logfmt.parser.hh"
 #include "log_vtab_impl.hh"
 #include "ptimec.hh"
-#include "scn/scn.h"
+#include "scn/scan.h"
 #include "sql_util.hh"
 #include "yajlpp/yajlpp.hh"
 
@@ -70,7 +70,7 @@ public:
             return scan_match{1};
         }
 
-        return scan_no_match{""};
+        return scan_no_match{"not a piper capture"};
     }
 
     void annotate(logfile* lf,
@@ -80,7 +80,7 @@ public:
                   bool annotate_module) const override
     {
         auto lr = line_range{0, 0};
-        sa.emplace_back(lr, logline::L_TIMESTAMP.value());
+        sa.emplace_back(lr, L_TIMESTAMP.value());
         log_format::annotate(lf, line_number, sa, values, annotate_module);
     }
 
@@ -101,7 +101,7 @@ public:
             tmptm.et_flags |= ETF_ZONE_SET;
             tmptm.et_gmtoff
                 = lnav::local_time_to_info(
-                      date::local_seconds{std::chrono::seconds{ll.get_time()}})
+                      date::local_seconds{ll.get_time<std::chrono::seconds>()})
                       .first.offset.count();
             off_t zone_len = 0;
             ftime_z(zone_str, zone_len, sizeof(zone_str), tmptm);
@@ -234,11 +234,17 @@ public:
 
             if (!(this->lf_timestamp_flags
                   & (ETF_MILLIS_SET | ETF_MICROS_SET | ETF_NANOS_SET))
-                && !dst.empty() && dst.back().get_time() == log_tv.tv_sec
-                && dst.back().get_millis() != 0)
+                && !dst.empty()
+                && dst.back().get_time<std::chrono::seconds>().count()
+                    == log_tv.tv_sec
+                && dst.back()
+                        .get_subsecond_time<std::chrono::microseconds>()
+                        .count()
+                    != 0)
             {
                 auto log_ms
-                    = std::chrono::milliseconds(dst.back().get_millis());
+                    = dst.back()
+                          .get_subsecond_time<std::chrono::microseconds>();
 
                 log_time.et_nsec
                     = std::chrono::duration_cast<std::chrono::nanoseconds>(
@@ -278,7 +284,7 @@ public:
 
         auto ts_cap = md[fmt.pf_timestamp_index].value();
         auto lr = to_line_range(ts_cap.trim());
-        sa.emplace_back(lr, logline::L_TIMESTAMP.value());
+        sa.emplace_back(lr, L_TIMESTAMP.value());
 
         values.lvv_values.emplace_back(TS_META, line, lr);
         values.lvv_values.back().lv_meta.lvm_format = (log_format*) this;
@@ -300,7 +306,7 @@ public:
 
         lr.lr_start = 0;
         lr.lr_end = prefix_len;
-        sa.emplace_back(lr, logline::L_PREFIX.value());
+        sa.emplace_back(lr, L_PREFIX.value());
 
         lr.lr_start = prefix_len;
         lr.lr_end = line.length();
@@ -322,10 +328,15 @@ public:
         if (field_name == TS_META.lvm_name) {
             TS_META.lvm_user_hidden = val;
             return true;
-        } else if (field_name == LEVEL_META.lvm_name) {
+        }
+        if (field_name == LEVEL_META.lvm_name) {
             LEVEL_META.lvm_user_hidden = val;
             return true;
         }
+        if (field_name == OPID_META.lvm_name) {
+            OPID_META.lvm_user_hidden = val;
+            return true;
+        }
         return false;
     }
 
@@ -334,12 +345,14 @@ public:
         return {
             {TS_META.lvm_name, TS_META},
             {LEVEL_META.lvm_name, LEVEL_META},
+            {OPID_META.lvm_name, OPID_META},
         };
     }
 
 private:
     static logline_value_meta TS_META;
     static logline_value_meta LEVEL_META;
+    static logline_value_meta OPID_META;
 };
 
 logline_value_meta generic_log_format::TS_META{
@@ -347,10 +360,17 @@ logline_value_meta generic_log_format::TS_META{
     value_kind_t::VALUE_TEXT,
     logline_value_meta::table_column{2},
 };
+
 logline_value_meta generic_log_format::LEVEL_META{
     intern_string::lookup("log_level"),
     value_kind_t::VALUE_TEXT,
-    logline_value_meta::table_column{4},
+    logline_value_meta::table_column{3},
+};
+
+logline_value_meta generic_log_format::OPID_META{
+    intern_string::lookup("log_opid"),
+    value_kind_t::VALUE_TEXT,
+    logline_value_meta::internal_column{},
 };
 
 std::string
@@ -493,6 +513,7 @@ struct separated_string {
 
 class bro_log_format : public log_format {
 public:
+    static const intern_string_t TS;
     struct field_def {
         logline_value_meta fd_meta;
         logline_value_meta* fd_root_meta;
@@ -572,7 +593,6 @@ public:
     {
         static const intern_string_t STATUS_CODE
             = intern_string::lookup("bro_status_code");
-        static const intern_string_t TS = intern_string::lookup("bro_ts");
         static const intern_string_t UID = intern_string::lookup("bro_uid");
         static const intern_string_t ID_ORIG_H
             = intern_string::lookup("bro_id_orig_h");
@@ -630,7 +650,7 @@ public:
                         auto scan_float_res = scn::scan_value<double>(sv);
                         if (scan_float_res) {
                             this->lf_value_stats[fd.fd_numeric_index.value()]
-                                .add_value(scan_float_res.value());
+                                .add_value(scan_float_res->value());
                         }
                         break;
                     }
@@ -664,7 +684,7 @@ public:
             dst.emplace_back(li.li_file_range.fr_offset, tv, level, 0, opid);
             return scan_match{2000};
         }
-        return scan_no_match{};
+        return scan_no_match{"no header found"};
     }
 
     scan_result_t scan(logfile& lf,
@@ -694,7 +714,7 @@ public:
         if (dst.empty() || dst.size() > 20 || sbr.empty()
             || sbr.get_data()[0] == '#')
         {
-            return scan_no_match{};
+            return scan_no_match{"no header found"};
         }
 
         auto line_iter = dst.begin();
@@ -846,7 +866,7 @@ public:
         this->blf_format_name.clear();
         this->lf_value_stats.clear();
 
-        return scan_no_match{};
+        return scan_no_match{"no header found"};
     }
 
     void annotate(logfile* lf,
@@ -855,7 +875,6 @@ public:
                   logline_value_vector& values,
                   bool annotate_module) const override
     {
-        static const intern_string_t TS = intern_string::lookup("bro_ts");
         static const intern_string_t UID = intern_string::lookup("bro_uid");
 
         auto& sbr = values.lvv_sbr;
@@ -880,9 +899,9 @@ public:
             auto lr = line_range(sf.sf_begin, sf.sf_end);
 
             if (fd.fd_meta.lvm_name == TS) {
-                sa.emplace_back(lr, logline::L_TIMESTAMP.value());
+                sa.emplace_back(lr, L_TIMESTAMP.value());
             } else if (fd.fd_meta.lvm_name == UID) {
-                sa.emplace_back(lr, logline::L_OPID.value());
+                sa.emplace_back(lr, L_OPID.value());
             }
 
             if (lr.is_valid()) {
@@ -916,8 +935,12 @@ public:
         return retval;
     }
 
-    bool hide_field(const intern_string_t field_name, bool val) override
+    bool hide_field(intern_string_t field_name, bool val) override
     {
+        if (field_name == LOG_TIME_STR) {
+            field_name = TS;
+        }
+
         auto fd_iter = FIELD_META.find(field_name);
         if (fd_iter == FIELD_META.end()) {
             return false;
@@ -970,13 +993,13 @@ public:
         }
 
         void get_foreign_keys(
-            std::vector<std::string>& keys_inout) const override
+            std::unordered_set<std::string>& keys_inout) const override
         {
             this->log_vtab_impl::get_foreign_keys(keys_inout);
 
             for (const auto& fd : this->blt_format.blf_field_defs) {
                 if (fd.fd_meta.lvm_identifier || fd.fd_meta.lvm_foreign_key) {
-                    keys_inout.push_back(fd.fd_meta.lvm_name.to_string());
+                    keys_inout.emplace(fd.fd_meta.lvm_name.to_string());
                 }
             }
         }
@@ -1001,7 +1024,7 @@ public:
         std::shared_ptr<bro_log_table> retval = nullptr;
 
         auto& tables = get_tables();
-        auto iter = tables.find(this->blf_format_name);
+        const auto iter = tables.find(this->blf_format_name);
         if (iter == tables.end()) {
             retval = std::make_shared<bro_log_table>(*this);
             tables[this->blf_format_name] = retval;
@@ -1027,6 +1050,8 @@ public:
 std::unordered_map<const intern_string_t, logline_value_meta>
     bro_log_format::FIELD_META;
 
+const intern_string_t bro_log_format::TS = intern_string::lookup("bro_ts");
+
 struct ws_separated_string {
     const char* ss_str;
     size_t ss_len;
@@ -1126,6 +1151,9 @@ struct ws_separated_string {
 
 class w3c_log_format : public log_format {
 public:
+    static const intern_string_t F_DATE;
+    static const intern_string_t F_TIME;
+
     struct field_def {
         const intern_string_t fd_name;
         logline_value_meta fd_meta;
@@ -1222,12 +1250,10 @@ public:
                            const line_info& li,
                            shared_buffer_ref& sbr)
     {
-        static const intern_string_t F_DATE = intern_string::lookup("date");
         static const intern_string_t F_DATE_LOCAL
             = intern_string::lookup("date-local");
         static const intern_string_t F_DATE_UTC
             = intern_string::lookup("date-UTC");
-        static const intern_string_t F_TIME = intern_string::lookup("time");
         static const intern_string_t F_TIME_LOCAL
             = intern_string::lookup("time-local");
         static const intern_string_t F_TIME_UTC
@@ -1236,10 +1262,8 @@ public:
             = intern_string::lookup("sc-status");
 
         ws_separated_string ss(sbr.get_data(), sbr.length());
-        struct timeval date_tv {
-            0, 0
-        }, time_tv{0, 0};
-        struct exttm date_tm, time_tm;
+        timeval date_tv{0, 0}, time_tv{0, 0};
+        exttm date_tm, time_tm;
         bool found_date = false, found_time = false;
         log_level_t level = LEVEL_INFO;
 
@@ -1276,8 +1300,10 @@ public:
                         }
                     }
                 }
-                dst.emplace_back(
-                    li.li_file_range.fr_offset, 0, 0, LEVEL_IGNORE, 0);
+                dst.emplace_back(li.li_file_range.fr_offset,
+                                 std::chrono::microseconds{0},
+                                 LEVEL_IGNORE,
+                                 0);
                 return scan_match{2000};
             }
 
@@ -1315,7 +1341,7 @@ public:
 
                         if (scan_float_res) {
                             this->lf_value_stats[fd.fd_numeric_index.value()]
-                                .add_value(scan_float_res.value());
+                                .add_value(scan_float_res->value());
                         }
                         break;
                     }
@@ -1326,8 +1352,8 @@ public:
         }
 
         if (found_time) {
-            struct exttm tm = time_tm;
-            struct timeval tv;
+            exttm tm = time_tm;
+            timeval tv;
 
             if (found_date) {
                 tm.et_tm.tm_year = date_tm.et_tm.tm_year;
@@ -1347,7 +1373,7 @@ public:
             return scan_match{2000};
         }
 
-        return scan_no_match{};
+        return scan_no_match{"no header found"};
     }
 
     scan_result_t scan(logfile& lf,
@@ -1382,7 +1408,7 @@ public:
         if (dst.empty() || dst.size() > 20 || sbr.empty()
             || sbr.get_data()[0] == '#')
         {
-            return scan_no_match{};
+            return scan_no_match{"no header found"};
         }
 
         this->clear();
@@ -1518,7 +1544,7 @@ public:
         this->wlf_format_name.clear();
         this->lf_value_stats.clear();
 
-        return scan_no_match{};
+        return scan_no_match{"no header found"};
     }
 
     void annotate(logfile* lf,
@@ -1590,6 +1616,18 @@ public:
 
     bool hide_field(const intern_string_t field_name, bool val) override
     {
+        if (field_name == LOG_TIME_STR) {
+            auto date_iter = FIELD_META.find(F_DATE);
+            auto time_iter = FIELD_META.find(F_TIME);
+            if (date_iter == FIELD_META.end() || time_iter == FIELD_META.end())
+            {
+                return false;
+            }
+            date_iter->second.lvm_user_hidden = val;
+            time_iter->second.lvm_user_hidden = val;
+            return true;
+        }
+
         auto fd_iter = FIELD_META.find(field_name);
         if (fd_iter == FIELD_META.end()) {
             return false;
@@ -1649,13 +1687,13 @@ public:
         };
 
         void get_foreign_keys(
-            std::vector<std::string>& keys_inout) const override
+            std::unordered_set<std::string>& keys_inout) const override
         {
             this->log_vtab_impl::get_foreign_keys(keys_inout);
 
             for (const auto& fd : KNOWN_FIELDS) {
                 if (fd.fd_meta.lvm_identifier || fd.fd_meta.lvm_foreign_key) {
-                    keys_inout.push_back(fd.fd_meta.lvm_name.to_string());
+                    keys_inout.emplace(fd.fd_meta.lvm_name.to_string());
                 }
             }
         }
@@ -1680,7 +1718,7 @@ public:
         std::shared_ptr<w3c_log_table> retval = nullptr;
 
         auto& tables = get_tables();
-        auto iter = tables.find(this->wlf_format_name);
+        const auto iter = tables.find(this->wlf_format_name);
         if (iter == tables.end()) {
             retval = std::make_shared<w3c_log_table>(*this);
             tables[this->wlf_format_name] = retval;
@@ -1703,6 +1741,9 @@ public:
 std::unordered_map<const intern_string_t, logline_value_meta>
     w3c_log_format::FIELD_META;
 
+const intern_string_t w3c_log_format::F_DATE = intern_string::lookup("date");
+const intern_string_t w3c_log_format::F_TIME = intern_string::lookup("time");
+
 static size_t KNOWN_FIELD_INDEX = 0;
 const std::vector<w3c_log_format::field_def> w3c_log_format::KNOWN_FIELDS = {
     {
@@ -1844,10 +1885,8 @@ struct logfmt_pair_handler {
 
     date_time_scanner& lph_dt_scanner;
     bool lph_found_time{false};
-    struct exttm lph_time_tm {};
-    struct timeval lph_tv {
-        0, 0
-    };
+    struct exttm lph_time_tm{};
+    struct timeval lph_tv{0, 0};
     log_level_t lph_level{log_level_t::LEVEL_INFO};
     string_fragment lph_key_frag{""};
 };
@@ -1931,7 +1970,8 @@ public:
                             handle = yajl_alloc(&cb, nullptr, &lph);
                             cb.yajl_string = +[](void* ctx,
                                                  const unsigned char* str,
-                                                 size_t len) -> int {
+                                                 size_t len,
+                                                 yajl_string_props_t*) -> int {
                                 auto& lph = *((logfmt_pair_handler*) ctx);
                                 string_fragment value_frag{str, 0, (int) len};
 
@@ -2048,7 +2088,7 @@ public:
                         = line_range{value_frag.sf_begin, value_frag.sf_end};
 
                     if (kvp.first == "time" || kvp.first == "ts") {
-                        sa.emplace_back(value_lr, logline::L_TIMESTAMP.value());
+                        sa.emplace_back(value_lr, L_TIMESTAMP.value());
                     } else if (kvp.first == "level") {
                     } else if (kvp.first == "msg") {
                         sa.emplace_back(value_lr, SA_BODY.value());
