@@ -47,6 +47,7 @@
 #include "base/injector.hh"
 #include "base/snippet_highlighters.hh"
 #include "base/string_util.hh"
+#include "base/time_util.hh"
 #include "config.h"
 #include "file_options.hh"
 #include "hasher.hh"
@@ -54,23 +55,28 @@
 #include "log.watch.hh"
 #include "log_format.hh"
 #include "logfile.cfg.hh"
-#include "piper.looper.hh"
+#include "piper.header.hh"
 #include "yajlpp/yajlpp_def.hh"
 
 using namespace lnav::roles::literals;
 
 static auto intern_lifetime = intern_string::get_table_lifetime();
 
-static const size_t INDEX_RESERVE_INCREMENT = 1024;
+static constexpr size_t INDEX_RESERVE_INCREMENT = 1024;
 
-static const typed_json_path_container<lnav::gzip::header> file_header_handlers
-    = {
+static const typed_json_path_container<lnav::gzip::header>&
+get_file_header_handlers()
+{
+    static const typed_json_path_container<lnav::gzip::header> retval = {
         yajlpp::property_handler("name").for_field(&lnav::gzip::header::h_name),
         yajlpp::property_handler("mtime").for_field(
             &lnav::gzip::header::h_mtime),
         yajlpp::property_handler("comment").for_field(
             &lnav::gzip::header::h_comment),
-};
+    };
+
+    return retval;
+}
 
 Result<std::shared_ptr<logfile>, std::string>
 logfile::open(std::filesystem::path filename,
@@ -153,7 +159,8 @@ logfile::open(std::filesystem::path filename,
                 if (!gzhdr.empty()) {
                     lf->lf_embedded_metadata["net.zlib.gzip.header"] = {
                         text_format_t::TF_JSON,
-                        file_header_handlers.formatter_for(gzhdr)
+                        get_file_header_handlers()
+                            .formatter_for(gzhdr)
                             .with_config(yajl_gen_beautify, 1)
                             .to_string(),
                     };
@@ -210,15 +217,14 @@ logfile::open(std::filesystem::path filename,
     return Ok(lf);
 }
 
-logfile::
-logfile(std::filesystem::path filename, const logfile_open_options& loo)
+logfile::logfile(std::filesystem::path filename,
+                 const logfile_open_options& loo)
     : lf_filename(std::move(filename)), lf_options(loo)
 {
     this->lf_opids.writeAccess()->los_opid_ranges.reserve(64);
 }
 
-logfile::~
-logfile()
+logfile::~logfile()
 {
     log_info("destructing logfile: %s", this->lf_filename.c_str());
 }
@@ -296,8 +302,8 @@ logfile::exists() const
         && this->lf_stat.st_size <= st.st_size;
 }
 
-void
-logfile::reset_state()
+auto
+logfile::reset_state() -> void
 {
     this->clear_time_offset();
     this->lf_indexing = this->lf_options.loo_is_visible;
@@ -335,7 +341,7 @@ logfile::process_prefix(shared_buffer_ref& sbr,
 
     log_format::scan_result_t found = log_format::scan_no_match{};
     size_t prescan_size = this->lf_index.size();
-    time_t prescan_time = 0;
+    auto prescan_time = std::chrono::microseconds{0};
     bool retval = false;
 
     if (this->lf_options.loo_detect_format
@@ -590,14 +596,14 @@ logfile::process_prefix(shared_buffer_ref& sbr,
             require_lt(starting_index_size, this->lf_index.size());
             for (size_t lpc = 0; lpc < starting_index_size; lpc++) {
                 if (this->lf_format->lf_multiline) {
-                    this->lf_index[lpc].set_time(last_line.get_time());
-                    this->lf_index[lpc].set_millis(last_line.get_millis());
+                    this->lf_index[lpc].set_time(
+                        last_line.get_time<std::chrono::microseconds>());
                     if (this->lf_format->lf_structured) {
                         this->lf_index[lpc].set_ignore(true);
                     }
                 } else {
-                    this->lf_index[lpc].set_time(last_line.get_time());
-                    this->lf_index[lpc].set_millis(last_line.get_millis());
+                    this->lf_index[lpc].set_time(
+                        last_line.get_time<std::chrono::microseconds>());
                     this->lf_index[lpc].set_level(LEVEL_INVALID);
                 }
                 retval = true;
@@ -607,7 +613,8 @@ logfile::process_prefix(shared_buffer_ref& sbr,
         }
     } else if (this->lf_format.get() != nullptr) {
         if (!this->lf_index.empty()) {
-            prescan_time = this->lf_index[prescan_size - 1].get_time();
+            prescan_time = this->lf_index[prescan_size - 1]
+                               .get_time<std::chrono::microseconds>();
         }
         /* We've locked onto a format, just use that scanner. */
         found = this->lf_format->scan(*this, this->lf_index, li, sbr, sbc);
@@ -621,9 +628,20 @@ logfile::process_prefix(shared_buffer_ref& sbr,
                                     && li.li_utf8_scan_result.is_valid());
             last_line.set_has_ansi(last_line.has_ansi()
                                    || li.li_utf8_scan_result.usr_has_ansi);
+            if (last_line.get_msg_level() == LEVEL_INVALID) {
+                if (this->lf_invalid_lines.ili_lines.size()
+                    < invalid_line_info::MAX_INVALID_LINES)
+                {
+                    this->lf_invalid_lines.ili_lines.push_back(
+                        this->lf_index.size() - 1);
+                }
+                this->lf_invalid_lines.ili_total += 1;
+            }
         }
         if (prescan_size > 0 && this->lf_index.size() >= prescan_size
-            && prescan_time != this->lf_index[prescan_size - 1].get_time())
+            && prescan_time
+                != this->lf_index[prescan_size - 1]
+                       .get_time<std::chrono::microseconds>())
         {
             retval = true;
         }
@@ -640,8 +658,9 @@ logfile::process_prefix(shared_buffer_ref& sbr,
                         auto& line_to_update = this->lf_index[lpc];
 
                         line_to_update.set_time_skew(true);
-                        line_to_update.set_time(second_to_last.get_time());
-                        line_to_update.set_millis(second_to_last.get_millis());
+                        line_to_update.set_time(
+                            second_to_last
+                                .get_time<std::chrono::microseconds>());
                     }
                 } else {
                     retval = true;
@@ -650,16 +669,13 @@ logfile::process_prefix(shared_buffer_ref& sbr,
         }
     } else if (found.is<log_format::scan_no_match>()) {
         log_level_t last_level = LEVEL_UNKNOWN;
-        time_t last_time = this->lf_index_time;
-        short last_millis = 0;
+        auto last_time = this->lf_index_time;
         uint8_t last_mod = 0, last_opid = 0;
 
         if (this->lf_format == nullptr && li.li_timestamp.tv_sec != 0) {
-            last_time = li.li_timestamp.tv_sec;
-            last_millis
-                = std::chrono::duration_cast<std::chrono::milliseconds>(
-                      std::chrono::microseconds(li.li_timestamp.tv_usec))
-                      .count();
+            last_time = std::chrono::duration_cast<std::chrono::microseconds>(
+                            std::chrono::seconds{li.li_timestamp.tv_sec})
+                + std::chrono::microseconds(li.li_timestamp.tv_usec);
             last_level = li.li_level;
         } else if (!this->lf_index.empty()) {
             const auto& ll = this->lf_index.back();
@@ -668,8 +684,7 @@ logfile::process_prefix(shared_buffer_ref& sbr,
              * Assume this line is part of the previous one(s) and copy the
              * metadata over.
              */
-            last_time = ll.get_time();
-            last_millis = ll.get_millis();
+            last_time = ll.get_time<std::chrono::microseconds>();
             if (this->lf_format.get() != nullptr) {
                 last_level = (log_level_t) (ll.get_level_and_flags()
                                             | LEVEL_CONTINUED);
@@ -679,7 +694,6 @@ logfile::process_prefix(shared_buffer_ref& sbr,
         }
         this->lf_index.emplace_back(li.li_file_range.fr_offset,
                                     last_time,
-                                    last_millis,
                                     last_level,
                                     last_mod,
                                     last_opid);
@@ -839,7 +853,7 @@ logfile::rebuild_index(std::optional<ui_clock::time_point> deadline)
                 auto check_line_off = last_line->get_offset();
                 auto last_length_res
                     = this->message_byte_length(last_line, false);
-                log_debug("flushing at %d", check_line_off);
+                log_debug("flushing at %" PRIu64, check_line_off);
                 this->lf_line_buffer.flush_at(check_line_off);
 
                 auto read_result = this->lf_line_buffer.read_range({
@@ -935,8 +949,7 @@ logfile::rebuild_index(std::optional<ui_clock::time_point> deadline)
                 this->lf_notes.writeAccess()->emplace(note_type::not_utf,
                                                       note_um);
                 if (this->lf_logfile_observer != nullptr) {
-                    this->lf_logfile_observer->logfile_indexing(
-                        this->shared_from_this(), 0, 0);
+                    this->lf_logfile_observer->logfile_indexing(this, 0, 0);
                 }
                 break;
             }
@@ -987,6 +1000,17 @@ logfile::rebuild_index(std::optional<ui_clock::time_point> deadline)
                           .unwrapOr(text_format_t::TF_UNKNOWN);
                 log_debug("setting text format to %s",
                           fmt::to_string(this->lf_text_format).c_str());
+                switch (this->lf_text_format) {
+                    case text_format_t::TF_DIFF:
+                    case text_format_t::TF_MAN:
+                    case text_format_t::TF_MARKDOWN:
+                        log_debug(
+                            "  file is text, disabling log format detection");
+                        this->lf_options.loo_detect_format = false;
+                        break;
+                    default:
+                        break;
+                }
             }
             if (!li.li_utf8_scan_result.is_valid()
                 && this->lf_text_format != text_format_t::TF_MARKDOWN
@@ -1034,7 +1058,7 @@ logfile::rebuild_index(std::optional<ui_clock::time_point> deadline)
 
             if (this->lf_logfile_observer != nullptr) {
                 auto indexing_res = this->lf_logfile_observer->logfile_indexing(
-                    this->shared_from_this(),
+                    this,
                     this->lf_line_buffer.get_read_offset(
                         li.li_file_range.next_offset()),
                     st.st_size);
@@ -1090,7 +1114,7 @@ logfile::rebuild_index(std::optional<ui_clock::time_point> deadline)
                 }
 
                 for (const auto& pd : this->lf_applicable_partitioners) {
-                    static thread_local auto part_md
+                    thread_local auto part_md
                         = lnav::pcre2pp::match_data::unitialized();
 
                     auto curr_ll = this->end() - 1;
@@ -1150,8 +1174,7 @@ logfile::rebuild_index(std::optional<ui_clock::time_point> deadline)
             this->lf_notes.writeAccess()->emplace(note_type::indexing_disabled,
                                                   note_um);
             if (this->lf_logfile_observer != nullptr) {
-                this->lf_logfile_observer->logfile_indexing(
-                    this->shared_from_this(), 0, 0);
+                this->lf_logfile_observer->logfile_indexing(this, 0, 0);
             }
         }
 
@@ -1211,14 +1234,22 @@ logfile::rebuild_index(std::optional<ui_clock::time_point> deadline)
         } else {
             retval = rebuild_result_t::NEW_LINES;
         }
+
+        {
+            auto est_rem = this->estimated_remaining_lines();
+            if (est_rem > 0) {
+                this->lf_index.reserve(this->lf_index.size() + est_rem);
+            }
+        }
     } else if (this->lf_sort_needed) {
         retval = rebuild_result_t::NEW_ORDER;
         this->lf_sort_needed = false;
     }
 
-    this->lf_index_time = this->lf_line_buffer.get_file_time();
-    if (!this->lf_index_time) {
-        this->lf_index_time = st.st_mtime;
+    this->lf_index_time
+        = std::chrono::seconds{this->lf_line_buffer.get_file_time()};
+    if (this->lf_index_time.count() == 0) {
+        this->lf_index_time = std::chrono::seconds{st.st_mtime};
     }
 
     if (this->lf_out_of_time_order_count) {
@@ -1292,9 +1323,9 @@ logfile::read_range(const file_range& fr)
 }
 
 void
-logfile::read_full_message(logfile::const_iterator ll,
+logfile::read_full_message(const_iterator ll,
                            shared_buffer_ref& msg_out,
-                           int max_lines)
+                           line_buffer::scan_direction dir)
 {
     require(ll->get_sub_offset() == 0);
 
@@ -1309,7 +1340,7 @@ logfile::read_full_message(logfile::const_iterator ll,
         if (range_for_line.fr_size > line_buffer::MAX_LINE_BUFFER_SIZE) {
             range_for_line.fr_size = line_buffer::MAX_LINE_BUFFER_SIZE;
         }
-        auto read_result = this->lf_line_buffer.read_range(range_for_line);
+        auto read_result = this->lf_line_buffer.read_range(range_for_line, dir);
 
         if (read_result.isErr()) {
             auto errmsg = read_result.unwrapErr();
@@ -1352,7 +1383,7 @@ logfile::reobserve_from(iterator iter)
 
         if (this->lf_logfile_observer != nullptr) {
             auto indexing_res = this->lf_logfile_observer->logfile_indexing(
-                this->shared_from_this(), offset, this->size());
+                this, offset, this->size());
             if (indexing_res == logfile_observer::indexing_result::BREAK) {
                 break;
             }
@@ -1370,7 +1401,7 @@ logfile::reobserve_from(iterator iter)
     }
     if (this->lf_logfile_observer != nullptr) {
         this->lf_logfile_observer->logfile_indexing(
-            this->shared_from_this(), this->size(), this->size());
+            this, this->size(), this->size());
         this->lf_logline_observer->logline_eof(*this);
     }
 }
@@ -1479,7 +1510,11 @@ logfile::mark_as_duplicate(const std::string& name)
 void
 logfile::adjust_content_time(int line, const timeval& tv, bool abs_offset)
 {
-    struct timeval old_time = this->lf_time_offset;
+    if (this->lf_time_offset == tv) {
+        return;
+    }
+
+    auto old_time = this->lf_time_offset;
 
     this->lf_time_offset_line = line;
     if (abs_offset) {
@@ -1488,7 +1523,7 @@ logfile::adjust_content_time(int line, const timeval& tv, bool abs_offset)
         timeradd(&old_time, &tv, &this->lf_time_offset);
     }
     for (auto& iter : *this) {
-        struct timeval curr, diff, new_time;
+        timeval curr, diff, new_time;
 
         curr = iter.get_timeval();
         timersub(&curr, &old_time, &diff);
@@ -1496,6 +1531,7 @@ logfile::adjust_content_time(int line, const timeval& tv, bool abs_offset)
         iter.set_time(new_time);
     }
     this->lf_sort_needed = true;
+    this->lf_index_generation += 1;
 }
 
 void
@@ -1666,3 +1702,19 @@ logfile::clear_logline_opid(uint32_t line_number)
         this->lf_invalidated_opids.insert(opid_sf);
     }
 }
+
+size_t
+logfile::estimated_remaining_lines() const
+{
+    if (this->lf_index.empty() || this->is_compressed()) {
+        return 10;
+    }
+
+    const auto bytes_per_line = this->lf_index_size / this->lf_index.size();
+    if (this->lf_index_size > this->lf_stat.st_size) {
+        return 0;
+    }
+    const auto remaining_bytes = this->lf_stat.st_size - this->lf_index_size;
+
+    return remaining_bytes / bytes_per_line;
+}
