@@ -34,6 +34,7 @@
 #include "base/itertools.hh"
 #include "base/lnav_log.hh"
 #include "base/map_util.hh"
+#include "base/types.hh"
 #include "document.sections.hh"
 #include "pcrepp/pcre2pp.hh"
 #include "pugixml/pugixml.hpp"
@@ -101,6 +102,11 @@ md2attr_line::flush_footnotes()
 Result<void, std::string>
 md2attr_line::enter_block(const md4cpp::event_handler::block& bl)
 {
+    if (this->ml_source_path) {
+        log_trace("enter_block %s",
+                  mapbox::util::apply_visitor(type_visitor(), bl));
+    }
+
     if (this->ml_list_stack.empty()
         && (bl.is<MD_BLOCK_H_DETAIL*>() || bl.is<block_hr>()
             || bl.is<block_p>()))
@@ -130,6 +136,11 @@ md2attr_line::enter_block(const md4cpp::event_handler::block& bl)
 Result<void, std::string>
 md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
 {
+    if (this->ml_source_path) {
+        log_trace("leave_block %s",
+                  mapbox::util::apply_visitor(type_visitor(), bl));
+    }
+
     auto block_text = std::move(this->ml_blocks.back());
     this->ml_blocks.pop_back();
 
@@ -227,14 +238,13 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
         if (tf_opt) {
             static const auto highlighters = get_highlight_map();
 
-            lnav::document::discover_structure(
-                block_text, line_range{0, -1}, tf_opt.value());
+            lnav::document::discover(block_text)
+                .with_text_format(tf_opt.value())
+                .perform();
             for (const auto& hl_pair : highlighters) {
                 const auto& hl = hl_pair.second;
 
-                if (!hl.h_text_formats.empty()
-                    && hl.h_text_formats.count(tf_opt.value()) == 0)
-                {
+                if (!hl.applies_to_format(tf_opt.value())) {
                     continue;
                 }
                 hl.annotate(block_text, 0);
@@ -289,6 +299,9 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
                     new_block_text.append(line).append("\n");
                 }
             }
+            if (!cmd_block.empty()) {
+                new_block_text.append(cmd_block);
+            }
             block_text = new_block_text.move();
         }
 
@@ -407,7 +420,7 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
                 if (lpc >= row.r_columns.size()) {
                     continue;
                 }
-                auto col_len = row.r_columns[lpc].column_width();
+                auto col_len = row.r_columns[lpc].c_contents.column_width();
                 if (col_len > max_col_sizes[lpc]) {
                     max_col_sizes[lpc] = col_len;
                 }
@@ -426,7 +439,7 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
 
             attr_line_t td_block;
             td_block.append(tab.t_headers[lpc], &tws);
-            cells.emplace_back(td_block.rtrim().split_lines());
+            cells.emplace_back(MD_ALIGN_CENTER, td_block.rtrim().split_lines());
             if (cells.back().cl_lines.size() > max_cell_lines) {
                 max_cell_lines = cells.back().cl_lines.size();
             }
@@ -449,19 +462,25 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
                     repeat("\u2550", full_width + col_sizes.size())))
                 .append("\n");
         }
+        size_t row_index = 0;
         for (const auto& row : tab.t_rows) {
             cells.clear();
             max_cell_lines = 0;
-            for (size_t lpc = 0; lpc < row.r_columns.size(); lpc++) {
-                tws.with_width(col_sizes[lpc]);
+            for (const auto& [col_index, cell] :
+                 lnav::itertools::enumerate(row.r_columns))
+            {
+                tws.with_width(col_sizes[col_index]);
 
                 attr_line_t td_block;
-                td_block.append(row.r_columns[lpc], &tws);
-                cells.emplace_back(td_block.rtrim().split_lines());
+                td_block.append(cell.c_contents, &tws);
+                cells.emplace_back(cell.c_align,
+                                   td_block.rtrim().split_lines());
                 if (cells.back().cl_lines.size() > max_cell_lines) {
                     max_cell_lines = cells.back().cl_lines.size();
                 }
             }
+            auto alt_row_index = row_index % 4;
+            auto line_lr = line_range{(int) block_text.al_string.size(), 0};
             for (size_t line_index = 0; line_index < max_cell_lines;
                  line_index++)
             {
@@ -469,13 +488,35 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
                 for (const auto& cell : cells) {
                     block_text.append(" ");
                     if (line_index < cell.cl_lines.size()) {
-                        block_text.append(cell.cl_lines[line_index]);
-                        if (col < col_sizes.size() - 1) {
-                            block_text.append(
-                                col_sizes[col]
-                                    - cell.cl_lines[line_index].column_width(),
-                                ' ');
+                        auto padding = size_t{0};
+
+                        if (col < col_sizes.size()) {
+                            if (col_sizes[col]
+                                > cell.cl_lines[line_index].column_width())
+                            {
+                                padding = col_sizes[col]
+                                    - cell.cl_lines[line_index].column_width();
+                            }
                         }
+
+                        auto lpadding = size_t{0};
+                        auto rpadding = size_t{0};
+                        switch (cell.cl_align) {
+                            case MD_ALIGN_DEFAULT:
+                            case MD_ALIGN_LEFT:
+                                rpadding = padding;
+                                break;
+                            case MD_ALIGN_CENTER:
+                                lpadding = padding / 2;
+                                rpadding = padding - lpadding;
+                                break;
+                            case MD_ALIGN_RIGHT:
+                                lpadding = padding;
+                                break;
+                        }
+                        block_text.append(lpadding, ' ');
+                        block_text.append(cell.cl_lines[line_index]);
+                        block_text.append(rpadding, ' ');
                     } else if (col < col_sizes.size() - 1) {
                         block_text.append(col_sizes[col], ' ');
                     }
@@ -483,6 +524,14 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
                 }
                 block_text.append("\n");
             }
+            if (alt_row_index == 2 || alt_row_index == 3) {
+                line_lr.lr_end = block_text.al_string.size();
+                block_text.al_attrs.emplace_back(
+                    line_lr, VC_ROLE.value(role_t::VCR_ALT_ROW));
+            }
+            if (max_cell_lines > 0) {
+                row_index += 1;
+            }
         }
         if (!block_text.empty()) {
             block_text.with_attr_for_all(SA_PREFORMATTED.value());
@@ -491,7 +540,9 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
     } else if (bl.is<block_th>()) {
         this->ml_tables.back().t_headers.push_back(block_text);
     } else if (bl.is<MD_BLOCK_TD_DETAIL*>()) {
-        this->ml_tables.back().t_rows.back().r_columns.push_back(block_text);
+        auto td_detail = bl.get<MD_BLOCK_TD_DETAIL*>();
+        this->ml_tables.back().t_rows.back().r_columns.emplace_back(
+            td_detail->align, block_text);
     } else {
         if (bl.is<block_html>()) {
             if (startswith(block_text.get_string(), "<!--")) {
@@ -515,6 +566,11 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
 Result<void, std::string>
 md2attr_line::enter_span(const md4cpp::event_handler::span& sp)
 {
+    if (this->ml_source_path) {
+        log_trace("enter_span %s",
+                  mapbox::util::apply_visitor(type_visitor(), sp));
+    }
+
     auto& last_block = this->ml_blocks.back();
     this->ml_span_starts.push_back(last_block.length());
     if (sp.is<span_code>()) {
@@ -529,6 +585,11 @@ md2attr_line::enter_span(const md4cpp::event_handler::span& sp)
 Result<void, std::string>
 md2attr_line::leave_span(const md4cpp::event_handler::span& sp)
 {
+    if (this->ml_source_path) {
+        log_trace("leave_span %s",
+                  mapbox::util::apply_visitor(type_visitor(), sp));
+    }
+
     auto& last_block = this->ml_blocks.back();
     if (sp.is<span_code>()) {
         this->ml_code_depth -= 1;
@@ -550,12 +611,10 @@ md2attr_line::leave_span(const md4cpp::event_handler::span& sp)
             static_cast<int>(this->ml_span_starts.back()),
             static_cast<int>(last_block.length()),
         };
-#if defined(A_ITALIC)
         last_block.with_attr({
             lr,
-            VC_STYLE.value(text_attrs{(int32_t) A_ITALIC}),
+            VC_STYLE.value(text_attrs{text_attrs::with_italic()}),
         });
-#endif
     } else if (sp.is<span_strong>()) {
         line_range lr{
             static_cast<int>(this->ml_span_starts.back()),
@@ -563,7 +622,7 @@ md2attr_line::leave_span(const md4cpp::event_handler::span& sp)
         };
         last_block.with_attr({
             lr,
-            VC_STYLE.value(text_attrs{A_BOLD}),
+            VC_STYLE.value(text_attrs::with_bold()),
         });
     } else if (sp.is<span_u>()) {
         line_range lr{
@@ -572,7 +631,16 @@ md2attr_line::leave_span(const md4cpp::event_handler::span& sp)
         };
         last_block.with_attr({
             lr,
-            VC_STYLE.value(text_attrs{A_UNDERLINE}),
+            VC_STYLE.value(text_attrs::with_underline()),
+        });
+    } else if (sp.is<span_del>()) {
+        auto lr = line_range{
+            static_cast<int>(this->ml_span_starts.back()),
+            static_cast<int>(last_block.length()),
+        };
+        last_block.with_attr({
+            lr,
+            VC_STYLE.value(text_attrs::with_struck()),
         });
     } else if (sp.is<MD_SPAN_A_DETAIL*>()) {
         const auto* a_detail = sp.get<MD_SPAN_A_DETAIL*>();
@@ -637,12 +705,12 @@ right_border_string(border_line_width width)
 static attr_line_t
 span_style_border(border_side side, const string_fragment& value)
 {
-    static const auto NAME_THIN = string_fragment::from_const("thin");
-    static const auto NAME_MEDIUM = string_fragment::from_const("medium");
-    static const auto NAME_THICK = string_fragment::from_const("thick");
-    static const auto NAME_SOLID = string_fragment::from_const("solid");
-    static const auto NAME_DASHED = string_fragment::from_const("dashed");
-    static const auto NAME_DOTTED = string_fragment::from_const("dotted");
+    static constexpr auto NAME_THIN = string_fragment::from_const("thin");
+    static constexpr auto NAME_MEDIUM = string_fragment::from_const("medium");
+    static constexpr auto NAME_THICK = string_fragment::from_const("thick");
+    static constexpr auto NAME_SOLID = string_fragment::from_const("solid");
+    static constexpr auto NAME_DASHED = string_fragment::from_const("dashed");
+    static constexpr auto NAME_DOTTED = string_fragment::from_const("dotted");
     static const auto& vc = view_colors::singleton();
 
     text_attrs border_attrs;
@@ -703,6 +771,10 @@ md2attr_line::to_attr_line(const pugi::xml_node& doc)
         = string_fragment::from_const("border-right");
     static const auto& vc = view_colors::singleton();
 
+    if (this->ml_source_path) {
+        log_trace("converting HTML to attr_line");
+    }
+
     attr_line_t retval;
     if (doc.children().empty()) {
         retval.append(doc.text().get());
@@ -822,7 +894,7 @@ md2attr_line::to_attr_line(const pugi::xml_node& doc)
                             }
                         } else if (key == NAME_FONT_WEIGHT) {
                             if (value == "bold" || value == "bolder") {
-                                ta.ta_attrs |= A_BOLD;
+                                ta |= text_attrs::style::bold;
                             }
                         } else if (key == NAME_TEXT_DECO) {
                             auto deco_sf = value;
@@ -833,7 +905,7 @@ md2attr_line::to_attr_line(const pugi::xml_node& doc)
 
                                 if (deco_split_res.first.trim() == "underline")
                                 {
-                                    ta.ta_attrs |= A_UNDERLINE;
+                                    ta |= text_attrs::style::underline;
                                 }
 
                                 deco_sf = deco_split_res.second;
@@ -917,12 +989,18 @@ md2attr_line::text(MD_TEXTTYPE tt, const string_fragment& sf)
             };
             struct empty_tag {};
 
-            mapbox::util::variant<open_tag, close_tag, empty_tag> tag{
-                mapbox::util::no_init{}};
+            using html_tag_t
+                = mapbox::util::variant<open_tag, close_tag, empty_tag>;
 
-            if (sf.startswith("</")) {
+            html_tag_t tag{mapbox::util::no_init{}};
+
+            auto lbracket = sf.find('<');
+            if (!lbracket) {
+            } else if (lbracket && lbracket.value() + 1 < sf.length()
+                       && sf[lbracket.value() + 1] == '/')
+            {
                 tag = close_tag{
-                    sf.substr(2)
+                    sf.substr(lbracket.value() + 2)
                         .split_when(string_fragment::tag1{'>'})
                         .first.to_string(),
                 };
@@ -950,9 +1028,14 @@ md2attr_line::text(MD_TEXTTYPE tt, const string_fragment& sf)
                     },
                     [this, &last_block](const close_tag& ct) {
                         if (this->ml_html_starts.empty()) {
+                            log_warning("closing tag %s with no open tag",
+                                        ct.ct_name.c_str());
                             return;
                         }
                         if (this->ml_html_starts.back().first != ct.ct_name) {
+                            log_warning(
+                                "closing tag %s with no matching open tag",
+                                ct.ct_name.c_str());
                             return;
                         }
 
@@ -1016,7 +1099,6 @@ md2attr_line::text(MD_TEXTTYPE tt, const string_fragment& sf)
             }
 
             std::string span_text;
-
             auto loop_res = REPL_RE.capture_from(sf).for_each(
                 [&span_text](const lnav::pcre2pp::match_data& md) {
                     span_text += md.leading();
@@ -1048,7 +1130,8 @@ md2attr_line::text(MD_TEXTTYPE tt, const string_fragment& sf)
             text_wrap_settings tws
                 = {0, this->ml_blocks.size() == 1 ? 70 : 10000};
 
-            last_block.append(span_text, &tws);
+            auto span_al = attr_line_t::from_ansi_str(span_text);
+            last_block.append(span_al, &tws);
             break;
         }
     }
@@ -1065,7 +1148,7 @@ md2attr_line::append_url_footnote(std::string href_str)
             (int) this->ml_span_starts.back(),
             (int) last_block.length(),
         },
-        VC_STYLE.value(text_attrs{A_UNDERLINE}),
+        VC_STYLE.value(text_attrs::with_underline()),
     });
     if (is_internal) {
         return href_str;
