@@ -32,17 +32,20 @@
 #include "base/is_utf8.hh"
 #include "base/lnav_log.hh"
 #include "emojis-json.h"
+#include "pcrepp/pcre2pp.hh"
 #include "xml-entities-json.h"
 #include "yajlpp/yajlpp_def.hh"
 
 namespace md4cpp {
 
-static const typed_json_path_container<xml_entity> xml_entity_handlers = {
-    yajlpp::property_handler("characters").for_field(&xml_entity::xe_chars),
-};
+static const typed_json_path_container<xml_entity_map>&
+get_xml_entity_map_handlers()
+{
+    static const typed_json_path_container<xml_entity> xml_entity_handlers = {
+        yajlpp::property_handler("characters").for_field(&xml_entity::xe_chars),
+    };
 
-static const typed_json_path_container<xml_entity_map> xml_entity_map_handlers
-    = {
+    static const typed_json_path_container<xml_entity_map> retval = {
         yajlpp::pattern_property_handler("(?<var_name>\\&\\w+;?)")
             .with_synopsis("<name>")
             .with_path_provider<xml_entity_map>(
@@ -58,27 +61,38 @@ static const typed_json_path_container<xml_entity_map> xml_entity_map_handlers
                     return &xem->xem_entities[entity_name];
                 })
             .with_children(xml_entity_handlers),
-};
+    };
 
-static const typed_json_path_container<emoji> emoji_handlers = {
-    yajlpp::property_handler("emoji").for_field(&emoji::e_value),
-    yajlpp::property_handler("shortname").for_field(&emoji::e_shortname),
-};
+    return retval;
+}
 
-static const typed_json_path_container<emoji_map> emoji_map_handlers = {
-    yajlpp::property_handler("emojis#")
-        .for_field(&emoji_map::em_emojis)
-        .with_children(emoji_handlers),
-};
+static const typed_json_path_container<emoji_map>&
+get_emoji_map_handlers()
+{
+    static const typed_json_path_container<emoji> emoji_handlers = {
+        yajlpp::property_handler("emoji").for_field(&emoji::e_value),
+        yajlpp::property_handler("shortname").for_field(&emoji::e_shortname),
+    };
+
+    static const typed_json_path_container<emoji_map> retval = {
+        yajlpp::property_handler("emojis#")
+            .for_field(&emoji_map::em_emojis)
+            .with_children(emoji_handlers),
+    };
+
+    return retval;
+}
 
 static xml_entity_map
 load_xml_entity_map()
 {
     static const intern_string_t name
         = intern_string::lookup(xml_entities_json.get_name());
-    auto parse_res
-        = xml_entity_map_handlers.parser_for(name).with_ignore_unused(true).of(
-            xml_entities_json.to_string_fragment());
+    auto sfp = xml_entities_json.to_string_fragment_producer();
+    auto parse_res = get_xml_entity_map_handlers()
+                         .parser_for(name)
+                         .with_ignore_unused(true)
+                         .of(*sfp);
 
     assert(parse_res.isOk());
 
@@ -98,9 +112,10 @@ load_emoji_map()
 {
     static const intern_string_t name
         = intern_string::lookup(emojis_json.get_name());
+    auto sfp = emojis_json.to_string_fragment_producer();
     auto parse_res
-        = emoji_map_handlers.parser_for(name).with_ignore_unused(true).of(
-            emojis_json.to_string_fragment());
+        = get_emoji_map_handlers().parser_for(name).with_ignore_unused(true).of(
+            *sfp);
 
     assert(parse_res.isOk());
 
@@ -121,12 +136,12 @@ get_emoji_map()
 }
 
 std::string
-escape_html(const std::string& content)
+escape_html(string_fragment content)
 {
     std::string retval;
 
-    retval.reserve(content.size());
-    for (auto ch : content) {
+    retval.reserve(content.length());
+    for (const auto ch : content) {
         switch (ch) {
             case '"':
                 retval.append("&quot;");
@@ -283,6 +298,7 @@ static int
 md4cpp_text(MD_TEXTTYPE type, const MD_CHAR* text, MD_SIZE size, void* userdata)
 {
     auto* pu = static_cast<parse_userdata*>(userdata);
+
     auto text_res = pu->pu_handler.text(type, string_fragment(text, 0, size));
     if (text_res.isErr()) {
         pu->pu_error_msg = text_res.unwrapErr();
@@ -308,7 +324,8 @@ parse(const string_fragment& sf, event_handler& eh)
     auto pu = parse_userdata{eh};
 
     parser.abi_version = 0;
-    parser.flags = (MD_DIALECT_GITHUB | MD_FLAG_UNDERLINE)
+    parser.flags
+        = (MD_DIALECT_GITHUB | MD_FLAG_UNDERLINE | MD_FLAG_STRIKETHROUGH)
         & ~(MD_FLAG_PERMISSIVEAUTOLINKS);
     parser.enter_block = md4cpp_enter_block;
     parser.leave_block = md4cpp_leave_block;
