@@ -64,7 +64,7 @@ using namespace lnav::roles::literals;
 #define ANSI_REV(msg) ANSI_CSI "7m" msg ANSI_NORM
 #define ANSI_STR(msg) ANSI_CSI "32m" msg ANSI_NORM
 
-const char *RE_HELP =
+const char * const RE_HELP =
     " "  ANSI_RE(".") "   Any character    "
     " "     "a" ANSI_RE("|") "b   a or b        "
     " " ANSI_RE("(?-i)") "   Case-sensitive search\n"
@@ -86,7 +86,7 @@ const char *RE_HELP =
     " " ANSI_RE("[") "ab" ANSI_RE("-") "d" ANSI_RE("]") "  Any of a, b, c, or d"
 ;
 
-const char *RE_EXAMPLE =
+const char * const RE_EXAMPLE =
     ANSI_UNDERLINE("Examples") "\n"
     "  abc" ANSI_RE("*") "       matches  "
     ANSI_STR("'ab'") ", " ANSI_STR("'abc'") ", " ANSI_STR("'abccc'") "\n"
@@ -99,19 +99,19 @@ const char *RE_EXAMPLE =
     "  " ANSI_RE("(?-i)") "ABC   matches  " ANSI_STR("'ABC'") " and " ANSI_UNDERLINE("not") " " ANSI_STR("'abc'")
 ;
 
-const char* CMD_HELP =
+const char* const CMD_HELP =
     " " ANSI_KW(":goto") "              Go to a line #, timestamp, etc...\n"
     " " ANSI_KW(":filter-out") "        Filter out lines that match a pattern\n"
     " " ANSI_KW(":hide-lines-before") " Hide lines before a timestamp\n"
     " " ANSI_KW(":open") "              Open another file/directory\n";
 
-const char* CMD_EXAMPLE =
+const char* const CMD_EXAMPLE =
     ANSI_UNDERLINE("Examples") "\n"
     "  " ANSI_KW(":goto") " 123\n"
     "  " ANSI_KW(":filter-out") " spam\n"
     "  " ANSI_KW(":hide-lines-before") " here\n";
 
-const char *SQL_HELP =
+const char * const SQL_HELP =
     " " ANSI_KW("SELECT") "  Select rows from a table      "
     " " ANSI_KW("DELETE") "  Delete rows from a table\n"
     " " ANSI_KW("INSERT") "  Insert rows into a table      "
@@ -122,14 +122,14 @@ const char *SQL_HELP =
     " " ANSI_KW("DETACH") "  Detach a SQLite database"
 ;
 
-const char *SQL_EXAMPLE =
+const char * const SQL_EXAMPLE =
     ANSI_UNDERLINE("Examples") "\n"
     "  SELECT * FROM %s WHERE log_level >= 'warning' LIMIT 10\n"
     "  UPDATE %s SET log_mark = 1 WHERE log_line = log_top_line()\n"
     "  SELECT * FROM logline LIMIT 10"
 ;
 
-const char *PRQL_HELP =
+const char * const PRQL_HELP =
     " " ANSI_KW("from") "    Specify a data source       "
     " " ANSI_KW("derive") "     Derive one or more columns\n"
     " " ANSI_KW("select") "  Select one or more columns  "
@@ -138,13 +138,13 @@ const char *PRQL_HELP =
     " " ANSI_KW("filter") "     Pick rows based on their values\n"
     ;
 
-const char *PRQL_EXAMPLE =
+const char * const PRQL_EXAMPLE =
     ANSI_UNDERLINE("Examples") "\n"
         "  from %s | stats.count_by { log_level }\n"
         "  from %s | filter log_line == lnav.view.top_line\n"
     ;
 
-static const char* LNAV_CMD_PROMPT = "Enter an lnav command: " ABORT_MSG;
+static const char* const LNAV_CMD_PROMPT = "Enter an lnav command: " ABORT_MSG;
 
 static attr_line_t
 format_sql_example(const char* sql_example_fmt)
@@ -498,6 +498,8 @@ rl_change(readline_curses* rc)
 static void
 rl_search_internal(readline_curses* rc, ln_mode_t mode, bool complete = false)
 {
+    static const intern_string_t SRC = intern_string::lookup("prompt");
+
     auto* tc = get_textview_for_mode(mode);
     std::string term_val;
     std::string name;
@@ -522,12 +524,19 @@ rl_search_internal(readline_curses* rc, ln_mode_t mode, bool complete = false)
             break;
 
         case ln_mode_t::COMMAND: {
-            lnav_data.ld_exec_context.ec_dry_run = true;
+            auto& ec = lnav_data.ld_exec_context;
+            ec.ec_dry_run = true;
 
             lnav_data.ld_preview_generation += 1;
             clear_preview();
-            auto result = execute_command(lnav_data.ld_exec_context,
-                                          rc->get_value().get_string());
+            auto src_guard = ec.enter_source(
+                SRC,
+                1,
+                fmt::format(FMT_STRING(":{}"), rc->get_value().get_string()));
+            readline_lnav_highlighter(ec.ec_source.back().s_content, -1);
+            ec.ec_source.back().s_content.with_attr_for_all(
+                VC_ROLE.value(role_t::VCR_QUOTED_CODE));
+            auto result = execute_command(ec, rc->get_value().get_string());
 
             if (result.isOk()) {
                 auto msg = result.unwrap();
@@ -547,7 +556,7 @@ rl_search_internal(readline_curses* rc, ln_mode_t mode, bool complete = false)
 
             lnav_data.ld_preview_view[0].reload_data();
 
-            lnav_data.ld_exec_context.ec_dry_run = false;
+            ec.ec_dry_run = false;
             return;
         }
 
@@ -619,6 +628,8 @@ rl_search_internal(readline_curses* rc, ln_mode_t mode, bool complete = false)
                     prev_stage_prql.append(" | take 5");
 
                     curr_stage_index = 1;
+                    auto src_guard = lnav_data.ld_exec_context.enter_source(
+                        SRC, 1, prev_stage_prql.get_string());
                     auto db_guard = lnav_data.ld_exec_context.enter_db_source(
                         &lnav_data.ld_db_preview_source[0]);
                     auto exec_res = execute_sql(lnav_data.ld_exec_context,
@@ -652,6 +663,8 @@ rl_search_internal(readline_curses* rc, ln_mode_t mode, bool complete = false)
                     }
                 }
 
+                auto src_guard = lnav_data.ld_exec_context.enter_source(
+                    SRC, 1, curr_stage_prql.get_string());
                 auto db_guard = lnav_data.ld_exec_context.enter_db_source(
                     &lnav_data.ld_db_preview_source[curr_stage_index]);
                 auto exec_res = execute_sql(lnav_data.ld_exec_context,
@@ -843,6 +856,8 @@ lnav_rl_abort(readline_curses* rc)
 static void
 rl_callback_int(readline_curses* rc, bool is_alt)
 {
+    static const intern_string_t SRC = intern_string::lookup("prompt");
+
     textview_curses* tc = get_textview_for_mode(lnav_data.ld_mode);
     exec_context& ec = lnav_data.ld_exec_context;
     std::string alt_msg;
@@ -885,8 +900,10 @@ rl_callback_int(readline_curses* rc, bool is_alt)
 
         case ln_mode_t::COMMAND: {
             rc->set_alt_value("");
-            ec.ec_source.back().s_content
-                = fmt::format(FMT_STRING(":{}"), rc->get_value().get_string());
+            auto src_guard = lnav_data.ld_exec_context.enter_source(
+                SRC,
+                1,
+                fmt::format(FMT_STRING(":{}"), rc->get_value().get_string()));
             readline_lnav_highlighter(ec.ec_source.back().s_content, -1);
             ec.ec_source.back().s_content.with_attr_for_all(
                 VC_ROLE.value(role_t::VCR_QUOTED_CODE));
@@ -970,8 +987,8 @@ rl_callback_int(readline_curses* rc, bool is_alt)
 
         case ln_mode_t::SQL: {
             auto sql_str = rc->get_value().get_string();
-            ec.ec_source.back().s_content
-                = fmt::format(FMT_STRING(";{}"), sql_str);
+            auto src_guard = lnav_data.ld_exec_context.enter_source(
+                SRC, 1, fmt::format(FMT_STRING(";{}"), sql_str));
             readline_lnav_highlighter(ec.ec_source.back().s_content, -1);
             ec.ec_source.back().s_content.with_attr_for_all(
                 VC_ROLE.value(role_t::VCR_QUOTED_CODE));
@@ -996,7 +1013,7 @@ rl_callback_int(readline_curses* rc, bool is_alt)
                                      .append(attr_line_t::from_ansi_str(
                                          msg.c_str())))
                                  .to_attr_line();
-                    if (dls.dls_rows.size() > 1) {
+                    if (dls.dls_row_cursors.size() > 1) {
                         ensure_view(&lnav_data.ld_views[LNV_DB]);
                     }
                 }
@@ -1031,6 +1048,7 @@ rl_callback_int(readline_curses* rc, bool is_alt)
                 const auto path_and_args = rc->get_value();
                 auto tmp_pair = open_temp_res.unwrap();
                 auto fd_copy = tmp_pair.second.dup();
+                auto tf = text_format_t::TF_UNKNOWN;
 
                 {
                     exec_context::output_guard og(
@@ -1038,11 +1056,16 @@ rl_callback_int(readline_curses* rc, bool is_alt)
                         "tmp",
                         std::make_pair(fdopen(tmp_pair.second.release(), "w"),
                                        fclose));
-
+                    auto src_guard = lnav_data.ld_exec_context.enter_source(
+                        SRC,
+                        1,
+                        fmt::format(FMT_STRING("|{}"),
+                                    path_and_args.get_string()));
                     auto exec_res
                         = execute_file(ec, path_and_args.get_string());
                     if (exec_res.isOk()) {
                         rc->set_value(exec_res.unwrap());
+                        tf = ec.ec_output_stack.back().od_format;
                     } else {
                         auto um = exec_res.unwrapErr();
 
@@ -1072,6 +1095,7 @@ rl_callback_int(readline_curses* rc, bool is_alt)
                         .with_filename(desc)
                         .with_include_in_session(false)
                         .with_detect_format(false)
+                        .with_text_format(tf)
                         .with_init_location(0_vl);
                     lnav_data.ld_files_to_front.emplace_back(desc, 0_vl);
 
@@ -1103,14 +1127,11 @@ rl_display_matches(readline_curses* rc)
 {
     const auto& matches = rc->get_matches();
     auto& tc = lnav_data.ld_match_view;
-    unsigned long width;
-    __attribute((unused)) unsigned long height;
-    int max_len, cols;
+    int cols;
 
-    getmaxyx(lnav_data.ld_window, height, width);
-
-    max_len = rc->get_max_match_length() + 2;
-    cols = std::max(1UL, width / max_len);
+    auto width = ncplane_dim_x(lnav_data.ld_window);
+    auto max_len = rc->get_max_match_length() + 2;
+    cols = std::max(1U, width / max_len);
 
     if (matches.empty()) {
         lnav_data.ld_match_source.clear();
@@ -1126,7 +1147,7 @@ rl_display_matches(readline_curses* rc)
                 add_nl = false;
             }
             if (match == current_match) {
-                al.append(match, VC_STYLE.value(text_attrs{A_REVERSE}));
+                al.append(match, VC_STYLE.value(text_attrs::with_reverse()));
             } else {
                 al.append(match);
             }
@@ -1189,6 +1210,7 @@ rl_blur(readline_curses* rc)
                    .get_overlay_source();
 
     fos->fos_contexts.pop();
+    ensure(!fos->fos_contexts.empty());
     for (auto& tc : lnav_data.ld_views) {
         tc.set_sync_selection_and_top(false);
     }
