@@ -73,7 +73,10 @@ find_matching_bracket(
             } else if (line[lpc] == left && is_bracket(line, lpc, is_lit)) {
                 if (depth == 0) {
                     alb.overlay_attr_for_char(
-                        lpc, VC_STYLE.value(text_attrs{A_BOLD | A_REVERSE}));
+                        lpc,
+                        VC_STYLE.value(text_attrs ::with_styles(
+                            text_attrs::style::bold,
+                            text_attrs::style::reverse)));
                     alb.overlay_attr_for_char(lpc,
                                               VC_ROLE.value(role_t::VCR_OK));
                     break;
@@ -90,7 +93,10 @@ find_matching_bracket(
             } else if (line[lpc] == right && is_bracket(line, lpc, is_lit)) {
                 if (depth == 0) {
                     alb.overlay_attr_for_char(
-                        lpc, VC_STYLE.value(text_attrs{A_BOLD | A_REVERSE}));
+                        lpc,
+                        VC_STYLE.value(text_attrs ::with_styles(
+                            text_attrs::style::bold,
+                            text_attrs::style::reverse)));
                     alb.overlay_attr_for_char(lpc,
                                               VC_ROLE.value(role_t::VCR_OK));
                     break;
@@ -116,7 +122,9 @@ find_matching_bracket(
             } else {
                 auto lr = line_range(is_lit ? lpc - 1 : lpc, lpc + 1);
                 alb.overlay_attr(
-                    lr, VC_STYLE.value(text_attrs{A_BOLD | A_REVERSE}));
+                    lr,
+                    VC_STYLE.value(text_attrs ::with_styles(
+                        text_attrs::style::bold, text_attrs::style::reverse)));
                 alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_ERROR));
             }
         }
@@ -126,7 +134,10 @@ find_matching_bracket(
         auto lr
             = line_range(is_lit ? first_left.value() - 1 : first_left.value(),
                          first_left.value() + 1);
-        alb.overlay_attr(lr, VC_STYLE.value(text_attrs{A_BOLD | A_REVERSE}));
+        alb.overlay_attr(
+            lr,
+            VC_STYLE.value(text_attrs ::with_styles(
+                text_attrs::style::bold, text_attrs::style::reverse)));
         alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_ERROR));
     }
 }
@@ -188,11 +199,10 @@ readline_command_highlighter_int(attr_line_t& al,
                     .then([&](const auto& rgb_fg) {
                         auto color
                             = view_colors::singleton().match_color(rgb_fg);
+                        auto ta = text_attrs::with_bold();
+                        ta.ta_fg_color = color;
                         alb.overlay_attr(to_line_range(md[0].value()),
-                                         VC_STYLE.value(text_attrs{
-                                             A_BOLD,
-                                             color,
-                                         }));
+                                         VC_STYLE.value(ta));
                     });
             });
     }
@@ -207,9 +217,7 @@ readline_command_highlighter_int(attr_line_t& al,
             for (last = start; last < sub.length() && !isspace(line[last]);
                  last++)
                 ;
-            struct line_range lr {
-                (int) start, (int) last
-            };
+            line_range lr{(int) start, (int) last};
 
             if (x && lr.length() > 0 && !lr.contains(x.value())
                 && !lr.contains(x.value() - 1))
@@ -287,7 +295,7 @@ readline_sqlite_highlighter_int(attr_line_t& al,
                 alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_STRING));
             } else {
                 alb.overlay_attr_for_char(
-                    lr.lr_start, VC_STYLE.value(text_attrs{A_REVERSE}));
+                    lr.lr_start, VC_STYLE.value(text_attrs::with_reverse()));
                 alb.overlay_attr_for_char(lr.lr_start,
                                           VC_ROLE.value(role_t::VCR_ERROR));
             }
@@ -335,7 +343,7 @@ readline_shlex_highlighter_int(attr_line_t& al,
 
             alb.overlay_attr(line_range(sub.lr_start + te.te_source.sf_begin,
                                         sub.lr_start + te.te_source.sf_end),
-                             VC_STYLE.value(text_attrs{A_REVERSE}));
+                             VC_STYLE.value(text_attrs::with_reverse()));
             alb.overlay_attr(line_range(sub.lr_start + te.te_source.sf_begin,
                                         sub.lr_start + te.te_source.sf_end),
                              VC_ROLE.value(role_t::VCR_ERROR));
@@ -512,3 +520,24 @@ readline_lnav_highlighter(attr_line_t& al, std::optional<int> x)
                                       });
     }
 }
+
+void
+highlight_syntax(text_format_t tf, attr_line_t& al)
+{
+     switch (tf) {
+        case text_format_t::TF_SQL: {
+            readline_sqlite_highlighter(al, std::nullopt);
+            break;
+        }
+         case text_format_t::TF_SHELL_SCRIPT: {
+            readline_shlex_highlighter(al, std::nullopt);
+            break;
+        }
+         case text_format_t::TF_LNAV_SCRIPT: {
+            readline_lnav_highlighter(al, std::nullopt);
+            break;
+        }
+        default:
+            break;
+    }
+}
