@@ -37,7 +37,7 @@
 #include "config.h"
 #include "lnav_util.hh"
 #include "pcrepp/pcre2pp.hh"
-#include "scn/scn.h"
+#include "scn/scan.h"
 #include "sql_help.hh"
 #include "sql_util.hh"
 #include "sqlitepp.hh"
@@ -282,12 +282,17 @@ struct regexp_capture_flags {
     bool convert_numbers{true};
 };
 
-const typed_json_path_container<regexp_capture_flags>
-    regexp_capture_flags_handlers
-    = typed_json_path_container<regexp_capture_flags>{
-        yajlpp::property_handler("convert-numbers")
-            .for_field(&regexp_capture_flags::convert_numbers),
-    };
+const typed_json_path_container<regexp_capture_flags>&
+get_regexp_capture_flags_handlers()
+{
+    static const typed_json_path_container<regexp_capture_flags> retval
+        = typed_json_path_container<regexp_capture_flags>{
+            yajlpp::property_handler("convert-numbers")
+                .for_field(&regexp_capture_flags::convert_numbers),
+        };
+
+    return retval;
+}
 
 struct regexp_capture_into_json {
     static constexpr const char* NAME = "regexp_capture_into_json";
@@ -381,17 +386,17 @@ CREATE TABLE regexp_capture_into_json (
                         if (!vc.c_flags || vc.c_flags->convert_numbers) {
                             auto cap_view = cap->to_string_view();
                             auto scan_int_res
-                                = scn::scan_value<int64_t>(cap_view);
+                                = scn::scan_int<int64_t>(cap_view, 0);
 
-                            if (scan_int_res && scan_int_res.range().empty()) {
-                                yajl_gen_integer(gen, scan_int_res.value());
+                            if (scan_int_res && scan_int_res->range().empty()) {
+                                yajl_gen_integer(gen, scan_int_res->value());
                                 continue;
                             }
 
                             auto scan_float_res
                                 = scn::scan_value<double>(cap_view);
                             if (scan_float_res
-                                && scan_float_res.range().empty())
+                                && scan_float_res->range().empty())
                             {
                                 yajl_gen_number(
                                     gen, cap_view.data(), cap_view.length());
@@ -510,7 +515,7 @@ rcjFilter(sqlite3_vtab_cursor* pVtabCursor,
 
         if (!flags_json.empty()) {
             const auto parse_res
-                = regexp_capture_flags_handlers.parser_for(FLAGS_SRC).of(
+                = get_regexp_capture_flags_handlers().parser_for(FLAGS_SRC).of(
                     flags_json);
 
             if (parse_res.isErr()) {
