@@ -32,7 +32,7 @@
 #include "base/itertools.hh"
 #include "lnav.hh"
 #include "logfile_sub_source.hh"
-#include "scn/scn.h"
+#include "scn/scan.h"
 
 using namespace lnav::roles::literals;
 
@@ -43,16 +43,18 @@ class filtered_sub_source
 public:
     size_t text_line_count() override { return this->fss_lines.size(); }
 
-    void text_value_for_line(textview_curses& tc,
-                             int line,
-                             std::string& value_out,
-                             line_flags_t flags) override
+    line_info text_value_for_line(textview_curses& tc,
+                                  int line,
+                                  std::string& value_out,
+                                  line_flags_t flags) override
     {
         this->fss_lines | lnav::itertools::nth(line)
             | lnav::itertools::for_each([&](const auto row) {
                   this->fss_delegate->text_value_for_line(
                       tc, *row, value_out, flags);
               });
+
+        return {};
     }
 
     size_t text_size_for_line(textview_curses& tc,
@@ -105,8 +107,7 @@ public:
     std::vector<vis_line_t> fss_lines;
 };
 
-log_spectro_value_source::
-log_spectro_value_source(intern_string_t colname)
+log_spectro_value_source::log_spectro_value_source(intern_string_t colname)
     : lsvs_colname(colname)
 {
     this->update_stats();
@@ -117,8 +118,8 @@ log_spectro_value_source::update_stats()
 {
     auto& lss = lnav_data.ld_log_source;
 
-    this->lsvs_begin_time = 0;
-    this->lsvs_end_time = 0;
+    this->lsvs_begin_time = std::chrono::microseconds::zero();
+    this->lsvs_end_time = std::chrono::microseconds::zero();
     this->lsvs_stats.clear();
     for (auto& ls : lss) {
         auto* lf = ls->get_file_ptr();
@@ -136,26 +137,28 @@ log_spectro_value_source::update_stats()
 
         auto ll = lf->begin();
 
-        if (this->lsvs_begin_time == 0
-            || ll->get_time() < this->lsvs_begin_time)
+        if (this->lsvs_begin_time == std::chrono::microseconds::zero()
+            || ll->get_time<std::chrono::microseconds>()
+                < this->lsvs_begin_time)
         {
-            this->lsvs_begin_time = ll->get_time();
+            this->lsvs_begin_time = ll->get_time<std::chrono::microseconds>();
         }
         ll = lf->end();
         --ll;
-        if (ll->get_time() > this->lsvs_end_time) {
-            this->lsvs_end_time = ll->get_time();
+        if (ll->get_time<std::chrono::microseconds>() > this->lsvs_end_time) {
+            this->lsvs_end_time = ll->get_time<std::chrono::microseconds>();
         }
 
         this->lsvs_found = true;
         this->lsvs_stats.merge(*stats);
     }
 
-    if (this->lsvs_begin_time) {
-        time_t filtered_begin_time = lss.find_line(lss.at(0_vl))->get_time();
-        time_t filtered_end_time
+    if (this->lsvs_begin_time > std::chrono::microseconds::zero()) {
+        auto filtered_begin_time = lss.find_line(lss.at(0_vl))
+                                       ->get_time<std::chrono::microseconds>();
+        auto filtered_end_time
             = lss.find_line(lss.at(vis_line_t(lss.text_line_count() - 1)))
-                  ->get_time();
+                  ->get_time<std::chrono::microseconds>();
 
         if (filtered_begin_time > this->lsvs_begin_time) {
             this->lsvs_begin_time = filtered_begin_time;
@@ -189,20 +192,22 @@ log_spectro_value_source::spectro_row(spectrogram_request& sr,
                                       spectrogram_row& row_out)
 {
     auto& lss = lnav_data.ld_log_source;
-    auto begin_line = lss.find_from_time(sr.sr_begin_time).value_or(0_vl);
-    auto end_line = lss.find_from_time(sr.sr_end_time)
+    auto begin_line
+        = lss.find_from_time(timeval{to_time_t(sr.sr_begin_time), 0})
+              .value_or(0_vl);
+    auto end_line = lss.find_from_time(timeval{to_time_t(sr.sr_end_time), 0})
                         .value_or(vis_line_t(lss.text_line_count()));
 
     for (const auto& msg_info : lss.window_at(begin_line, end_line)) {
         const auto& ll = msg_info.get_logline();
-        if (ll.get_time() >= sr.sr_end_time) {
+        if (ll.get_time<std::chrono::microseconds>() >= sr.sr_end_time) {
             break;
         }
 
         const auto& values = msg_info.get_values();
         auto lv_iter = find_if(values.lvv_values.begin(),
                                values.lvv_values.end(),
-                               logline_value_cmp(&this->lsvs_colname));
+                               logline_value_name_cmp(&this->lsvs_colname));
 
         if (lv_iter != values.lvv_values.end()) {
             switch (lv_iter->lv_meta.lvm_kind) {
@@ -224,23 +229,26 @@ log_spectro_value_source::spectro_row(spectrogram_request& sr,
                                                 double range_max) {
         auto& lss = lnav_data.ld_log_source;
         auto retval = std::make_unique<filtered_sub_source>();
-        auto begin_line = lss.find_from_time(sr.sr_begin_time).value_or(0_vl);
-        auto end_line = lss.find_from_time(sr.sr_end_time)
-                            .value_or(vis_line_t(lss.text_line_count()));
+        auto begin_line
+            = lss.find_from_time(timeval{to_time_t(sr.sr_begin_time), 0})
+                  .value_or(0_vl);
+        auto end_line
+            = lss.find_from_time(timeval{to_time_t(sr.sr_end_time), 0})
+                  .value_or(vis_line_t(lss.text_line_count()));
 
         retval->fss_delegate = &lss;
         retval->fss_time_delegate = &lss;
         retval->fss_overlay_delegate = nullptr;
         for (const auto& msg_info : lss.window_at(begin_line, end_line)) {
             const auto& ll = msg_info.get_logline();
-            if (ll.get_time() >= sr.sr_end_time) {
+            if (ll.get_time<std::chrono::microseconds>() >= sr.sr_end_time) {
                 break;
             }
 
             const auto& values = msg_info.get_values();
             auto lv_iter = find_if(values.lvv_values.begin(),
                                    values.lvv_values.end(),
-                                   logline_value_cmp(&this->lsvs_colname));
+                                   logline_value_name_cmp(&this->lsvs_colname));
 
             if (lv_iter != values.lvv_values.end()) {
                 switch (lv_iter->lv_meta.lvm_kind) {
@@ -272,17 +280,18 @@ log_spectro_value_source::spectro_row(spectrogram_request& sr,
 
 void
 log_spectro_value_source::spectro_mark(textview_curses& tc,
-                                       time_t begin_time,
-                                       time_t end_time,
+                                       std::chrono::microseconds begin_time,
+                                       std::chrono::microseconds end_time,
                                        double range_min,
                                        double range_max)
 {
     // XXX need to refactor this and the above method
     auto& log_tc = lnav_data.ld_views[LNV_LOG];
     auto& lss = lnav_data.ld_log_source;
-    vis_line_t begin_line = lss.find_from_time(begin_time).value_or(0_vl);
-    vis_line_t end_line = lss.find_from_time(end_time).value_or(
-        vis_line_t(lss.text_line_count()));
+    auto begin_line
+        = lss.find_from_time(timeval{to_time_t(begin_time), 0}).value_or(0_vl);
+    auto end_line = lss.find_from_time(timeval{to_time_t(end_time), 0})
+                        .value_or(vis_line_t(lss.text_line_count()));
     logline_value_vector values;
     string_attrs_t sa;
 
@@ -304,7 +313,7 @@ log_spectro_value_source::spectro_mark(textview_curses& tc,
 
         auto lv_iter = find_if(values.lvv_values.begin(),
                                values.lvv_values.end(),
-                               logline_value_cmp(&this->lsvs_colname));
+                               logline_value_name_cmp(&this->lsvs_colname));
 
         if (lv_iter != values.lvv_values.end()) {
             switch (lv_iter->lv_meta.lvm_kind) {
@@ -331,8 +340,7 @@ log_spectro_value_source::spectro_mark(textview_curses& tc,
     }
 }
 
-db_spectro_value_source::
-db_spectro_value_source(std::string colname)
+db_spectro_value_source::db_spectro_value_source(std::string colname)
     : dsvs_colname(std::move(colname))
 {
     this->update_stats();
@@ -341,8 +349,8 @@ db_spectro_value_source(std::string colname)
 void
 db_spectro_value_source::update_stats()
 {
-    this->dsvs_begin_time = 0;
-    this->dsvs_end_time = 0;
+    this->dsvs_begin_time = std::chrono::microseconds::zero();
+    this->dsvs_end_time = std::chrono::microseconds::zero();
     this->dsvs_stats.clear();
 
     auto& dls = lnav_data.ld_db_row_source;
@@ -422,7 +430,7 @@ db_spectro_value_source::update_stats()
         return;
     }
 
-    if (!dls.dls_headers[this->dsvs_column_index.value()].hm_graphable) {
+    if (!dls.dls_headers[this->dsvs_column_index.value()].is_graphable()) {
         this->dsvs_error_msg
             = lnav::console::user_message::error(
                   "Cannot generate spectrogram for database results")
@@ -435,7 +443,7 @@ db_spectro_value_source::update_stats()
         return;
     }
 
-    if (dls.dls_rows.empty()) {
+    if (dls.dls_row_cursors.empty()) {
         this->dsvs_error_msg
             = lnav::console::user_message::error(
                   "Cannot generate spectrogram for database results")
@@ -444,8 +452,8 @@ db_spectro_value_source::update_stats()
         return;
     }
 
-    this->dsvs_begin_time = dls.dls_time_column.front().tv_sec;
-    this->dsvs_end_time = dls.dls_time_column.back().tv_sec;
+    this->dsvs_begin_time = to_us(dls.dls_time_column.front());
+    this->dsvs_end_time = to_us(dls.dls_time_column.back());
 
     auto find_res
         = dls.dls_headers | lnav::itertools::find_if([this](const auto& elem) {
@@ -458,7 +466,7 @@ db_spectro_value_source::update_stats()
         this->dsvs_stats.lvs_max_value = bs.bs_max_value;
     }
 
-    this->dsvs_stats.lvs_count = dls.dls_rows.size();
+    this->dsvs_stats.lvs_count = dls.dls_row_cursors.size();
 }
 
 void
@@ -484,16 +492,17 @@ db_spectro_value_source::spectro_row(spectrogram_request& sr,
                                      spectrogram_row& row_out)
 {
     auto& dls = lnav_data.ld_db_row_source;
-    auto begin_row = dls.row_for_time({sr.sr_begin_time, 0}).value_or(0_vl);
-    auto end_row = dls.row_for_time({sr.sr_end_time, 0})
-                       .value_or(vis_line_t(dls.dls_rows.size()));
+    auto begin_row
+        = dls.row_for_time({to_time_t(sr.sr_begin_time), 0}).value_or(0_vl);
+    auto end_row = dls.row_for_time({to_time_t(sr.sr_end_time), 0})
+                       .value_or(vis_line_t(dls.dls_row_cursors.size()));
 
     for (auto lpc = begin_row; lpc < end_row; ++lpc) {
-        auto scan_res = scn::scan_value<double>(scn::string_view{
-            dls.dls_rows[lpc][this->dsvs_column_index.value()]});
+        auto get_res
+            = dls.get_cell_as_double(lpc, this->dsvs_column_index.value());
 
-        if (scan_res) {
-            row_out.add_value(sr, scan_res.value(), false);
+        if (get_res) {
+            row_out.add_value(sr, get_res.value(), false);
         }
     }
 
@@ -506,17 +515,18 @@ db_spectro_value_source::spectro_row(spectrogram_request& sr,
         retval->fss_delegate = &dls;
         retval->fss_time_delegate = &dls;
         retval->fss_overlay_delegate = &lnav_data.ld_db_overlay;
-        auto begin_row = dls.row_for_time({sr.sr_begin_time, 0}).value_or(0_vl);
-        auto end_row = dls.row_for_time({sr.sr_end_time, 0})
-                           .value_or(vis_line_t(dls.dls_rows.size()));
+        auto begin_row
+            = dls.row_for_time({to_time_t(sr.sr_begin_time), 0}).value_or(0_vl);
+        auto end_row = dls.row_for_time({to_time_t(sr.sr_end_time), 0})
+                           .value_or(vis_line_t(dls.dls_row_cursors.size()));
 
         for (auto lpc = begin_row; lpc < end_row; ++lpc) {
-            auto scan_res = scn::scan_value<double>(scn::string_view{
-                dls.dls_rows[lpc][this->dsvs_column_index.value()]});
-            if (!scan_res) {
+            auto get_res
+                = dls.get_cell_as_double(lpc, this->dsvs_column_index.value());
+            if (!get_res) {
                 continue;
             }
-            auto value = scan_res.value();
+            auto value = get_res.value();
             if ((range_min == value)
                 || (range_min < value && value < range_max))
             {
