@@ -18,6 +18,7 @@
 #include <string.h>
 
 #include "base/humanize.hh"
+#include "base/is_utf8.hh"
 #include "base/lnav.gzip.hh"
 #include "base/string_util.hh"
 #include "column_namer.hh"
@@ -25,14 +26,15 @@
 #include "data_parser.hh"
 #include "data_scanner.hh"
 #include "elem_to_json.hh"
+#include "fmt/format.h"
 #include "formats/logfmt/logfmt.parser.hh"
+#include "hasher.hh"
 #include "libbase64.h"
 #include "mapbox/variant.hpp"
 #include "pcrepp/pcre2pp.hh"
 #include "pretty_printer.hh"
 #include "safe/safe.h"
-#include "scn/scn.h"
-#include "spookyhash/SpookyV2.h"
+#include "scn/scan.h"
 #include "sqlite-extension-func.hh"
 #include "text_anonymizer.hh"
 #include "view_curses.hh"
@@ -55,7 +57,7 @@ enum class encode_algo {
 
 template<>
 struct from_sqlite<encode_algo> {
-    inline encode_algo operator()(int argc, sqlite3_value** val, int argi)
+    encode_algo operator()(int argc, sqlite3_value** val, int argi)
     {
         const char* algo_name = (const char*) sqlite3_value_text(val[argi]);
 
@@ -82,12 +84,12 @@ struct cache_entry {
         std::make_shared<column_namer>(column_namer::language::JSON)};
 };
 
-static cache_entry*
+cache_entry*
 find_re(string_fragment re)
 {
     using re_cache_t
         = std::unordered_map<string_fragment, cache_entry, frag_hasher>;
-    static thread_local re_cache_t cache;
+    thread_local re_cache_t cache;
 
     auto iter = cache.find(re);
     if (iter == cache.end()) {
@@ -146,9 +148,6 @@ mapbox::util::
         throw std::runtime_error(err.get_message());
     }
 
-    yajlpp_gen gen;
-    yajl_gen_config(gen, yajl_gen_beautify, false);
-
     if (extractor.get_capture_count() == 1) {
         auto cap = md[1];
 
@@ -156,18 +155,24 @@ mapbox::util::
             return static_cast<const char*>(nullptr);
         }
 
-        auto scan_int_res = scn::scan_value<int64_t>(cap->to_string_view());
-        if (scan_int_res && scan_int_res.empty()) {
-            return scan_int_res.value();
-        }
-
-        auto scan_float_res = scn::scan_value<double>(cap->to_string_view());
-        if (scan_float_res && scan_float_res.empty()) {
-            return scan_float_res.value();
+        auto scan_int_res = scn::scan_int<int64_t>(cap->to_string_view());
+        if (scan_int_res) {
+            if (scan_int_res->range().empty()) {
+                return scan_int_res->value();
+            }
+            auto scan_float_res
+                = scn::scan_value<double>(cap->to_string_view());
+            if (scan_float_res && scan_float_res->range().empty()) {
+                return scan_float_res->value();
+            }
         }
 
         return cap.value();
-    } else {
+    }
+
+    yajlpp_gen gen;
+    yajl_gen_config(gen, yajl_gen_beautify, false);
+    {
         yajlpp_map root_map(gen);
 
         for (size_t lpc = 0; lpc < extractor.get_capture_count(); lpc++) {
@@ -181,12 +186,12 @@ mapbox::util::
             } else {
                 auto scan_int_res
                     = scn::scan_value<int64_t>(cap->to_string_view());
-                if (scan_int_res && scan_int_res.empty()) {
-                    yajl_gen_integer(gen, scan_int_res.value());
+                if (scan_int_res && scan_int_res->range().empty()) {
+                    yajl_gen_integer(gen, scan_int_res->value());
                 } else {
                     auto scan_float_res
                         = scn::scan_value<double>(cap->to_string_view());
-                    if (scan_float_res && scan_float_res.empty()) {
+                    if (scan_float_res && scan_float_res->range().empty()) {
                         yajl_gen_number(gen, cap->data(), cap->length());
                     } else {
                         yajl_gen_pstring(gen, cap->data(), cap->length());
@@ -279,25 +284,24 @@ regexp_replace(string_fragment str, string_fragment re, const char* repl)
     return reobj->re2->replace(str, repl);
 }
 
-std::string
+string_fragment
 spooky_hash(const std::vector<const char*>& args)
 {
-    byte_array<2, uint64> hash;
-    SpookyHash context;
+    thread_local char hash_str_buf[hasher::STRING_SIZE];
 
-    context.Init(0, 0);
+    hasher context;
     for (const auto* const arg : args) {
         int64_t len = arg != nullptr ? strlen(arg) : 0;
 
-        context.Update(&len, sizeof(len));
+        context.update((const char*) &len, sizeof(len));
         if (arg == nullptr) {
             continue;
         }
-        context.Update(arg, len);
+        context.update(arg, len);
     }
-    context.Final(hash.out(0), hash.out(1));
+    context.to_string(hash_str_buf);
 
-    return hash.to_string();
+    return string_fragment::from_bytes(hash_str_buf, sizeof(hash_str_buf) - 1);
 }
 
 void
@@ -318,7 +322,7 @@ sql_spooky_hash_step(sqlite3_context* context, int argc, sqlite3_value** argv)
     }
 }
 
-static void
+void
 sql_spooky_hash_final(sqlite3_context* context)
 {
     auto* hasher
@@ -343,7 +347,7 @@ struct sparkline_context {
     std::vector<double> sc_values;
 };
 
-static void
+void
 sparkline_step(sqlite3_context* context, int argc, sqlite3_value** argv)
 {
     auto* sc = (sparkline_context*) sqlite3_aggregate_context(
@@ -366,7 +370,7 @@ sparkline_step(sqlite3_context* context, int argc, sqlite3_value** argv)
     }
 }
 
-static void
+void
 sparkline_final(sqlite3_context* context)
 {
     auto* sc = (sparkline_context*) sqlite3_aggregate_context(
@@ -474,7 +478,7 @@ get_curl_easy()
 }
 #endif
 
-static mapbox::util::variant<text_auto_buffer, auto_mem<char>, null_value_t>
+mapbox::util::variant<text_auto_buffer, auto_mem<char>, null_value_t>
 sql_encode(sqlite3_value* value, encode_algo algo)
 {
     switch (sqlite3_value_type(value)) {
@@ -554,7 +558,7 @@ sql_encode(sqlite3_value* value, encode_algo algo)
     ensure(false);
 }
 
-static mapbox::util::variant<blob_auto_buffer, auto_mem<char>>
+mapbox::util::variant<blob_auto_buffer, auto_mem<char>>
 sql_decode(string_fragment str, encode_algo algo)
 {
     switch (algo) {
@@ -571,15 +575,16 @@ sql_decode(string_fragment str, encode_algo algo)
             auto sv = str.to_string_view();
 
             while (!sv.empty()) {
-                int32_t value;
-                auto scan_res = scn::scan(sv, "{:2x}", value);
+                auto scan_res = scn::scan<int32_t>(sv, "{:2x}");
                 if (!scan_res) {
                     throw sqlite_func_error(
                         "invalid hex input at: {}",
                         std::distance(str.begin(), sv.begin()));
                 }
+                auto value = scan_res->value();
                 buf.push_back((char) (value & 0xff));
-                sv = scan_res.range_as_string_view();
+                sv = std::string_view{scan_res->range().begin(),
+                                      scan_res->range().size()};
             }
 
             return blob_auto_buffer{std::move(buf)};
@@ -604,7 +609,7 @@ sql_humanize_file_size(file_ssize_t value)
     return humanize::file_size(value, humanize::alignment::columnar);
 }
 
-static std::string
+std::string
 sql_anonymize(string_fragment frag)
 {
     static safe::Safe<lnav::text_anonymizer> ta;
@@ -630,9 +635,21 @@ sql_parse_url(std::string url)
     auto rc = curl_url_set(
         cu, CURLUPART_URL, url.c_str(), CURLU_NON_SUPPORT_SCHEME);
     if (rc != CURLUE_OK) {
-        throw lnav::console::user_message::error(
-            attr_line_t("invalid URL: ").append(lnav::roles::file(url)))
-            .with_reason(curl_url_strerror(rc));
+        auto_mem<char> url_part(curl_free);
+        yajlpp_gen gen;
+        yajl_gen_config(gen, yajl_gen_beautify, false);
+
+        {
+            yajlpp_map root(gen);
+            root.gen("error");
+            root.gen("invalid-url");
+            root.gen("url");
+            root.gen(url);
+            root.gen("reason");
+            root.gen(curl_url_strerror(rc));
+        }
+
+        return json_string(gen);
     }
 
     auto_mem<char> url_part(curl_free);
@@ -681,7 +698,18 @@ sql_parse_url(std::string url)
         root.gen("path");
         rc = curl_url_get(cu, CURLUPART_PATH, url_part.out(), CURLU_URLDECODE);
         if (rc == CURLUE_OK) {
-            root.gen(string_fragment::from_c_str(url_part.in()));
+            auto path_frag = string_fragment::from_c_str(url_part.in());
+            auto path_utf_res = is_utf8(path_frag);
+            if (path_utf_res.is_valid()) {
+                root.gen(path_frag);
+            } else {
+                rc = curl_url_get(cu, CURLUPART_PATH, url_part.out(), 0);
+                if (rc == CURLUE_OK) {
+                    root.gen(string_fragment::from_c_str(url_part.in()));
+                } else {
+                    root.gen();
+                }
+            }
         } else {
             root.gen();
         }
@@ -720,12 +748,24 @@ sql_parse_url(std::string url)
                 if (eq_index_opt) {
                     auto key = kv_pair_frag.sub_range(0, eq_index_opt.value());
                     auto val = kv_pair_frag.substr(eq_index_opt.value() + 1);
-                    auto key_str = key.to_string();
 
-                    if (seen_keys.count(key_str) == 0) {
-                        seen_keys.emplace(key_str);
-                        query_map.gen(key);
-                        query_map.gen(val);
+                    auto key_utf_res = is_utf8(key);
+                    auto val_utf_res = is_utf8(val);
+                    if (key_utf_res.is_valid()) {
+                        auto key_str = key.to_string();
+
+                        if (seen_keys.count(key_str) == 0) {
+                            seen_keys.emplace(key_str);
+                            query_map.gen(key);
+                            if (val_utf_res.is_valid()) {
+                                query_map.gen(val);
+                            } else {
+                                auto eq = strchr(kv_pair_encoded.data(), '=');
+                                query_map.gen(
+                                    string_fragment::from_c_str(eq + 1));
+                            }
+                        }
+                    } else {
                     }
                 } else {
                     auto val_str = split_res.first.to_string();
@@ -774,30 +814,37 @@ struct url_parts {
     std::optional<std::string> up_fragment;
 };
 
-static const json_path_container url_params_handlers = {
-    yajlpp::pattern_property_handler("(?<param>.*)")
-        .for_field(&url_parts::up_parameters),
-};
+const typed_json_path_container<url_parts>&
+get_url_parts_handlers()
+{
+    static const json_path_container url_params_handlers = {
+        yajlpp::pattern_property_handler("(?<param>.*)")
+            .for_field(&url_parts::up_parameters),
+    };
 
-static const typed_json_path_container<url_parts> url_parts_handlers = {
-    yajlpp::property_handler("scheme").for_field(&url_parts::up_scheme),
-    yajlpp::property_handler("username").for_field(&url_parts::up_username),
-    yajlpp::property_handler("password").for_field(&url_parts::up_password),
-    yajlpp::property_handler("host").for_field(&url_parts::up_host),
-    yajlpp::property_handler("port").for_field(&url_parts::up_port),
-    yajlpp::property_handler("path").for_field(&url_parts::up_path),
-    yajlpp::property_handler("query").for_field(&url_parts::up_query),
-    yajlpp::property_handler("parameters").with_children(url_params_handlers),
-    yajlpp::property_handler("fragment").for_field(&url_parts::up_fragment),
-};
+    static const typed_json_path_container<url_parts> retval = {
+        yajlpp::property_handler("scheme").for_field(&url_parts::up_scheme),
+        yajlpp::property_handler("username").for_field(&url_parts::up_username),
+        yajlpp::property_handler("password").for_field(&url_parts::up_password),
+        yajlpp::property_handler("host").for_field(&url_parts::up_host),
+        yajlpp::property_handler("port").for_field(&url_parts::up_port),
+        yajlpp::property_handler("path").for_field(&url_parts::up_path),
+        yajlpp::property_handler("query").for_field(&url_parts::up_query),
+        yajlpp::property_handler("parameters")
+            .with_children(url_params_handlers),
+        yajlpp::property_handler("fragment").for_field(&url_parts::up_fragment),
+    };
 
-static auto_mem<char>
+    return retval;
+}
+
+auto_mem<char>
 sql_unparse_url(string_fragment in)
 {
     static auto* CURL_HANDLE = get_curl_easy();
     static intern_string_t SRC = intern_string::lookup("arg");
 
-    auto parse_res = url_parts_handlers.parser_for(SRC).of(in);
+    auto parse_res = get_url_parts_handlers().parser_for(SRC).of(in);
     if (parse_res.isErr()) {
         throw parse_res.unwrapErr()[0];
     }
@@ -886,7 +933,8 @@ sql_humanize_id(string_fragment id)
     auto attrs = vc.attrs_for_ident(id.data(), id.length());
 
     return fmt::format(FMT_STRING("\x1b[38;5;{}m{}\x1b[0m"),
-                       attrs.ta_fg_color.value_or(COLOR_CYAN),
+                       // XXX attrs.ta_fg_color.value_or(COLOR_CYAN),
+                       (int8_t) ansi_color::cyan,
                        id);
 }
 
