@@ -33,11 +33,38 @@
 
 #include "text_format.hh"
 
+#include "base/from_trait.hh"
+#include "base/itertools.enumerate.hh"
+#include "base/itertools.hh"
 #include "base/lnav_log.hh"
 #include "config.h"
 #include "pcrepp/pcre2pp.hh"
 #include "yajl/api/yajl_parse.h"
 
+const string_fragment
+    TEXT_FORMAT_STRINGS[lnav::enums::to_underlying(text_format_t::TF_UNKNOWN)
+                        + 1]
+    = {
+        string_fragment::from_const("application/octet-stream"),
+        string_fragment::from_const("text/c"),
+        string_fragment::from_const("text/java"),
+        string_fragment::from_const("application/json"),
+        string_fragment::from_const("text/log"),
+        string_fragment::from_const("text/x-makefile"),
+        string_fragment::from_const("text/man"),
+        string_fragment::from_const("text/markdown"),
+        string_fragment::from_const("text/python"),
+        string_fragment::from_const("text/rust"),
+        string_fragment::from_const("application/sql"),
+        string_fragment::from_const("text/xml"),
+        string_fragment::from_const("application/yaml"),
+        string_fragment::from_const("application/toml"),
+        string_fragment::from_const("text/x-diff"),
+        string_fragment::from_const("text/x-shellscript"),
+        string_fragment::from_const("text/x-lnav-script"),
+        string_fragment::from_const("text/plain"),
+};
+
 text_format_t
 detect_text_format(string_fragment sf,
                    std::optional<std::filesystem::path> path)
@@ -70,6 +97,7 @@ detect_text_format(string_fragment sf,
     static const auto MD_EXT = std::filesystem::path(".md");
     static const auto MARKDOWN_EXT = std::filesystem::path(".markdown");
     static const auto SH_EXT = std::filesystem::path(".sh");
+    static const auto LNAV_EXT = std::filesystem::path(".lnav");
 
     static const auto DIFF_MATCHERS = lnav::pcre2pp::code::from_const(
         R"(^--- .*\n\+\+\+ .*\n)", PCRE2_MULTILINE);
@@ -136,6 +164,13 @@ detect_text_format(string_fragment sf,
     static const auto SH_MATCHERS
         = lnav::pcre2pp::code::from_const("^#!.+sh\\b", PCRE2_MULTILINE);
 
+    static const auto LNAV_MATCHERS = lnav::pcre2pp::code::from_const(
+        "(?:"
+        "^;SELECT\\s+|"
+        "^:[a-z0-9\\-]+\\s+"
+        ")",
+        PCRE2_MULTILINE | PCRE2_CASELESS);
+
     if (path) {
         while (FILTER_EXTS.count(path->extension()) > 0) {
             path = path->stem();
@@ -186,6 +221,10 @@ detect_text_format(string_fragment sf,
         if (stem == SH_EXT) {
             return text_format_t::TF_SHELL_SCRIPT;
         }
+
+        if (stem == LNAV_EXT) {
+            return text_format_t::TF_LNAV_SCRIPT;
+        }
     }
 
     {
@@ -225,6 +264,10 @@ detect_text_format(string_fragment sf,
         return text_format_t::TF_C_LIKE;
     }
 
+    if (LNAV_MATCHERS.find_in(sf).ignore_error()) {
+        return text_format_t::TF_LNAV_SCRIPT;
+    }
+
     if (SQL_MATCHERS.find_in(sf).ignore_error()) {
         return text_format_t::TF_SQL;
     }
@@ -244,8 +287,7 @@ extract_text_meta(string_fragment sf, text_format_t tf)
 
     switch (tf) {
         case text_format_t::TF_MAN: {
-            static thread_local auto md
-                = lnav::pcre2pp::match_data::unitialized();
+            thread_local auto md = lnav::pcre2pp::match_data::unitialized();
 
             auto find_res
                 = MAN_NAME.capture_from(sf).into(md).matches().ignore_error();
@@ -263,3 +305,17 @@ extract_text_meta(string_fragment sf, text_format_t tf)
 
     return std::nullopt;
 }
+
+template<>
+Result<text_format_t, std::string>
+from(const string_fragment sf)
+{
+    for (const auto& [index, format_sf] :
+         lnav::itertools::enumerate(TEXT_FORMAT_STRINGS))
+    {
+        if (format_sf == sf) {
+            return Ok(static_cast<text_format_t>(index));
+        }
+    }
+    return Err(fmt::format(FMT_STRING("unrecognized text format: {}"), sf));
+}
