@@ -46,7 +46,7 @@
 #include "lnav.events.hh"
 #include "md2attr_line.hh"
 #include "pretty_printer.hh"
-#include "scn/scn.h"
+#include "scn/scan.h"
 #include "sql_util.hh"
 #include "sqlitepp.hh"
 #include "textfile_sub_source.cfg.hh"
@@ -102,7 +102,7 @@ textfile_sub_source::text_line_count()
     return retval;
 }
 
-void
+line_info
 textfile_sub_source::text_value_for_line(textview_curses& tc,
                                          int line,
                                          std::string& value_out,
@@ -110,7 +110,7 @@ textfile_sub_source::text_value_for_line(textview_curses& tc,
 {
     if (this->tss_files.empty() || line < 0) {
         value_out.clear();
-        return;
+        return {};
     }
 
     const auto lf = this->current_file();
@@ -120,7 +120,7 @@ textfile_sub_source::text_value_for_line(textview_curses& tc,
     {
         rend_iter->second.rf_text_source->text_value_for_line(
             tc, line, value_out, flags);
-        return;
+        return {};
     }
 
     if (lf->get_text_format() == text_format_t::TF_BINARY) {
@@ -136,7 +136,7 @@ textfile_sub_source::text_value_for_line(textview_curses& tc,
                       fr.fr_offset,
                       fr.fr_size,
                       read_res.unwrapErr().c_str());
-            return;
+            return {};
         }
 
         auto sbr = read_res.unwrap();
@@ -154,20 +154,25 @@ textfile_sub_source::text_value_for_line(textview_curses& tc,
         }
 
         value_out = this->tss_hex_line.get_string();
-        return;
+        return {};
     }
 
     auto* lfo = dynamic_cast<line_filter_observer*>(lf->get_logline_observer());
     if (lfo == nullptr || line >= lfo->lfo_filter_state.tfs_index.size()) {
         value_out.clear();
-        return;
+        return {};
     }
 
-    auto ll = lf->begin() + lfo->lfo_filter_state.tfs_index[line];
+    const auto ll = lf->begin() + lfo->lfo_filter_state.tfs_index[line];
     auto read_result = lf->read_line(ll);
     this->tss_line_indent_size = 0;
     if (read_result.isOk()) {
-        value_out = to_string(read_result.unwrap());
+        auto sbr = read_result.unwrap();
+        value_out = to_string(sbr);
+        this->tss_plain_line_attrs.clear();
+        if (sbr.get_metadata().m_has_ansi) {
+            scrub_ansi_string(value_out, &this->tss_plain_line_attrs);
+        }
         for (const auto& ch : value_out) {
             if (ch == ' ') {
                 this->tss_line_indent_size += 1;
@@ -185,6 +190,8 @@ textfile_sub_source::text_value_for_line(textview_curses& tc,
                 = fmt::format(FMT_STRING("{: >12}|{}"), relstr, value_out);
         }
     }
+
+    return {};
 }
 
 void
@@ -210,6 +217,7 @@ textfile_sub_source::text_attrs_for_line(textview_curses& tc,
     } else if (lf->get_text_format() == text_format_t::TF_BINARY) {
         value_out = this->tss_hex_line.get_attrs();
     } else {
+        value_out = this->tss_plain_line_attrs;
         auto* lfo
             = dynamic_cast<line_filter_observer*>(lf->get_logline_observer());
         if (lfo != nullptr && row >= 0
@@ -228,17 +236,17 @@ textfile_sub_source::text_attrs_for_line(textview_curses& tc,
                 value_out.emplace_back(lr,
                                        VC_ROLE.value(role_t::VCR_OFFSET_TIME));
                 value_out.emplace_back(line_range(12, 13),
-                                       VC_GRAPHIC.value(ACS_VLINE));
+                                       VC_GRAPHIC.value(NCACS_VLINE));
 
-                role_t bar_role = role_t::VCR_NONE;
+                auto bar_role = role_t::VCR_NONE;
 
                 switch (this->get_line_accel_direction(vis_line_t(row))) {
-                    case log_accel::A_STEADY:
+                    case log_accel::direction_t::A_STEADY:
                         break;
-                    case log_accel::A_DECEL:
+                    case log_accel::direction_t::A_DECEL:
                         bar_role = role_t::VCR_DIFF_DELETE;
                         break;
-                    case log_accel::A_ACCEL:
+                    case log_accel::direction_t::A_ACCEL:
                         bar_role = role_t::VCR_DIFF_ADD;
                         break;
                 }
@@ -302,7 +310,7 @@ textfile_sub_source::text_attrs_for_line(textview_curses& tc,
         }
     }
 
-    value_out.emplace_back(lr, logline::L_FILE.value(this->current_file()));
+    value_out.emplace_back(lr, L_FILE.value(this->current_file()));
 }
 
 size_t
@@ -848,10 +856,9 @@ textfile_sub_source::rescan_files(textfile_sub_source::scan_callback& callback,
                                 = metadata_state{
                                     st.st_mtime,
                                     lf->get_index_size(),
-                                    lnav::document::discover_structure(
-                                        content,
-                                        line_range{0, -1},
-                                        lf->get_text_format()),
+                                    lnav::document::discover(content)
+                                        .with_text_format(lf->get_text_format())
+                                        .perform(),
                                 };
                         }
                     } else {
@@ -1136,8 +1143,8 @@ textfile_sub_source::row_for_anchor(const std::string& id)
             auto comp_pair = hier_sf.split_when(string_fragment::tag1{'/'});
             auto scan_res
                 = scn::scan_value<int64_t>(comp_pair.first.to_string_view());
-            if (scan_res && scan_res.empty()) {
-                path.emplace_back(scan_res.value());
+            if (scan_res && scan_res->range().empty()) {
+                path.emplace_back(scan_res->value());
             } else {
                 path.emplace_back(json_ptr::decode(comp_pair.first));
             }
@@ -1505,8 +1512,7 @@ textfile_sub_source::get_effective_view_mode() const
     return retval;
 }
 
-textfile_header_overlay::
-textfile_header_overlay(textfile_sub_source* src)
+textfile_header_overlay::textfile_header_overlay(textfile_sub_source* src)
     : tho_src(src)
 {
 }
@@ -1530,9 +1536,7 @@ textfile_header_overlay::list_static_overlay(const listview_curses& lv,
         && this->tho_src->get_effective_view_mode()
             == textfile_sub_source::view_mode::rendered)
     {
-        text_attrs ta;
-
-        ta.ta_attrs |= A_UNDERLINE;
+        auto ta = text_attrs::with_underline();
         value_out.append("\u24d8"_info)
             .append(" The following is a rendered view of the content.  Use ")
             .append(lnav::roles::quoted_code(":set-text-view-mode raw"))
@@ -1566,6 +1570,6 @@ textfile_header_overlay::list_static_overlay(const listview_curses& lv,
             alb.appendf(FMT_STRING("  {:^17}"), "ASCII");
         }
     }
-    value_out.with_attr_for_all(VC_STYLE.value(text_attrs{A_UNDERLINE}));
+    value_out.with_attr_for_all(VC_STYLE.value(text_attrs::with_underline()));
     return true;
 }
