@@ -38,7 +38,6 @@
 #include "base/time_util.hh"
 #include "config.h"
 #include "data_scanner.hh"
-#include "date/solar_hijri.h"
 #include "fmt/format.h"
 #include "lnav_config.hh"
 #include "log_format_fwd.hh"
@@ -46,7 +45,7 @@
 #include "shlex.hh"
 #include "view_curses.hh"
 
-const auto REVERSE_SEARCH_OFFSET = 2000_vl;
+constexpr auto REVERSE_SEARCH_OFFSET = 2000_vl;
 
 void
 text_filter::revert_to_last(logfile_filter_state& lfs, size_t rollback_size)
@@ -139,7 +138,9 @@ text_accel_source::get_line_accel_direction(vis_line_t vl)
             continue;
         }
 
-        if (!la.add_point(curr_line->get_time_in_millis())) {
+        if (!la.add_point(
+                curr_line->get_time<std::chrono::milliseconds>().count()))
+        {
             break;
         }
 
@@ -178,21 +179,21 @@ text_accel_source::get_time_offset_for_line(textview_curses& tc, vis_line_t vl)
     return humanize::time::duration::from_tv(diff_tv).to_string();
 }
 
+const bookmark_type_t textview_curses::BM_ERRORS("error");
+const bookmark_type_t textview_curses::BM_WARNINGS("warning");
 const bookmark_type_t textview_curses::BM_USER("user");
 const bookmark_type_t textview_curses::BM_USER_EXPR("user-expr");
 const bookmark_type_t textview_curses::BM_SEARCH("search");
 const bookmark_type_t textview_curses::BM_META("meta");
 const bookmark_type_t textview_curses::BM_PARTITION("partition");
 
-textview_curses::
-textview_curses()
+textview_curses::textview_curses()
     : lnav_config_listener(__FILE__), tc_search_action(noop_func{})
 {
     this->set_data_source(this);
 }
 
-textview_curses::~
-textview_curses()
+textview_curses::~textview_curses()
 {
     this->tc_search_action = noop_func{};
 }
@@ -201,6 +202,7 @@ void
 textview_curses::reload_config(error_reporter& reporter)
 {
     const static auto DEFAULT_THEME_NAME = std::string("default");
+    const auto& vc = view_colors::singleton();
 
     for (auto iter = this->tc_highlights.begin();
          iter != this->tc_highlights.end();)
@@ -213,15 +215,6 @@ textview_curses::reload_config(error_reporter& reporter)
         iter = this->tc_highlights.erase(iter);
     }
 
-    std::map<std::string, scoped_value_t> vars;
-    auto curr_theme_iter
-        = lnav_config.lc_ui_theme_defs.find(lnav_config.lc_ui_theme);
-    if (curr_theme_iter != lnav_config.lc_ui_theme_defs.end()) {
-        for (const auto& vpair : curr_theme_iter->second.lt_vars) {
-            vars[vpair.first] = vpair.second;
-        }
-    }
-
     for (const auto& theme_name : {DEFAULT_THEME_NAME, lnav_config.lc_ui_theme})
     {
         auto theme_iter = lnav_config.lc_ui_theme_defs.find(theme_name);
@@ -230,55 +223,57 @@ textview_curses::reload_config(error_reporter& reporter)
             continue;
         }
 
+        auto vars = &theme_iter->second.lt_vars;
         for (const auto& hl_pair : theme_iter->second.lt_highlights) {
             if (hl_pair.second.hc_regex.pp_value == nullptr) {
                 continue;
             }
 
             const auto& sc = hl_pair.second.hc_style;
-            std::string fg1, bg1, fg_color, bg_color, errmsg;
+            std::string fg_color, bg_color, errmsg;
             bool invalid = false;
             text_attrs attrs;
 
-            fg1 = sc.sc_color;
-            bg1 = sc.sc_background_color;
-            shlex(fg1).eval(fg_color, scoped_resolver{&vars});
-            shlex(bg1).eval(bg_color, scoped_resolver{&vars});
+            auto fg1 = sc.sc_color;
+            auto bg1 = sc.sc_background_color;
+            shlex(fg1).eval(fg_color, scoped_resolver{vars});
+            shlex(bg1).eval(bg_color, scoped_resolver{vars});
 
-            auto fg = styling::color_unit::from_str(fg_color).unwrapOrElse(
-                [&](const auto& msg) {
-                    reporter(&sc.sc_color,
-                             lnav::console::user_message::error(
-                                 attr_line_t("invalid color -- ")
-                                     .append_quoted(sc.sc_color))
-                                 .with_reason(msg));
-                    invalid = true;
-                    return styling::color_unit::make_empty();
-                });
-            auto bg = styling::color_unit::from_str(bg_color).unwrapOrElse(
-                [&](const auto& msg) {
-                    reporter(&sc.sc_background_color,
-                             lnav::console::user_message::error(
-                                 attr_line_t("invalid background color -- ")
-                                     .append_quoted(sc.sc_background_color))
-                                 .with_reason(msg));
-                    invalid = true;
-                    return styling::color_unit::make_empty();
-                });
+            attrs.ta_fg_color = vc.match_color(
+                styling::color_unit::from_str(fg_color).unwrapOrElse(
+                    [&](const auto& msg) {
+                        reporter(&sc.sc_color,
+                                 lnav::console::user_message::error(
+                                     attr_line_t("invalid color -- ")
+                                         .append_quoted(sc.sc_color))
+                                     .with_reason(msg));
+                        invalid = true;
+                        return styling::color_unit::make_empty();
+                    }));
+            attrs.ta_bg_color = vc.match_color(
+                styling::color_unit::from_str(bg_color).unwrapOrElse(
+                    [&](const auto& msg) {
+                        reporter(&sc.sc_background_color,
+                                 lnav::console::user_message::error(
+                                     attr_line_t("invalid background color -- ")
+                                         .append_quoted(sc.sc_background_color))
+                                     .with_reason(msg));
+                        invalid = true;
+                        return styling::color_unit::make_empty();
+                    }));
             if (invalid) {
                 continue;
             }
 
             if (sc.sc_bold) {
-                attrs.ta_attrs |= A_BOLD;
+                attrs |= text_attrs::style::bold;
             }
             if (sc.sc_underline) {
-                attrs.ta_attrs |= A_UNDERLINE;
+                attrs |= text_attrs::style::underline;
             }
             this->tc_highlights[{highlight_source_t::THEME, hl_pair.first}]
                 = highlighter(hl_pair.second.hc_regex.pp_value)
                       .with_attrs(attrs)
-                      .with_color(fg, bg)
                       .with_nestable(false);
         }
     }
@@ -305,15 +300,14 @@ textview_curses::reload_data()
     this->tc_selected_text = std::nullopt;
     if (this->tc_sub_source != nullptr) {
         this->tc_sub_source->text_update_marks(this->tc_bookmarks);
-    }
-    if (this->tc_sub_source != nullptr) {
+
         auto* ttt = dynamic_cast<text_time_translator*>(this->tc_sub_source);
 
         if (ttt != nullptr) {
             ttt->data_reloaded(this);
         }
+        listview_curses::reload_data();
     }
-    listview_curses::reload_data();
 }
 
 void
@@ -375,15 +369,22 @@ textview_curses::grep_end(grep_proc<vis_line_t>& gp)
 {
     this->tc_searching -= 1;
     this->grep_end_batch(gp);
+    if (this->tc_searching == 0 && this->tc_search_start_time) {
+        const auto now = std::chrono::steady_clock::now();
+        this->tc_search_duration
+            = std::chrono::duration_cast<std::chrono::milliseconds>(
+                now - this->tc_search_start_time.value());
+        this->tc_search_start_time = std::nullopt;
+        if (this->tc_state_event_handler) {
+            this->tc_state_event_handler(*this);
+        }
+    }
 
     ensure(this->tc_searching >= 0);
 }
 
 void
-textview_curses::grep_match(grep_proc<vis_line_t>& gp,
-                            vis_line_t line,
-                            int start,
-                            int end)
+textview_curses::grep_match(grep_proc<vis_line_t>& gp, vis_line_t line)
 {
     this->tc_bookmarks[&BM_SEARCH].insert_once(vis_line_t(line));
     if (this->tc_sub_source != nullptr) {
@@ -555,6 +556,7 @@ textview_curses::handle_mouse(mouse_event& me)
                                             tok_sf.sf_begin,
                                             tok_sf.sf_end,
                                         },
+                                        al.al_attrs,
                                         tok_sf.to_string(),
                                     };
                                     this->set_needs_update();
@@ -611,6 +613,7 @@ textview_curses::handle_mouse(mouse_event& me)
                                     cursor_sf.sf_begin,
                                     cursor_sf.sf_end,
                                 },
+                                al.al_attrs,
                                 cursor_sf.to_string(),
                             };
                         }
@@ -667,7 +670,7 @@ textview_curses::handle_mouse(mouse_event& me)
             if (me.is_click_in(mouse_button_t::BUTTON_RIGHT, 0, INT_MAX)) {
                 auto* lov = this->get_overlay_source();
                 if (lov != nullptr) {
-                    lov->set_show_details_in_overlay(
+                    this->set_show_details_in_overlay(
                         !lov->get_show_details_in_overlay());
                 }
             }
@@ -680,9 +683,7 @@ textview_curses::handle_mouse(mouse_event& me)
                 }
                 this->tc_selection_start = std::nullopt;
             }
-            if (me.me_button == mouse_button_t::BUTTON_LEFT
-                && mouse_line.is<main_content>())
-            {
+            if (mouse_line.is<main_content>()) {
                 const auto& [mc_line] = mouse_line.get<main_content>();
                 attr_line_t al;
 
@@ -690,18 +691,31 @@ textview_curses::handle_mouse(mouse_event& me)
                 auto line_sf = string_fragment::from_str(al.get_string());
                 auto cursor_sf = line_sf.sub_cell_range(
                     this->lv_left + me.me_x, this->lv_left + me.me_x);
-                auto attr_iter = find_string_attr_containing(
+                auto link_iter = find_string_attr_containing(
                     al.get_attrs(), &VC_HYPERLINK, cursor_sf.sf_begin);
-                if (attr_iter != al.get_attrs().end()) {
-                    auto href = attr_iter->sa_value.get<std::string>();
+                if (link_iter != al.get_attrs().end()) {
+                    auto href = link_iter->sa_value.get<std::string>();
+                    auto* ta = dynamic_cast<text_anchors*>(this->tc_sub_source);
 
-                    this->tc_selected_text = selected_text_info{
-                        me.me_x,
-                        mc_line,
-                        attr_iter->sa_range,
-                        al.to_string_fragment(attr_iter).to_string(),
-                        href,
-                    };
+                    if (me.me_button == mouse_button_t::BUTTON_LEFT
+                        && ta != nullptr && startswith(href, "#")
+                        && !startswith(href, "#/frontmatter"))
+                    {
+                        auto row_opt = ta->row_for_anchor(href);
+
+                        if (row_opt.has_value()) {
+                            this->set_selection(row_opt.value());
+                        }
+                    } else {
+                        this->tc_selected_text = selected_text_info{
+                            me.me_x,
+                            mc_line,
+                            link_iter->sa_range,
+                            al.get_attrs(),
+                            al.to_string_fragment(link_iter).to_string(),
+                            href,
+                        };
+                    }
                 }
             }
             if (this->tc_delegate != nullptr) {
@@ -721,13 +735,53 @@ textview_curses::handle_mouse(mouse_event& me)
     return true;
 }
 
+void
+textview_curses::apply_highlights(attr_line_t& al,
+                                  const line_range& body,
+                                  const line_range& orig_line)
+{
+    intern_string_t format_name;
+
+    auto format_attr_opt = get_string_attr(al.al_attrs, SA_FORMAT);
+    if (format_attr_opt.has_value()) {
+        format_name = format_attr_opt.value().get();
+    }
+
+    auto source_format = this->tc_sub_source->get_text_format();
+    for (const auto& tc_highlight : this->tc_highlights) {
+        bool internal_hl
+            = tc_highlight.first.first == highlight_source_t::INTERNAL
+            || tc_highlight.first.first == highlight_source_t::THEME;
+
+        if (!tc_highlight.second.applies_to_format(source_format)) {
+            continue;
+        }
+
+        if (!tc_highlight.second.h_format_name.empty()
+            && tc_highlight.second.h_format_name != format_name)
+        {
+            continue;
+        }
+
+        if (this->tc_disabled_highlights.count(tc_highlight.first.first)) {
+            continue;
+        }
+
+        // Internal highlights should only apply to the log message body so
+        // that we don't start highlighting other fields.  User-provided
+        // highlights should apply only to the line itself and not any of
+        // the surrounding decorations that are added (for example, the file
+        // lines that are inserted at the beginning of the log view).
+        int start_pos = internal_hl ? body.lr_start : orig_line.lr_start;
+        tc_highlight.second.annotate(al, start_pos);
+    }
+}
+
 void
 textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
 {
     auto& sa = value_out.get_attrs();
     auto& str = value_out.get_string();
-    auto source_format = this->tc_sub_source->get_text_format();
-    intern_string_t format_name;
 
     this->tc_sub_source->text_value_for_line(*this, row, str);
     this->tc_sub_source->text_attrs_for_line(*this, row, sa);
@@ -736,11 +790,10 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
         require_ge(attr.sa_range.lr_start, 0);
     }
 
-    scrub_ansi_string(str, &sa);
-    struct line_range body, orig_line;
+    line_range body, orig_line;
 
     body = find_string_attr_range(sa, &SA_BODY);
-    if (body.lr_start == -1) {
+    if (!body.is_valid()) {
         body.lr_start = 0;
         body.lr_end = str.size();
     }
@@ -751,11 +804,6 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
         orig_line.lr_end = str.size();
     }
 
-    auto format_attr_opt = get_string_attr(sa, SA_FORMAT);
-    if (format_attr_opt) {
-        format_name = format_attr_opt.value().get();
-    }
-
     if (this->is_selectable() && this->tc_cursor_role
         && this->tc_disabled_cursor_role)
     {
@@ -780,34 +828,8 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
         }
     }
 
-    for (auto& tc_highlight : this->tc_highlights) {
-        bool internal_hl
-            = tc_highlight.first.first == highlight_source_t::INTERNAL
-            || tc_highlight.first.first == highlight_source_t::THEME;
-
-        if (!tc_highlight.second.h_text_formats.empty()
-            && tc_highlight.second.h_text_formats.count(source_format) == 0)
-        {
-            continue;
-        }
-
-        if (!tc_highlight.second.h_format_name.empty()
-            && tc_highlight.second.h_format_name != format_name)
-        {
-            continue;
-        }
-
-        if (this->tc_disabled_highlights.count(tc_highlight.first.first)) {
-            continue;
-        }
-
-        // Internal highlights should only apply to the log message body so
-        // that we don't start highlighting other fields.  User-provided
-        // highlights should apply only to the line itself and not any of the
-        // surrounding decorations that are added (for example, the file lines
-        // that are inserted at the beginning of the log view).
-        int start_pos = internal_hl ? body.lr_start : orig_line.lr_start;
-        tc_highlight.second.annotate(value_out, start_pos);
+    if (!body.empty() || !orig_line.empty()) {
+        this->apply_highlights(value_out, body, orig_line);
     }
 
     if (this->tc_hide_fields) {
@@ -821,7 +843,7 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
             user_expr_marks.begin(), user_expr_marks.end(), row))
     {
         sa.emplace_back(line_range{orig_line.lr_start, -1},
-                        VC_STYLE.value(text_attrs{A_REVERSE}));
+                        VC_STYLE.value(text_attrs::with_reverse()));
     }
 
     if (this->tc_selected_text) {
@@ -896,6 +918,8 @@ textview_curses::execute_search(const std::string& regex_orig)
             if (top > 0) {
                 gp->queue_request(0_vl, top);
             }
+            this->tc_search_start_time = std::chrono::steady_clock::now();
+            this->tc_search_duration = std::nullopt;
             gp->start();
 
             this->tc_search_child = std::make_shared<grep_highlighter>(
@@ -1002,13 +1026,11 @@ textview_curses::toggle_user_mark(const bookmark_type_t* bm,
     if (end_line >= this->get_inner_height()) {
         end_line = vis_line_t(this->get_inner_height() - 1);
     }
-    for (vis_line_t curr_line = start_line; curr_line <= end_line; ++curr_line)
-    {
-        bookmark_vector<vis_line_t>& bv = this->tc_bookmarks[bm];
-        bookmark_vector<vis_line_t>::iterator iter;
+    for (auto curr_line = start_line; curr_line <= end_line; ++curr_line) {
+        auto& bv = this->tc_bookmarks[bm];
         bool added;
 
-        iter = bv.insert_once(curr_line);
+        auto iter = bv.insert_once(curr_line);
         if (iter == bv.end()) {
             added = true;
         } else {
@@ -1074,21 +1096,26 @@ textview_curses::set_sub_source(text_sub_source* src)
     return *this;
 }
 
-bool
+std::optional<line_info>
 textview_curses::grep_value_for_line(vis_line_t line, std::string& value_out)
 {
-    bool retval = false;
-
     if (this->tc_sub_source
         && line < (int) this->tc_sub_source->text_line_count())
     {
-        this->tc_sub_source->text_value_for_line(
+        auto retval = this->tc_sub_source->text_value_for_line(
             *this, line, value_out, text_sub_source::RF_RAW);
-        scrub_ansi_string(value_out, nullptr);
-        retval = true;
+        if (retval.li_utf8_scan_result.is_valid()
+            && retval.li_utf8_scan_result.usr_has_ansi)
+        {
+            // log_debug("has ansi %d",
+            // retval.li_utf8_scan_result.usr_has_ansi);
+            auto new_size = erase_ansi_escapes(value_out);
+            value_out.resize(new_size);
+        }
+        return retval;
     }
 
-    return retval;
+    return std::nullopt;
 }
 
 void
@@ -1311,8 +1338,7 @@ text_sub_source::text_crumbs_for_line(int line,
 {
 }
 
-logfile_filter_state::
-logfile_filter_state(std::shared_ptr<logfile> lf)
+logfile_filter_state::logfile_filter_state(std::shared_ptr<logfile> lf)
     : tfs_logfile(std::move(lf))
 {
     memset(this->tfs_filter_count, 0, sizeof(this->tfs_filter_count));
@@ -1383,6 +1409,12 @@ logfile_filter_state::resize(size_t newsize)
     }
 }
 
+void
+logfile_filter_state::reserve(size_t expected)
+{
+    this->tfs_mask.reserve(expected);
+}
+
 std::optional<size_t>
 logfile_filter_state::content_line_to_vis_line(uint32_t line)
 {
