@@ -36,6 +36,7 @@
 #include "base/humanize.hh"
 #include "base/humanize.time.hh"
 #include "base/itertools.hh"
+#include "base/keycodes.hh"
 #include "base/math_util.hh"
 #include "command_executor.hh"
 #include "intervaltree/IntervalTree.h"
@@ -174,8 +175,8 @@ timeline_preview_overlay::list_overlay_menu(const listview_curses& lv,
     return retval;
 }
 
-timeline_header_overlay::
-timeline_header_overlay(const std::shared_ptr<timeline_source>& src)
+timeline_header_overlay::timeline_header_overlay(
+    const std::shared_ptr<timeline_source>& src)
     : gho_src(src)
 {
 }
@@ -280,15 +281,14 @@ timeline_header_overlay::list_static_overlay(const listview_curses& lv,
         value_out.with_attr_for_all(VC_ROLE.value(role_t::VCR_CURSOR_LINE));
     } else {
         value_out.append("   Duration   "_h1)
-            .append("|", VC_GRAPHIC.value(ACS_VLINE))
+            .append("|", VC_GRAPHIC.value(NCACS_VLINE))
             .append(" ")
             .append("\u2718"_error)
             .append("\u25b2"_warning)
             .append(" ")
-            .append("|", VC_GRAPHIC.value(ACS_VLINE))
+            .append("|", VC_GRAPHIC.value(NCACS_VLINE))
             .append(" Operation"_h1);
-        auto hdr_attrs = text_attrs{};
-        hdr_attrs.ta_attrs = A_UNDERLINE;
+        auto hdr_attrs = text_attrs::with_underline();
         value_out.get_attrs().emplace_back(line_range{0, -1},
                                            VC_STYLE.value(hdr_attrs));
         value_out.with_attr_for_all(VC_ROLE.value(role_t::VCR_STATUS_INFO));
@@ -371,16 +371,12 @@ timeline_header_overlay::list_value_for_overlay(
             lr.lr_end += 1;
         }
 
-        auto block_attrs = text_attrs{};
-        block_attrs.ta_attrs = A_REVERSE;
+        auto block_attrs = text_attrs::with_reverse();
         attrs.emplace_back(lr, VC_STYLE.value(block_attrs));
     }
     if (!value_out.empty()) {
-        text_attrs ta_under;
-
-        ta_under.ta_attrs = A_UNDERLINE;
-        value_out.back().get_attrs().emplace_back(line_range{0, -1},
-                                                  VC_STYLE.value(ta_under));
+        value_out.back().get_attrs().emplace_back(
+            line_range{0, -1}, VC_STYLE.value(text_attrs::with_underline()));
     }
 }
 std::optional<attr_line_t>
@@ -397,13 +393,12 @@ timeline_header_overlay::list_header_for_overlay(const listview_curses& lv,
         .append(" to focus on this panel");
 }
 
-timeline_source::
-timeline_source(textview_curses& log_view,
-                logfile_sub_source& lss,
-                textview_curses& preview_view,
-                plain_text_source& preview_source,
-                statusview_curses& preview_status_view,
-                timeline_status_source& preview_status_source)
+timeline_source::timeline_source(textview_curses& log_view,
+                                 logfile_sub_source& lss,
+                                 textview_curses& preview_view,
+                                 plain_text_source& preview_source,
+                                 statusview_curses& preview_status_view,
+                                 timeline_status_source& preview_status_source)
     : gs_log_view(log_view), gs_lss(lss), gs_preview_view(preview_view),
       gs_preview_source(preview_source),
       gs_preview_status_view(preview_status_view),
@@ -414,9 +409,9 @@ timeline_source(textview_curses& log_view,
 }
 
 bool
-timeline_source::list_input_handle_key(listview_curses& lv, int ch)
+timeline_source::list_input_handle_key(listview_curses& lv, const ncinput& ch)
 {
-    switch (ch) {
+    switch (ch.eff_text[0]) {
         case 'q':
         case KEY_ESCAPE: {
             if (this->gs_preview_focused) {
@@ -431,7 +426,7 @@ timeline_source::list_input_handle_key(listview_curses& lv, int ch)
         }
         case '\n':
         case '\r':
-        case KEY_ENTER: {
+        case NCKEY_ENTER: {
             this->gs_preview_focused = !this->gs_preview_focused;
             this->gs_preview_status_view.set_enabled(this->gs_preview_focused);
             if (this->gs_preview_focused) {
@@ -466,8 +461,11 @@ timeline_source::text_handle_mouse(
     const listview_curses::display_line_content_t&,
     mouse_event& me)
 {
+    auto nci = ncinput{};
     if (me.is_double_click_in(mouse_button_t::BUTTON_LEFT, line_range{0, -1})) {
-        this->list_input_handle_key(tc, '\r');
+        nci.id = '\r';
+        nci.eff_text[0] = '\r';
+        this->list_input_handle_key(tc, nci);
     }
 
     return false;
@@ -522,7 +520,7 @@ timeline_source::text_line_count()
     return this->gs_time_order.size();
 }
 
-void
+line_info
 timeline_source::text_value_for_line(textview_curses& tc,
                                      int line,
                                      std::string& value_out,
@@ -560,6 +558,8 @@ timeline_source::text_value_for_line(textview_curses& tc,
 
         value_out = this->gs_rendered_line.get_string();
     }
+
+    return {};
 }
 
 void
@@ -609,8 +609,7 @@ timeline_source::text_attrs_for_line(textview_curses& tc,
                     }
                 }
 
-                auto block_attrs = text_attrs{};
-                block_attrs.ta_attrs = A_REVERSE;
+                auto block_attrs = text_attrs::with_reverse();
                 value_out.emplace_back(lr, VC_STYLE.value(block_attrs));
             }
         }
@@ -634,8 +633,8 @@ void
 timeline_source::rebuild_indexes()
 {
     auto& bm = this->tss_view->get_bookmarks();
-    auto& bm_errs = bm[&logfile_sub_source::BM_ERRORS];
-    auto& bm_warns = bm[&logfile_sub_source::BM_WARNINGS];
+    auto& bm_errs = bm[&textview_curses::BM_ERRORS];
+    auto& bm_warns = bm[&textview_curses::BM_WARNINGS];
 
     bm_errs.clear();
     bm_warns.clear();
@@ -1057,7 +1056,7 @@ timeline_source::get_filtered_count_for(size_t filter_index) const
     return this->gs_filter_hits[filter_index];
 }
 
-static std::vector<breadcrumb::possibility>
+static const std::vector<breadcrumb::possibility>&
 timestamp_poss()
 {
     const static std::vector<breadcrumb::possibility> retval = {
@@ -1082,6 +1081,7 @@ void
 timeline_source::text_crumbs_for_line(int line,
                                       std::vector<breadcrumb::crumb>& crumbs)
 {
+    static intern_string_t SRC = intern_string::lookup("crumb");
     text_sub_source::text_crumbs_for_line(line, crumbs);
 
     if (line >= this->gs_time_order.size()) {
@@ -1093,13 +1093,15 @@ timeline_source::text_crumbs_for_line(int line,
 
     sql_strftime(ts, sizeof(ts), row.or_value.otr_range.tr_begin, 'T');
 
-    crumbs.emplace_back(
-        std::string(ts),
-        timestamp_poss,
-        [ec = this->gs_exec_context](const auto& ts) {
-            ec->execute(fmt::format(FMT_STRING(":goto {}"),
-                                    ts.template get<std::string>()));
-        });
+    crumbs.emplace_back(std::string(ts),
+                        timestamp_poss,
+                        [ec = this->gs_exec_context](const auto& ts) {
+                            auto cmd
+                                = fmt::format(FMT_STRING(":goto {}"),
+                                              ts.template get<std::string>());
+                            auto src_guard = ec->enter_source(SRC, 1, cmd);
+                            ec->execute(cmd);
+                        });
     crumbs.back().c_expected_input
         = breadcrumb::crumb::expected_input_t::anything;
     crumbs.back().c_search_placeholder = "(Enter an absolute or relative time)";
