@@ -27,8 +27,12 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <memory>
+#include <utility>
+
 #include "view_helpers.hh"
 
+#include "base/itertools.enumerate.hh"
 #include "base/itertools.hh"
 #include "bound_tags.hh"
 #include "config.h"
@@ -55,7 +59,7 @@
 using namespace std::chrono_literals;
 using namespace lnav::roles::literals;
 
-const char* lnav_view_strings[LNV__MAX + 1] = {
+const char* const lnav_view_strings[LNV__MAX + 1] = {
     "log",
     "text",
     "help",
@@ -69,7 +73,7 @@ const char* lnav_view_strings[LNV__MAX + 1] = {
     nullptr,
 };
 
-const char* lnav_view_titles[LNV__MAX] = {
+const char* const lnav_view_titles[LNV__MAX] = {
     "LOG",
     "TEXT",
     "HELP",
@@ -81,7 +85,8 @@ const char* lnav_view_titles[LNV__MAX] = {
     "TIMELINE",
 };
 
-const char* lnav_mode_strings[lnav::enums::to_underlying(ln_mode_t::BUSY) + 1]
+const char* const
+    lnav_mode_strings[lnav::enums::to_underlying(ln_mode_t::BUSY) + 1]
     = {
         "PAGING",
         "BREADCRUMBS",
@@ -334,7 +339,8 @@ public:
 static void
 open_pretty_view()
 {
-    static const char* NOTHING_MSG = "Nothing to pretty-print";
+    static auto NOTHING_MSG
+        = string_fragment::from_const("Nothing to pretty-print");
 
     auto* top_tc = *lnav_data.ld_view_stack.top();
     auto* pretty_tc = &lnav_data.ld_views[LNV_PRETTY];
@@ -365,23 +371,28 @@ open_pretty_view()
     std::vector<std::unique_ptr<lnav::document::hier_node>> hier_nodes;
     std::vector<pretty_sub_source::hier_interval_t> hier_tree_vec;
     std::set<size_t> pretty_indents;
+    std::optional<vis_line_t> pretty_selected_line;
     if (top_tc == log_tc) {
         auto& lss = lnav_data.ld_log_source;
-        bool first_line = true;
         auto start_off = size_t{0};
+        auto line_count = 0_vl;
 
         for (auto vl = log_tc->get_top(); vl <= log_tc->get_bottom(); ++vl) {
-            content_line_t cl = lss.at(vl);
+            auto cl = lss.at(vl);
             auto lf = lss.find(cl);
             auto ll = lf->begin() + cl;
             shared_buffer_ref sbr;
 
-            if (!first_line && !ll->is_message()) {
+            if (line_count > 0_vl && !ll->is_message()) {
                 continue;
             }
             auto ll_start = lf->message_start(ll);
             attr_line_t al;
 
+            if (vl == log_tc->get_selection()) {
+                pretty_selected_line = line_count;
+            }
+
             vl -= vis_line_t(std::distance(ll_start, ll));
             lss.text_value_for_line(
                 *log_tc,
@@ -457,7 +468,7 @@ open_pretty_view()
                 full_text.append("\n");
             }
 
-            first_line = false;
+            line_count += vis_line_t(pretty_lines.size());
             for (auto& interval : curr_intervals) {
                 interval.start += start_off;
                 interval.stop += start_off;
@@ -477,14 +488,19 @@ open_pretty_view()
             start_off += line_off;
         }
     } else if (top_tc == text_tc) {
-        if (text_tc->listview_rows(*text_tc)) {
+        if (text_tc->listview_rows(*text_tc) > 0) {
             std::vector<attr_line_t> rows;
             rows.resize(text_tc->get_bottom() - text_tc->get_top() + 1);
             text_tc->listview_value_for_rows(
                 *text_tc, text_tc->get_top(), rows);
+
             attr_line_t orig_al;
 
+            auto curr_vl = text_tc->get_top();
             for (auto& row : rows) {
+                if (curr_vl == text_tc->get_selection()) {
+                    row.with_attr_for_all(SA_ORIGIN_OFFSET.value(int64_t{0}));
+                }
                 remove_string_attr(row.get_attrs(), &VC_BLOCK_ELEM);
                 for (auto& attr : row.get_attrs()) {
                     if (attr.sa_type == &VC_ROLE) {
@@ -498,6 +514,7 @@ open_pretty_view()
                     }
                 }
                 orig_al.append(row);
+                curr_vl += 1_vl;
             }
 
             data_scanner ds(orig_al.get_string());
@@ -505,6 +522,16 @@ open_pretty_view()
 
             pp.append_to(full_text);
 
+            auto origin_opt
+                = get_string_attr(full_text.get_attrs(), &SA_ORIGIN_OFFSET);
+            if (origin_opt.has_value()) {
+                auto leading = string_fragment::from_byte_range(
+                    full_text.al_string.data(),
+                    0,
+                    origin_opt.value()->sa_range.lr_start);
+                auto leading_lines = leading.count('\n');
+                pretty_selected_line = vis_line_t(leading_lines);
+            }
             all_intervals = pp.take_intervals();
             hier_nodes.emplace_back(pp.take_hier_root());
             hier_tree_vec.emplace_back(
@@ -528,6 +555,7 @@ open_pretty_view()
     if (lnav_data.ld_last_pretty_print_top != log_tc->get_top()) {
         pretty_tc->set_top(0_vl);
     }
+    pretty_tc->set_selection(pretty_selected_line.value_or(0_vl));
     lnav_data.ld_last_pretty_print_top = log_tc->get_top();
     pretty_tc->redo_search();
 }
@@ -539,7 +567,8 @@ build_all_help_text()
         return;
     }
 
-    shlex lexer(help_md.to_string_fragment());
+    auto help_md_str = help_md.to_string_fragment_producer()->to_string();
+    shlex lexer(help_md_str);
     std::string sub_help_text;
 
     lexer.with_ignore_quotes(true).eval(
@@ -609,7 +638,7 @@ build_all_help_text()
 }
 
 bool
-handle_winch()
+handle_winch(screen_curses* sc)
 {
     static auto* filter_source = injector::get<filter_sub_source*>();
 
@@ -617,20 +646,20 @@ handle_winch()
         return false;
     }
 
-    struct winsize size;
+    if (sc) {
+        notcurses_refresh(sc->get_notcurses(), nullptr, nullptr);
+        notcurses_render(sc->get_notcurses());
+        notcurses_refresh(sc->get_notcurses(), nullptr, nullptr);
+    }
 
     lnav_data.ld_winched = false;
-
-    if (ioctl(fileno(stdout), TIOCGWINSZ, &size) == 0) {
-        resizeterm(size.ws_row, size.ws_col);
-    }
     if (lnav_data.ld_rl_view != nullptr) {
         lnav_data.ld_rl_view->do_update();
         lnav_data.ld_rl_view->window_change();
     }
     filter_source->fss_editor->window_change();
-    for (auto& sc : lnav_data.ld_status) {
-        sc.window_change();
+    for (auto& stat : lnav_data.ld_status) {
+        stat.window_change();
     }
     lnav_data.ld_view_stack.set_needs_update();
     lnav_data.ld_doc_view.set_needs_update();
@@ -649,12 +678,13 @@ handle_winch()
 void
 layout_views()
 {
-    static constexpr auto FILES_FOCUSED_WIDTH = 40;
-    static constexpr auto FILES_BLURRED_WIDTH = 20;
+    static constexpr auto FILES_FOCUSED_WIDTH = 40U;
+    static constexpr auto FILES_BLURRED_WIDTH = 20U;
 
     static auto* breadcrumb_view = injector::get<breadcrumb_curses*>();
-    int width, height;
-    getmaxyx(lnav_data.ld_window, height, width);
+
+    unsigned int width, height;
+    ncplane_dim_yx(lnav_data.ld_window, &height, &width);
 
     int doc_height;
     bool doc_side_by_side = width > (90 + 60);
@@ -707,7 +737,7 @@ layout_views()
     }
 
     int match_rows = lnav_data.ld_match_source.text_line_count();
-    int match_height = std::min(match_rows, (height - 4) / 2);
+    int match_height = std::min(match_rows, (int) (height - 4) / 2);
     lnav_data.ld_match_view.set_height(vis_line_t(match_height));
 
     int um_rows = lnav_data.ld_user_message_source.text_line_count();
@@ -718,7 +748,7 @@ layout_views()
         lnav_data.ld_user_message_source.clear();
         um_rows = 0;
     }
-    auto um_height = std::min(um_rows, (height - 4) / 2);
+    auto um_height = std::min(um_rows, (int) (height - 4) / 2);
     lnav_data.ld_user_message_view.set_height(vis_line_t(um_height));
 
     auto config_panel_open = (lnav_data.ld_mode == ln_mode_t::FILTER
@@ -751,7 +781,7 @@ layout_views()
 
     bool breadcrumb_open = (lnav_data.ld_mode == ln_mode_t::BREADCRUMBS);
 
-    auto bottom_min = std::min(2 + 3, height);
+    auto bottom_min = std::min(2U + 3U, height);
     auto bottom = clamped<int>::from(height, bottom_min, height);
 
     lnav_data.ld_rl_view->set_y(height - 1);
@@ -859,7 +889,7 @@ layout_views()
     lnav_data.ld_file_details_view.set_y(bottom + 2);
     lnav_data.ld_file_details_view.set_x(files_width);
     lnav_data.ld_file_details_view.set_width(
-        std::clamp(width - files_width, 0, width));
+        std::clamp(width - files_width, 0U, width));
     lnav_data.ld_file_details_view.set_visible(files_open && vis);
 
     lnav_data.ld_status[LNS_FILTER_HELP].set_visible(config_panel_open && vis);
@@ -892,9 +922,11 @@ layout_views()
 void
 update_hits(textview_curses* tc)
 {
+#if 0
     if (isendwin()) {
         return;
     }
+#endif
 
     auto top_tc = lnav_data.ld_view_stack.top();
 
@@ -989,22 +1021,27 @@ update_hits(textview_curses* tc)
     }
 }
 
-static std::unordered_map<std::string, attr_line_t> EXAMPLE_RESULTS;
+using safe_example_results
+    = safe::Safe<std::unordered_map<std::string, attr_line_t>>;
+
+static safe_example_results EXAMPLE_RESULTS;
 
 static void
-execute_example(const help_text& ht)
+execute_example(std::unordered_map<std::string, attr_line_t>& res_map,
+                const help_text& ht)
 {
-    static const std::set<std::string> IGNORED_NAMES = {"ATTACH"};
+    static const std::set<std::string> IGNORED_NAMES = {"ATTACH", "DETACH"};
 
     if (IGNORED_NAMES.count(ht.ht_name)) {
         return;
     }
 
-    auto& dls = lnav_data.ld_db_row_source;
-    auto& dos = lnav_data.ld_db_overlay;
+    auto& dls = lnav_data.ld_db_example_row_source;
+    auto& dos = lnav_data.ld_db_example_overlay;
     auto& db_tc = lnav_data.ld_views[LNV_DB];
 
-    for (const auto& ex : ht.ht_example) {
+    for (const auto& [index, ex] : lnav::itertools::enumerate(ht.ht_example, 1))
+    {
         std::string alt_msg;
         attr_line_t result;
 
@@ -1012,7 +1049,7 @@ execute_example(const help_text& ht)
             continue;
         }
 
-        if (EXAMPLE_RESULTS.count(ex.he_cmd)) {
+        if (res_map.count(ex.he_cmd)) {
             continue;
         }
 
@@ -1023,8 +1060,11 @@ execute_example(const help_text& ht)
             case help_context_t::HC_SQL_TABLE_VALUED_FUNCTION:
             case help_context_t::HC_PRQL_TRANSFORM:
             case help_context_t::HC_PRQL_FUNCTION: {
+                intern_string_t ex_src = intern_string::lookup(ht.ht_name);
                 exec_context ec;
 
+                auto src_guard = ec.enter_source(ex_src, index, ex.he_cmd);
+                dls.clear();
                 ec.ec_label_source_stack.push_back(&dls);
 
                 auto exec_res = execute_sql(ec, ex.he_cmd, alt_msg);
@@ -1032,10 +1072,10 @@ execute_example(const help_text& ht)
                 if (exec_res.isErr()) {
                     auto um = exec_res.unwrapErr();
                     result.append(um.to_attr_line());
-                } else if (dls.dls_rows.size() == 1
-                           && dls.dls_rows[0].size() == 1)
+                } else if (dls.dls_row_cursors.size() == 1
+                           && dls.dls_headers.size() == 1)
                 {
-                    result.append(dls.dls_rows[0][0]);
+                    result.append(dls.get_row_as_string(0_vl));
                 } else {
                     attr_line_t al;
                     dos.list_static_overlay(db_tc, 0, 1, al);
@@ -1053,11 +1093,12 @@ execute_example(const help_text& ht)
                         result.append("\n").append(al);
                     }
                 }
-
-                EXAMPLE_RESULTS[ex.he_cmd] = result;
-
+                result.with_attr_for_all(SA_PREFORMATTED.value());
                 log_trace("example: %s", ex.he_cmd);
                 log_trace("example result: %s", result.get_string().c_str());
+
+                scrub_ansi_string(result.al_string, &result.al_attrs);
+                res_map.emplace(ex.he_cmd, std::move(result));
                 break;
             }
             default:
@@ -1067,50 +1108,31 @@ execute_example(const help_text& ht)
     }
 }
 
-void
-execute_examples()
-{
-    static const auto* sql_cmd_map
-        = injector::get<readline_context::command_map_t*, sql_cmd_map_tag>();
-
-    auto& dls = lnav_data.ld_db_row_source;
-
-    auto old_width = dls.dls_max_column_width;
-    dls.dls_max_column_width = 15;
-    for (const auto& help_pair : sqlite_function_help) {
-        execute_example(*help_pair.second);
-    }
-    for (const auto& help_pair : lnav::sql::prql_functions) {
-        if (help_pair.second->ht_context != help_context_t::HC_PRQL_FUNCTION) {
-            continue;
-        }
-        execute_example(*help_pair.second);
-    }
-    for (const auto& cmd_pair : *sql_cmd_map) {
-        if (cmd_pair.second->c_help.ht_context
-                != help_context_t::HC_PRQL_TRANSFORM
-            && cmd_pair.second->c_help.ht_context
-                != help_context_t::HC_PRQL_FUNCTION)
-        {
-            continue;
-        }
-        execute_example(cmd_pair.second->c_help);
-    }
-    dls.dls_max_column_width = old_width;
-
-    dls.clear();
-}
-
-attr_line_t
+const attr_line_t&
 eval_example(const help_text& ht, const help_example& ex)
 {
-    auto iter = EXAMPLE_RESULTS.find(ex.he_cmd);
+    static const auto EMPTY = attr_line_t();
+    auto res_map = EXAMPLE_RESULTS.writeAccess();
 
-    if (iter != EXAMPLE_RESULTS.end()) {
-        return iter->second;
+    for ([[maybe_unused]] auto _attempt : {0, 1}) {
+        const auto iter = res_map->find(ex.he_cmd);
+        if (iter != res_map->end()) {
+            return iter->second;
+        }
+
+        switch (ht.ht_context) {
+            case help_context_t::HC_NONE:
+            case help_context_t::HC_PARAMETER:
+            case help_context_t::HC_RESULT:
+                break;
+            default: {
+                execute_example(*res_map, ht);
+                break;
+            }
+        }
     }
 
-    return "";
+    return EMPTY;
 }
 
 bool
@@ -1321,8 +1343,7 @@ get_textview_for_mode(ln_mode_t mode)
     }
 }
 
-hist_index_delegate::
-hist_index_delegate(hist_source2& hs, textview_curses& tc)
+hist_index_delegate::hist_index_delegate(hist_source2& hs, textview_curses& tc)
     : hid_source(hs), hid_view(tc)
 {
 }
@@ -1338,7 +1359,10 @@ hist_index_delegate::index_line(logfile_sub_source& lss,
                                 logfile* lf,
                                 logfile::iterator ll)
 {
-    if (ll->is_continued() || ll->get_time() == 0) {
+    if (ll->is_continued()
+        || ll->get_time<std::chrono::microseconds>()
+            == std::chrono::microseconds::zero())
+    {
         return;
     }
 
@@ -1358,9 +1382,10 @@ hist_index_delegate::index_line(logfile_sub_source& lss,
             break;
     }
 
-    this->hid_source.add_value(ll->get_time(), ht);
+    this->hid_source.add_value(ll->get_time<std::chrono::microseconds>(), ht);
     if (ll->is_marked() || ll->is_expr_marked()) {
-        this->hid_source.add_value(ll->get_time(), hist_source2::HT_MARK);
+        this->hid_source.add_value(ll->get_time<std::chrono::microseconds>(),
+                                   hist_source2::HT_MARK);
     }
 }
 
@@ -1391,7 +1416,7 @@ view_title_poss()
                 units = "file";
                 break;
             case LNV_DB:
-                quantity = lnav_data.ld_db_row_source.dls_rows.size();
+                quantity = lnav_data.ld_db_row_source.dls_row_cursors.size();
                 units = "row";
                 break;
         }
@@ -1492,6 +1517,10 @@ set_view_mode(ln_mode_t mode)
             break;
     }
     switch (mode) {
+        case ln_mode_t::SEARCH: {
+            lnav_data.ld_status[LNS_DOC].set_needs_update();
+            break;
+        }
         case ln_mode_t::BREADCRUMBS: {
             breadcrumb_view->focus();
             break;
@@ -1538,12 +1567,12 @@ all_views()
 }
 
 void
-lnav_behavior::mouse_event(int button, bool release, int x, int y)
+lnav_behavior::mouse_event(
+    notcurses* nc, int button, bool release, int x, int y)
 {
     static auto* breadcrumb_view = injector::get<breadcrumb_curses*>();
     static const auto VIEWS = all_views();
-    static const auto CLICK_INTERVAL
-        = std::chrono::milliseconds(mouseinterval(-1) * 2);
+    static const auto CLICK_INTERVAL = 333ms;
 
     struct mouse_event me;
 
@@ -1582,15 +1611,20 @@ lnav_behavior::mouse_event(int button, bool release, int x, int y)
         me.me_state = mouse_button_state_t::BUTTON_STATE_PRESSED;
     }
 
-    auto width = getmaxx(lnav_data.ld_window);
+    auto width = ncplane_dim_x(lnav_data.ld_window);
 
-    me.me_press_x = this->lb_last_event.me_press_x;
-    me.me_press_y = this->lb_last_event.me_press_y;
-    me.me_x = x - 1;
+    me.me_x = x;
     if (me.me_x >= width) {
         me.me_x = width - 1;
     }
     me.me_y = y - 1;
+    if (me.me_state == mouse_button_state_t::BUTTON_STATE_PRESSED) {
+        me.me_press_x = me.me_x;
+        me.me_press_y = me.me_y;
+    } else {
+        me.me_press_x = this->lb_last_event.me_press_x;
+        me.me_press_y = this->lb_last_event.me_press_y;
+    }
 
     switch (me.me_state) {
         case mouse_button_state_t::BUTTON_STATE_PRESSED:
