@@ -190,6 +190,7 @@ yajl_do_parse(yajl_handle hand, const unsigned char * jsonText,
     const unsigned char * buf;
     size_t bufLen;
     size_t * offset = &(hand->bytesConsumed);
+    yajl_string_props_t str_props;
 
     *offset = 0;
 
@@ -203,7 +204,7 @@ yajl_do_parse(yajl_handle hand, const unsigned char * jsonText,
             if (!(hand->flags & yajl_allow_trailing_garbage)) {
                 if (*offset != jsonTextLen) {
                     tok = yajl_lex_lex(hand->lexer, jsonText, jsonTextLen,
-                                       offset, &buf, &bufLen);
+                                       offset, &buf, &bufLen, &str_props);
                     if (tok != yajl_tok_eof) {
                         yajl_bs_set(hand->stateStack, yajl_state_parse_error);
                         hand->parseError = "trailing garbage";
@@ -230,7 +231,7 @@ yajl_do_parse(yajl_handle hand, const unsigned char * jsonText,
             yajl_state stateToPush = yajl_state_start;
 
             tok = yajl_lex_lex(hand->lexer, jsonText, jsonTextLen,
-                               offset, &buf, &bufLen);
+                               offset, &buf, &bufLen, &str_props);
 
             switch (tok) {
                 case yajl_tok_eof:
@@ -241,16 +242,18 @@ yajl_do_parse(yajl_handle hand, const unsigned char * jsonText,
                 case yajl_tok_string:
                     if (hand->callbacks && hand->callbacks->yajl_string) {
                         _CC_CHK(hand->callbacks->yajl_string(hand->ctx,
-                                                             buf, bufLen));
+                                                             buf, bufLen,
+                                                             &str_props));
                     }
                     break;
                 case yajl_tok_string_with_escapes:
                     if (hand->callbacks && hand->callbacks->yajl_string) {
                         yajl_buf_clear(hand->decodeBuf);
-                        yajl_string_decode(hand->decodeBuf, buf, bufLen);
+                        yajl_string_decode(hand->decodeBuf, buf, bufLen, &str_props);
                         _CC_CHK(hand->callbacks->yajl_string(
                                     hand->ctx, yajl_buf_data(hand->decodeBuf),
-                                    yajl_buf_len(hand->decodeBuf)));
+                                    yajl_buf_len(hand->decodeBuf),
+                                    &str_props));
                     }
                     break;
                 case yajl_tok_bool:
@@ -379,7 +382,7 @@ yajl_do_parse(yajl_handle hand, const unsigned char * jsonText,
              * start '}' is valid, whereas in need_key, we've parsed
              * a comma, and a string key _must_ follow */
             tok = yajl_lex_lex(hand->lexer, jsonText, jsonTextLen,
-                               offset, &buf, &bufLen);
+                               offset, &buf, &bufLen, &str_props);
             switch (tok) {
                 case yajl_tok_eof:
                     return yajl_status_ok;
@@ -389,7 +392,7 @@ yajl_do_parse(yajl_handle hand, const unsigned char * jsonText,
                 case yajl_tok_string_with_escapes:
                     if (hand->callbacks && hand->callbacks->yajl_map_key) {
                         yajl_buf_clear(hand->decodeBuf);
-                        yajl_string_decode(hand->decodeBuf, buf, bufLen);
+                        yajl_string_decode(hand->decodeBuf, buf, bufLen, &str_props);
                         buf = yajl_buf_data(hand->decodeBuf);
                         bufLen = yajl_buf_len(hand->decodeBuf);
                     }
@@ -420,7 +423,7 @@ yajl_do_parse(yajl_handle hand, const unsigned char * jsonText,
         }
         case yajl_state_map_sep: {
             tok = yajl_lex_lex(hand->lexer, jsonText, jsonTextLen,
-                               offset, &buf, &bufLen);
+                               offset, &buf, &bufLen, &str_props);
             switch (tok) {
                 case yajl_tok_colon:
                     yajl_bs_set(hand->stateStack, yajl_state_map_need_val);
@@ -439,7 +442,7 @@ yajl_do_parse(yajl_handle hand, const unsigned char * jsonText,
         }
         case yajl_state_map_got_val: {
             tok = yajl_lex_lex(hand->lexer, jsonText, jsonTextLen,
-                               offset, &buf, &bufLen);
+                               offset, &buf, &bufLen, &str_props);
             switch (tok) {
                 case yajl_tok_right_bracket:
                     if (hand->callbacks && hand->callbacks->yajl_end_map) {
@@ -467,7 +470,7 @@ yajl_do_parse(yajl_handle hand, const unsigned char * jsonText,
         }
         case yajl_state_array_got_val: {
             tok = yajl_lex_lex(hand->lexer, jsonText, jsonTextLen,
-                               offset, &buf, &bufLen);
+                               offset, &buf, &bufLen, &str_props);
             switch (tok) {
                 case yajl_tok_right_brace:
                     if (hand->callbacks && hand->callbacks->yajl_end_array) {
