@@ -110,7 +110,7 @@ handle_number(void* ctx, const char* numberVal, size_t numberLen)
 }
 
 static int
-handle_string(void* ctx, const unsigned char* stringVal, size_t len)
+handle_string(void* ctx, const unsigned char* stringVal, size_t len, yajl_string_props_t*)
 {
     json_op* jo = (json_op*) ctx;
     yajl_gen gen = (yajl_gen) jo->jo_ptr_data;
