@@ -37,6 +37,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "base/injector.bind.hh"
 #include "base/injector.hh"
 #include "config.h"
 #include "data_parser.hh"
@@ -51,6 +52,9 @@
 
 const char* TMP_NAME = "scanned.tmp";
 
+static auto bound_file_options_hier
+    = injector::bind<lnav::safe_file_options_hier>::to_singleton();
+
 int
 main(int argc, char* argv[])
 {
@@ -70,7 +74,7 @@ main(int argc, char* argv[])
     }
 
     {
-        std::vector<ghc::filesystem::path> paths;
+        std::vector<std::filesystem::path> paths;
         std::vector<lnav::console::user_message> errors;
 
         load_formats(paths, errors);
@@ -190,11 +194,12 @@ main(int argc, char* argv[])
                 string_attrs_t sa;
 
                 if (format.get() != nullptr) {
-                    format->annotate(0, sa, ll_values);
+                    format->annotate(nullptr, 0, sa, ll_values, false);
                     body = find_string_attr_range(sa, &SA_BODY);
                 }
 
                 data_parser::TRACE_FILE = fopen("scanned.dpt", "w");
+                setvbuf(data_parser::TRACE_FILE, nullptr, _IONBF, 0);
 
                 data_scanner ds(sub_line, body.lr_start);
 
