@@ -918,3 +918,37 @@ run_cap_test ${lnav_test} -Nn \
 run_cap_test ${lnav_test} -n \
      -c ";.msgformats" \
      ${test_dir}/logfile_for_join.0
+
+run_cap_test ${lnav_test} -Nn \
+    -c ";SELECT log_line, sc_bytes, json_object('columns', json_object('sc_bytes', json_object('color', '#f00'))) AS __lnav_style__ FROM access_log"  \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -Nn \
+    -c ";SELECT log_line, sc_bytes, json_object('columns', json_object('sc_ytes', json_object('color', '#f00'))) AS __lnav_style__ FROM access_log"  \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT bro_response_body_len AS bro_response_body_len1, humanize_file_size(bro_response_body_len) AS bro_response_body_len2 FROM bro_http_log ORDER BY bro_response_body_len DESC" \
+    ${test_dir}/logfile_bro_http.log.0
+
+run_cap_test ${lnav_test} -nN \
+    -c ";SELECT '1us' AS Duration UNION ALL SELECT '500ms' UNION ALL SELECT '1s'"
+
+run_cap_test env TEST_COMMENT="alignment demo" ${lnav_test} -nN -f- <<'EOF'
+;SELECT 'left' AS "Alignment Demo", '{"columns": {"Alignment Demo": {"text-align": "start"}}}' AS __lnav_style__
+ UNION ALL
+ SELECT 'center', '{"columns": {"Alignment Demo": {"text-align": "center"}}}' AS __lnav_style__
+ UNION ALL
+ SELECT 'right', '{"columns": {"Alignment Demo": {"text-align": "end"}}}' AS __lnav_style__
+:switch-to-view db
+EOF
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT * FROM access_log" \
+    -c ":hide-fields log_time" \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT * FROM access_log" \
+    -c ":hide-fields bad" \
+    ${test_dir}/logfile_access_log.0
