@@ -1,5 +1,7 @@
 #! /bin/bash
 
+run_cap_test ./drive_sql_anno ".dump /foo/bar abc"
+
 # basic query
 run_cap_test ./drive_sql_anno "SELECT * FROM FOO"
 
@@ -48,3 +50,11 @@ run_cap_test ./drive_sql_anno \
    "SELECT * from vmw_log, regexp_capture(log_body, '--> /SessionStats/SessionPool/Session/(?<line>[abc]+)')"
 
 run_cap_test ./drive_sql_anno "SELECT * FROM foo.bar"
+
+run_cap_test ./drive_sql_anno "SELECT json_object('abc', 'def') ->> '$.abc'"
+
+run_cap_test ./drive_sql_anno "SELECT 0x77, 123, 123e4"
+
+run_cap_test ./drive_sql_anno "from access_log | filter cs_method == 'GET' || cs_method == 'PUT'" 2
+
+run_cap_test ./drive_sql_anno "from access_log | stats.count_by { c_ip }" 23
